@@ -40,11 +40,26 @@
 
 
 #include <stdio.h>
-#include <string.h>
-#include <memory.h>
-#include <stdlib.h>
 #include <sys/time.h>
 
+#if STDC_HEADERS
+#  include <stdlib.h>
+#else
+#  if HAVE_MALLOC_H
+#    include <malloc.h>
+#  endif
+#endif
+
+#if HAVE_STRING_H
+#include <string.h>
+#else
+#include <strings.h>
+#endif
+
+#if HAVE_MEMORY_H
+#include <memory.h>
+#endif
+
 #include "Builtin.h"
 #include "Identifier.h"
 #include "error.h"
