@@ -39,7 +39,12 @@
 
 
 #include <stdio.h>
+
+#if HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
+#endif
 
 /*#include "error.h"*/
 #include "FunctionCall.h"
@@ -84,6 +89,8 @@ extern int numlig;
 
    --------------------------------- */
 
+extern FCT(void, ClearSelectionIfNeeded,(char *));
+
 static char * check_parser_error()
 {
   if (*err_msg) {
@@ -110,6 +117,8 @@ char * ie_load_file(buff, filename)
   char * xcoral_lib = 0;
   char * smaclib = 0;
   
+  ClearSelectionIfNeeded(buff);
+  
   i_buffer_courant = buff;
 
   if ((yyin = fopen(filename, "r")) == 0) {
@@ -176,6 +185,8 @@ char * ie_eval_region(buff)
   extern FCT(void, ie_goto_char,(char *, int));
   extern FCT(char *, ie_filename,(char *));
   
+  ClearSelectionIfNeeded(buff);
+  
   i_buffer_courant = buff;
   yyin = 0;
   yyinflag = 0;
@@ -217,6 +228,8 @@ char * ie_eval_expression(buff, expr)
 
   if (! lexpr) return ("cannot allocate memory\n");
 
+  ClearSelectionIfNeeded(buff);
+  
   lexpr[0] = '{';
   strcpy(lexpr + 1, expr);
   lexpr[lg + 1] = '}';
@@ -254,6 +267,8 @@ char * ie_call_function(buff, fctname, narg, args)
      int narg;
      int * args;
 {
+  ClearSelectionIfNeeded(buff);
+  
   i_buffer_courant = buff;
   i_minibuffer = 0;
   yyin = 0;
@@ -835,31 +850,29 @@ Object builtin_cmd_shell(a)			/* system */
   return result;
 }
 
+static Object strdup_end(s)
+    char * s;
+{
+  Object result = (Object) ((s) ? RTCMalloc(strlen(s) + 1) : 0);
+  
+  if (result)
+    strcpy((char *) result, s);
+  
+  accept_ctrl_c();
+  return result;
+}
+
 Object builtin_cmd_shell_to_string(a)			/* system */
      Instruction ** a;
 {
   extern FCT(char *, ie_shell_to_string,(char *, char *));
   char * expr = (char *) Eval(*a);
-  Object result;
 
 #ifdef RUNTIMECHECK
   check_strlen(expr);
 #endif
   ignore_ctrl_c();
-  expr = ie_shell_to_string(i_buffer_courant, expr);
-  if (expr) {
-    result = (Object) RTCMalloc(strlen(expr) + 1);
-    if (result) {
-      char * dest = (char *) result;
-      
-      while ((*dest++ = *expr++) != 0);
-    }
-  }
-  else
-    result = 0;
-  accept_ctrl_c();
-
-  return result;
+  return strdup_end(ie_shell_to_string(i_buffer_courant, expr));
 }
 
 Object builtin_msearch(a)			/* msearch(str, fin, sens)   */
@@ -1147,48 +1160,18 @@ Object builtin_filename(a)
     Instruction ** a;
 {
   extern FCT(char *, ie_filename,(char *));
-  char * filename;
-  char * result;
   
   ignore_ctrl_c();
-  filename = ie_filename(i_buffer_courant);
-  if (filename) {
-    result = (char *) RTCMalloc(strlen(filename) + 1);
-    if (result) {
-      char * dest = result;
-      
-      while ((*dest++ = *filename++) != 0);
-    }
-  }
-  else
-    result = 0;
-  accept_ctrl_c();
-
-  return (Object) result;
+  return strdup_end(ie_filename(i_buffer_courant));
 }
 
 Object builtin_current_mode(a)
     Instruction ** a;
 {
   extern FCT(char *, ie_current_mode,(char *));
-  char * mode;
-  char * result;
   
   ignore_ctrl_c();
-  mode = ie_current_mode(i_buffer_courant);
-  if (mode) {
-    result = (char *) RTCMalloc(strlen(mode) + 1);
-    if (result) {
-      char * dest = result;
-      
-      while ((*dest++ = *mode++) != 0);
-    }
-  }
-  else
-    result = 0;
-  accept_ctrl_c();
-
-  return (Object) result;
+  return strdup_end(ie_current_mode(i_buffer_courant));
 }
 
 Object builtin_window_height(a)
@@ -1222,24 +1205,9 @@ Object builtin_file_select(a)
     Instruction ** a;
 {
   extern FCT(char *, ie_file_select,(char *));
-  char * filename;
-  char * result;
   
   ignore_ctrl_c();
-  filename = ie_file_select(i_buffer_courant);
-  if (filename) {
-    result = (char *) RTCMalloc(strlen(filename) + 1);
-    if (result) {
-      char * dest = result;
-
-      while ((*dest++ = *filename++) != 0);
-    }
-  }
-  else
-    result = 0;
-  accept_ctrl_c();
-
-  return (Object) result;
+  return strdup_end(ie_file_select(i_buffer_courant));
 }
 
 Object builtin_current_window(a)
@@ -1448,6 +1416,8 @@ static int n_item;
 Object builtin_select_from_list(a)
     Instruction ** a;
 {
+  extern FCT(void, ClearListBox ,());
+  extern FCT(char *, FillList ,(char *));
   extern FCT(char *, SelectFromListBox ,(char *));
   char * str = (char *) Eval(*a);
 
@@ -1457,6 +1427,14 @@ Object builtin_select_from_list(a)
   
   if (! n_item) return 0;
   
+  ClearListBox();
+  {
+    int i;
+    
+    for (i = 0; i != n_item; i += 1)
+      FillList(item_list[i]);
+  }
+  
   if (! (str = SelectFromListBox(str)))
     return 0;
   
@@ -1510,7 +1488,7 @@ Object builtin_add_list_item(a)
 Object builtin_clear_list(a)
     Instruction ** a;
 {
-  extern FCT(char *, ClearListBox ,());
+  extern FCT(void, ClearListBox ,());
   
   ClearListBox();
   if (item_list) {
@@ -1548,9 +1526,8 @@ Object builtin_getchar(a)
 Object builtin_gets(a)
     Instruction ** a;
 {
-    char * prompt;
-    extern FCT(char *, GetStringFromDB, (char * prompt, int one));
-    char * str;
+  char * prompt;
+  extern FCT(char *, GetStringFromDB, (char * prompt, int one));
     
   if (IsaNotEnoughArg(*a))
     prompt = "gets ? ";
@@ -1562,20 +1539,7 @@ Object builtin_gets(a)
 #endif
   }
   
-  str = GetStringFromDB(prompt, 0);
-  if (! str)
-    return (Object) 0;
-  {
-    char * result = (char *) RTCMalloc(strlen(str) + 1);
-    
-    if (result) {
-      char * dest = result;
-
-      while ((*dest++ = *str++) != 0);
-    }
-    
-    return (Object) result;
-  }
+  return strdup_end(GetStringFromDB(prompt, 0));
 }
 
 
@@ -1845,6 +1809,1046 @@ Object builtin_usleep(a)
   return 0;
 }
 
+#ifndef LIONEL
+#include "../result_types.h"
+#include "../file_dict.h"
+#include "../class_dict.h"
+#include "../proc_dict.h"
+#include "../global_dict.h"
+
+/* le browser */
+
+Object builtin_browser_set_pp(a)
+    Instruction ** a;
+{
+  extern FCT(void, browser_set_pp,(char *, char *));
+  char * mode = (char *) Eval(a[0]);
+  char * path = (char *) Eval(a[1]);
+#ifdef RUNTIMECHECK
+  check_strlen(mode);
+  check_strlen(path);
+#endif
+
+  browser_set_pp(mode, path);
+  return 0;
+}
+
+Object builtin_browser_set_pp_options(a)
+    Instruction ** a;
+{
+  extern FCT(void, browser_set_pp_options,(char *, char *));
+  char * mode = (char *) Eval(a[0]);
+  char * opt = (char *) Eval(a[1]);
+#ifdef RUNTIMECHECK
+  check_strlen(mode);
+  check_strlen(opt);
+#endif
+
+  browser_set_pp_options(mode, opt);
+  return 0;
+}
+
+Object builtin_browser_add(a)
+    Instruction ** a;
+{
+  extern FCT(void, browser_add,(char *, int));
+  char * file = (char *) Eval(a[0]);
+#ifdef RUNTIMECHECK
+  check_strlen(file);
+#endif
+  ignore_ctrl_c();
+  browser_add(file, (IsaNotEnoughArg(a[1])) ? 0 : (int) Eval(a[1]));
+  accept_ctrl_c();
+  return 0;
+}
+
+Object builtin_browser_del(a)
+    Instruction ** a;
+{
+  extern FCT(void, RemoveFiles,(char *));
+  char * file = (char *) ((IsaNotEnoughArg(a[1])) ? 0 : (char *) Eval(a[0]));
+#ifdef RUNTIMECHECK
+  if (file)
+    check_strlen(file);
+#endif
+
+  ignore_ctrl_c();
+  RemoveFiles(file);
+  accept_ctrl_c();
+  return 0;
+}
+
+Object builtin_browser_class_file(a)
+    Instruction ** a;
+{
+  extern FCT(char *, browser_class_file,(char *, int *));
+  char * name = (char *) Eval(a[0]);
+  int * p = (int *) ((IsaNotEnoughArg(a[1])) ? 0 : (int *) Eval(a[1]));
+  char * file;
+  int line;
+  extern FCT(char *, browser_class_file,(char *, int *));
+#ifdef RUNTIMECHECK
+  extern FCT(Object, check_int_indirection_assignment,(int *, Object));
+  
+  check_strlen(name);
+#endif
+  
+  ignore_ctrl_c();
+  file = browser_class_file(name, &line);
+ 
+  if (file) {
+    if (p) {
+#ifdef RUNTIMECHECK
+      check_int_indirection_assignment(p, line - 1);
+#else
+      *p = line - 1;
+#endif
+    }
+    return strdup_end(file);
+  }
+  
+  accept_ctrl_c();
+  return 0;
+}
+
+Object builtin_browser_class_parents(a)
+    Instruction ** a;
+{
+  FCT(extern ParentRec **, get_parents_list,(char *, int));
+  char * class_name = (char *) Eval(a[0]);
+  ParentRec ** list;
+  char ** result = 0;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(class_name);
+#endif
+
+  ignore_ctrl_c();
+  if ((list = get_parents_list(class_name, 1)) != 0) {
+    unsigned nentry;
+    unsigned names_size = 0;
+    
+    for (nentry = 0; list[nentry]; nentry += 1)
+      names_size += strlen(list[nentry]->_name) + 1;
+  
+    result = (char **) RTCMalloc(sizeof(char*) * (nentry * 2 + 1) + names_size);
+    
+    if (result) {
+      char * p = (char *) &result[2 * nentry + 1];
+      ParentRec ** l = list;
+      
+      for (nentry = 0; *l; nentry += 2, l += 1) {
+	static scope_to_bits[] = {
+	  0,	/* UNKNOWN_SCOPE */
+	  4,	/* PUBLIC_SCOPE */
+	  2,	/* PROTECTED_SCOPE */
+	  1	/* PRIVATE_SCOPE */
+	};
+	result[nentry] = p;
+	strcpy(p, (*l)->_name);
+	p += strlen((*l)->_name) + 1;
+	*((int *) &result[nentry + 1]) = scope_to_bits[(*l)->_scope];
+      }
+      result[nentry] = 0;
+    }
+  }
+  
+  free(list);
+  accept_ctrl_c();
+  return (Object) result;
+}
+
+Object builtin_browser_class_children(a)
+    Instruction ** a;
+{
+  extern FCT(char **, get_sons_list,(char *));
+  char * class_name = (char *) Eval(a[0]);
+  char ** list;
+  char ** result = 0;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(class_name);
+#endif
+  
+  ignore_ctrl_c();
+  if ((list = get_sons_list(class_name)) != 0) {
+    unsigned nentry;
+    unsigned names_size = 0;
+    
+    for (nentry = 0; list[nentry]; nentry += 1)
+      names_size += strlen(list[nentry]) + 1;
+  
+    result = (char **) RTCMalloc(sizeof(char*) * (nentry + 1) + names_size);
+    
+    if (result) {
+      char * p = (char *) &result[nentry + 1];
+      
+      for (nentry = 0; list[nentry]; nentry += 1) {
+	result[nentry] = p;
+	strcpy(p, list[nentry]);
+	p += strlen(list[nentry]) + 1;
+      }
+      result[nentry] = 0;
+    }
+  }
+  
+  free(list);
+  accept_ctrl_c();
+  return (Object) result;
+}
+
+Object builtin_browser_class_methods(a)
+    Instruction ** a;
+{
+  FCT(extern StringTable*, get_methods_list,(char *, int));
+  extern MethodRec* marked_list;
+  extern int marked_count;
+  char * result = (char *) Eval(a[0]);
+  
+#ifdef RUNTIMECHECK
+  check_strlen(result);
+#endif
+  
+  ignore_ctrl_c();
+  {
+    StringTable* x = get_methods_list(result, 0);
+    
+    if (x) {
+      char * declfile = marked_list->_decl_file->_name;
+      char * lastfile;
+      unsigned strsize = strlen(declfile) + 1;
+      MethodRec* list;
+      unsigned i;
+
+      free(x);
+      
+      lastfile = declfile;
+      i = marked_count;
+      list = marked_list;
+      while (i--) {
+	char * ifile;
+	
+	strsize += strlen(list->_name) + METHOD_PLENGTH + 1;
+	if (list->_impl_file &&
+	    strcmp(ifile = list->_impl_file->_name, declfile) &&
+	    strcmp(ifile, lastfile)) {
+	  strsize += strlen(ifile) + 1;
+	  lastfile = ifile;
+	}
+	list = list->_next_marked;
+      }
+      if ((result = RTCMalloc((marked_count * 5 + 2) * sizeof(char *) + 
+			      strsize))
+	  != 0) {
+	char * pdeclfile = (char *) &((char **) result)[marked_count * 5 + 2];
+	char * plastfile;
+	char * p;
+	unsigned index = 1;
+	
+	lastfile = declfile;
+	i = marked_count;
+	list = marked_list;
+	strcpy(pdeclfile, declfile);
+	p = pdeclfile + strlen(declfile) + 1;
+	((char **) result)[0] = pdeclfile;
+	while (i--) {
+	  char * ifile;
+	
+	  ((char **) result)[index] = p;
+	  strcpy(p, list->_name - METHOD_PLENGTH);
+	  p += strlen(list->_name) + METHOD_PLENGTH + 1;
+	  *((int *) &((char **) result)[index + 1]) = list->_decl_line - 1;
+	  *((int *) &((char **) result)[index + 2]) = list->_decl;
+	  if (! list->_impl_file)
+	    ((char **) result)[index + 3] = 0;
+	  else {
+	    if (! strcmp(ifile = list->_impl_file->_name, declfile))
+	      ((char **) result)[index + 3] = pdeclfile;
+	    else if (! strcmp(ifile, lastfile))
+	      ((char **) result)[index + 3] = plastfile;
+	    else {
+	      ((char **) result)[index + 3] = p;
+	      strcpy(p, ifile);
+	      plastfile = p;
+	      p += strlen(ifile) + 1;
+	      lastfile = ifile;
+	    }
+	    *((int *) &((char **) result)[index + 4]) = list->_impl_line - 1;
+	  }
+	  index += 5;
+	  {
+	    MethodRec* next = list->_next_marked;
+	    
+	    list->_next_marked = 0;
+	    list = next;
+	  }
+	}
+	((char **) result)[index] = 0;
+      }
+    }
+    else
+      result = 0;
+  }
+  
+  accept_ctrl_c();
+  return (Object) result;
+}
+
+Object builtin_browser_class_attributes(a)
+    Instruction ** a;
+{
+  FCT(extern StringTable*, get_attributes_list,(char *, int));
+  extern AttributeRec* marked_attr_list;
+  extern int marked_attr_count;
+  char * result = (char *) Eval(a[0]);
+  
+#ifdef RUNTIMECHECK
+  check_strlen(result);
+#endif
+  
+  ignore_ctrl_c();
+  {
+    StringTable* x = get_attributes_list(result, 0);
+    
+    if (x) {
+      char * declfile = marked_attr_list->_decl_file->_name;
+      unsigned strsize = strlen(declfile) + 1;
+      AttributeRec* list;
+      unsigned i;
+
+      free(x);
+      
+      i = marked_attr_count;
+      list = marked_attr_list;
+      while (i--) {
+	strsize += strlen(list->_name) + ATTRIBUTE_PLENGTH + 1;
+	list = list->_next_marked;
+      }
+      
+      if ((result = RTCMalloc((marked_attr_count * 3 + 2) * sizeof(char *) + 
+			      strsize))
+	  != 0) {
+	char * p = (char *) &((char **) result)[marked_attr_count * 3 + 2];
+	unsigned index = 1;
+	
+	i = marked_attr_count;
+	list = marked_attr_list;
+	strcpy(p, declfile);
+	((char **) result)[0] = p;
+	p += strlen(declfile) + 1;
+	while (i--) {
+	  ((char **) result)[index] = p;
+	  strcpy(p, list->_name - ATTRIBUTE_PLENGTH);
+	  p += strlen(list->_name) + ATTRIBUTE_PLENGTH + 1;
+	  *((int *) &((char **) result)[index + 1]) = list->_decl_line - 1;
+	  *((int *) &((char **) result)[index + 2]) = list->_decl;
+	  index += 3;
+	  {
+	    AttributeRec* next = list->_next_marked;
+	    
+	    list->_next_marked = 0;
+	    list = next;
+	  }
+	}
+	((char **) result)[index] = 0;
+      }
+    }
+    else
+      result = 0;
+  }
+  
+  accept_ctrl_c();
+  return (Object) result;
+}
+
+Object builtin_browser_class_flags(a)
+    Instruction ** a;
+{
+  extern FCT(char *, browser_class_flags,(char *));
+  char * class_name = (char *) Eval(a[0]);
+  
+#ifdef RUNTIMECHECK
+  check_strlen(class_name);
+#endif
+
+  ignore_ctrl_c();
+  return strdup_end(browser_class_flags(class_name));
+}
+
+Object builtin_browser_functions(a)
+    Instruction ** a;
+{
+  extern FCT(ProcRec **, get_procs_list,(char *));
+  char * prefix = (char *) Eval(a[0]);
+  ProcRec ** list;
+  char ** result;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(prefix);
+#endif
+
+  ignore_ctrl_c();
+  if ((list = get_procs_list(prefix)) != 0) {
+    ProcRec ** l;
+    unsigned n;
+    unsigned strsize = 0;
+    
+    for (n = 0, l = list; *l; l += 1, n += 1) {
+      strsize += strlen((*l)->_name) + PROC_PLENGTH + 1;
+      if ((*l)->_impl_file) {
+	ProcRec ** sl;
+	
+	for (sl = list; sl != l; sl += 1)
+	  if ((*sl)->_impl_file &&
+	      (! strcmp((*l)->_impl_file->_name, (*sl)->_impl_file->_name)))
+	    break;
+	if (sl == l)
+	  strsize += strlen((*l)->_impl_file->_name) + 1;
+      }
+    }
+    if ((result = (char **) RTCMalloc((n * 3 + 1) * sizeof(char *) + strsize))
+	!= 0) {
+      char * p = (char *) &result[n * 3 + 1];
+      
+      for (n = 0, l = list; *l; l += 1, n += 3) {
+	result[n] = p;
+	strcpy(p, (*l)->_name - PROC_PLENGTH);
+	p += strlen((*l)->_name) + PROC_PLENGTH + 1;
+	if ((*l)->_impl_file) {
+	  unsigned i;
+	
+	  for (i = 0; i != n; i += 3)
+	    if (result[i + 1] &&
+		(! strcmp(result[i + 1], (*l)->_impl_file->_name)))
+	      break;
+	  if (i == n) {
+	    result[n + 1] = p;
+	    strcpy(p, (*l)->_impl_file->_name);
+	    p += strlen((*l)->_impl_file->_name) + 1;
+	  }
+	  else
+	    result[n + 1] = result[i + 1];
+	  *((int *) &result[n + 2]) = (*l)->_impl_line;
+	}
+	else
+	  result[n + 1] = 0;
+      }
+      result[n] = 0;
+    }
+  }
+  else
+    result = 0;
+  
+  accept_ctrl_c();
+  return (Object) result;
+}
+
+Object builtin_browser_globals(a)
+    Instruction ** a;
+{
+  extern FCT(GlobalRec **, get_globals_list,(char *));
+  char * prefix = (char *) Eval(a[0]);
+  GlobalRec ** list;
+  char ** result;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(prefix);
+#endif
+
+  ignore_ctrl_c();
+  if ((list = get_globals_list(prefix)) != 0) {
+    GlobalRec ** l;
+    unsigned n;
+    unsigned strsize = 0;
+    
+    for (n = 0, l = list; *l; l += 1, n += 1) {
+      strsize += strlen((*l)->_name) + GLOBAL_PLENGTH + 1;
+      if ((*l)->_impl_file) {
+	GlobalRec ** sl;
+	
+	for (sl = list; sl != l; sl += 1)
+	  if ((*sl)->_impl_file &&
+	      (! strcmp((*l)->_impl_file->_name, (*sl)->_impl_file->_name)))
+	    break;
+	if (sl == l)
+	  strsize += strlen((*l)->_impl_file->_name) + 1;
+      }
+    }
+    if ((result = (char **) RTCMalloc((n * 3 + 1) * sizeof(char *) + strsize))
+	!= 0) {
+      char * p = (char *) &result[n * 3 + 1];
+      
+      for (n = 0, l = list; *l; l += 1, n += 3) {
+	result[n] = p;
+	strcpy(p, (*l)->_name - GLOBAL_PLENGTH);
+	p += strlen((*l)->_name) + GLOBAL_PLENGTH + 1;
+	if ((*l)->_impl_file) {
+	  unsigned i;
+	
+	  for (i = 0; i != n; i += 3)
+	    if (result[i + 1] &&
+		(! strcmp(result[i + 1], (*l)->_impl_file->_name)))
+	      break;
+	  if (i == n) {
+	    result[n + 1] = p;
+	    strcpy(p, (*l)->_impl_file->_name);
+	    p += strlen((*l)->_impl_file->_name) + 1;
+	  }
+	  else
+	    result[n + 1] = result[i + 1];
+	  *((int *) &result[n + 2]) = (*l)->_impl_line;
+	}
+	else
+	  result[n + 1] = 0;
+      }
+      result[n] = 0;
+    }
+  }
+  else
+    result = 0;
+  
+  accept_ctrl_c();
+  return (Object) result;
+}
+
+Object builtin_browser_select_class(a)
+    Instruction ** a;
+{
+  extern FCT(int, browser_select_class,(char *));
+  char * prefix = (char *) Eval(a[0]);
+  Object result;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(prefix);
+#endif
+
+  ignore_ctrl_c();
+  result = (Object) browser_select_class(prefix);
+  accept_ctrl_c();
+  return result;
+}
+
+Object builtin_browser_select_method(a)
+    Instruction ** a;
+{
+  extern FCT(int, browser_select_method,(char *));
+  char * prefix = (char *) Eval(a[0]);
+  Object result;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(prefix);
+#endif
+
+  ignore_ctrl_c();
+  result = (Object) browser_select_method(prefix);
+  accept_ctrl_c();
+  return result;
+}
+
+Object builtin_browser_select_attribute(a)
+    Instruction ** a;
+{
+  extern FCT(int, browser_select_attribute,(char *));
+  char * prefix = (char *) Eval(a[0]);
+  Object result;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(prefix);
+#endif
+
+  ignore_ctrl_c();
+  result = (Object) browser_select_attribute(prefix);
+  accept_ctrl_c();
+  return result;
+}
+
+Object builtin_browser_select_function(a)
+    Instruction ** a;
+{
+  extern FCT(int, browser_select_proc,(char *));
+  char * prefix = (char *) Eval(a[0]);
+  Object result;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(prefix);
+#endif
+
+  ignore_ctrl_c();
+  result = (Object) browser_select_proc(prefix);
+  accept_ctrl_c();
+  return result;
+}
+
+Object builtin_browser_select_global(a)
+    Instruction ** a;
+{
+  extern FCT(int, browser_select_global,(char *));
+  char * prefix = (char *) Eval(a[0]);
+  Object result;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(prefix);
+#endif
+
+  ignore_ctrl_c();
+  result = (Object) browser_select_global(prefix);
+  accept_ctrl_c();
+  return result;
+}
+
+Object builtin_browser_edit(a)
+    Instruction ** a;
+{
+  extern FCT(void, ButtonEdit,());
+  extern FCT(void, DisplayBrowserIfNeverMap,());
+  
+  ignore_ctrl_c();
+  DisplayBrowserIfNeverMap();
+  ButtonEdit();
+  accept_ctrl_c();
+  return 0;
+}
+
+Object builtin_browser_class_entry(a)
+    Instruction ** a;
+{
+  extern FCT(int, browser_class_entry,(char *));
+  char * prefix = (char *) Eval(a[0]);
+  Object result;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(prefix);
+#endif
+
+  ignore_ctrl_c();
+  result = (Object) browser_class_entry(prefix);
+  accept_ctrl_c();
+  return result;
+}
+
+Object builtin_browser_function_entry(a)
+    Instruction ** a;
+{
+  extern FCT(int, browser_function_entry,(char *));
+  char * prefix = (char *) Eval(a[0]);
+  Object result;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(prefix);
+#endif
+
+  ignore_ctrl_c();
+  result = (Object) browser_function_entry(prefix);
+  accept_ctrl_c();
+  return result;
+}
+
+Object builtin_browser_global_entry(a)
+    Instruction ** a;
+{
+  extern FCT(int, browser_global_entry,(char *));
+  char * prefix = (char *) Eval(a[0]);
+  Object result;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(prefix);
+#endif
+
+  ignore_ctrl_c();
+  result = (Object) browser_global_entry(prefix);
+  accept_ctrl_c();
+  return result;
+}
+
+Object builtin_browser_file_entry(a)
+    Instruction ** a;
+{
+  extern FCT(int, browser_file_entry,(char *));
+  char * prefix = (char *) Eval(a[0]);
+  Object result;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(prefix);
+#endif
+
+  ignore_ctrl_c();
+  result = (Object) browser_file_entry(prefix);
+  accept_ctrl_c();
+  return result;
+}
+
+Object builtin_browser_class(a)
+    Instruction ** a;
+{
+  extern FCT(char *, browser_class,(int));
+  int n = (int) Eval(a[0]);
+  
+  ignore_ctrl_c();
+  return strdup_end(browser_class(n));
+}
+
+Object builtin_browser_function(a)
+    Instruction ** a;
+{
+  extern FCT(char *, browser_function,(int));
+  int n = (int) Eval(a[0]);
+  
+  ignore_ctrl_c();
+  return strdup_end(browser_function(n));
+}
+
+Object builtin_browser_global(a)
+    Instruction ** a;
+{
+  extern FCT(char *, browser_global,(int));
+  int n = (int) Eval(a[0]);
+  
+  ignore_ctrl_c();
+  return strdup_end(browser_global(n));
+}
+
+Object builtin_browser_file(a)
+    Instruction ** a;
+{
+  extern FCT(char *, browser_file,(int));
+  int n = (int) Eval(a[0]);
+  
+  ignore_ctrl_c();
+  return strdup_end(browser_file(n));
+}
+
+Object builtin_browser_dump(a)
+    Instruction ** a;
+{
+  extern FCT(int, browser_dump,(char *));
+  char * fn = (char *) Eval(a[0]);
+  Object result;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(fn);
+#endif
+
+  ignore_ctrl_c();
+  result = (Object) browser_dump(fn);
+  accept_ctrl_c();
+  
+  return result;
+}
+
+Object builtin_browser_restore(a)
+    Instruction ** a;
+{
+  extern FCT(int, browser_restore,(char *));
+  char * fn = (char *) Eval(a[0]);
+  Object result;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(fn);
+#endif
+
+  ignore_ctrl_c();
+  result = (Object) browser_restore(fn);
+  accept_ctrl_c();
+  
+  return result;
+}
+
+Object builtin_browser_show_all(a)
+    Instruction ** a;
+{
+  extern FCT(void, browser_show_all,());
+  extern FCT(void, RefreshBrowserInfos,());
+  
+  ignore_ctrl_c();
+  browser_show_all();
+  RefreshBrowserInfos();
+  accept_ctrl_c();
+  
+  return 0;
+}
+
+static Object builtin_browser_hide_x(x)
+    unsigned x;
+{
+  extern FCT(void, RefreshBrowserInfos,());
+  
+  ignore_ctrl_c();
+  if (! (Hide_Bits & x)) {
+    Hide_Bits |= x;
+    RefreshBrowserInfos();
+  }
+  accept_ctrl_c();
+  
+  return 0;
+}
+
+static Object builtin_browser_show_x(x)
+    unsigned x;
+{
+  extern FCT(void, RefreshBrowserInfos,());
+  
+  ignore_ctrl_c();
+  if (Hide_Bits & x) {
+    Hide_Bits &= ~x;
+    RefreshBrowserInfos();
+  }
+  accept_ctrl_c();
+  
+  return 0;
+}
+
+Object builtin_browser_hide_private_members(a)
+    Instruction ** a;
+{
+  return builtin_browser_hide_x(HidePrivate);
+}
+
+Object builtin_browser_hide_protected_and_private_members(a)
+    Instruction ** a;
+{
+  extern FCT(void, RefreshBrowserInfos,());
+  
+  ignore_ctrl_c();
+  if ((Hide_Bits & (HidePrivate | HideProtected)) !=
+      (HidePrivate | HideProtected)) {
+    Hide_Bits |= HidePrivate | HideProtected;
+    RefreshBrowserInfos();
+  }
+  accept_ctrl_c();
+  
+  return 0;
+}
+
+Object builtin_browser_show_protected_and_private_members(a)
+    Instruction ** a;
+{
+  return builtin_browser_show_x(HidePrivate | HideProtected);
+}
+
+Object builtin_browser_hide_inherited_members(a)
+    Instruction ** a;
+{
+  return builtin_browser_hide_x(HideInherited);
+}
+
+Object builtin_browser_show_inherited_members(a)
+    Instruction ** a;
+{
+  return builtin_browser_show_x(HideInherited);
+}
+
+Object builtin_browser_hide_internal_types(a)
+    Instruction ** a;
+{
+  return builtin_browser_hide_x(HideInternalTypes);
+}
+
+Object builtin_browser_show_internal_types(a)
+    Instruction ** a;
+{
+  return builtin_browser_show_x(HideInternalTypes);
+}
+
+Object builtin_browser_hide_static_functions(a)
+    Instruction ** a;
+{
+  return builtin_browser_hide_x(HideStaticFunctions);
+}
+
+Object builtin_browser_show_static_functions(a)
+    Instruction ** a;
+{
+  return builtin_browser_show_x(HideStaticFunctions);
+}
+
+Object builtin_browser_hide_static_globals(a)
+    Instruction ** a;
+{
+  return builtin_browser_hide_x(HideStaticGlobals);
+}
+
+Object builtin_browser_show_static_globals(a)
+    Instruction ** a;
+{
+  return builtin_browser_show_x(HideStaticGlobals);
+}
+
+Object builtin_browser_hide_children_of(a)
+    Instruction ** a;
+{
+  extern FCT(void, RefreshBrowserInfos,());
+  extern FCT(ClassRec *, find_class,(char *));
+  char * name = (char *) Eval(a[0]);
+  ClassRec * obj;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(name);
+#endif
+  
+  ignore_ctrl_c();
+  if (((obj = find_class(name)) != 0) && (! (obj->_hide & HideChildrenOf))) {
+    obj->_hide |= HideChildrenOf;
+    RefreshBrowserInfos();
+  }
+  accept_ctrl_c();
+  
+  return 0;
+}
+
+Object builtin_browser_show_children_of(a)
+    Instruction ** a;
+{
+  extern FCT(void, RefreshBrowserInfos,());
+  extern FCT(ClassRec *, find_class,(char *));
+  char * name = (char *) Eval(a[0]);
+  ClassRec * obj;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(name);
+#endif
+  
+  ignore_ctrl_c();
+  if (((obj = find_class(name)) != 0) && (obj->_hide & HideChildrenOf)) {
+    obj->_hide &= ~HideChildrenOf;
+    RefreshBrowserInfos();
+  }
+  accept_ctrl_c();
+  
+  return 0;
+}
+
+Object builtin_browser_hide_class(a)
+    Instruction ** a;
+{
+  extern FCT(void, RefreshBrowserInfos,());
+  extern FCT(ClassRec *, find_class,(char *));
+  char * name = (char *) Eval(a[0]);
+  ClassRec * obj;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(name);
+#endif
+  
+  ignore_ctrl_c();
+  if (((obj = find_class(name)) != 0) && (! (obj->_hide & UserHide))) {
+    obj->_hide |= UserHide;
+    RefreshBrowserInfos();
+  }
+  accept_ctrl_c();
+  
+  return 0;
+}
+
+Object builtin_browser_show_class(a)
+    Instruction ** a;
+{
+  extern FCT(void, RefreshBrowserInfos,());
+  extern FCT(ClassRec *, find_class,(char *));
+  char * name = (char *) Eval(a[0]);
+  ClassRec * obj;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(name);
+#endif
+  
+  ignore_ctrl_c();
+  if (((obj = find_class(name)) != 0) && (obj->_hide & UserHide)) {
+    obj->_hide &= ~UserHide;
+    RefreshBrowserInfos();
+  }
+  accept_ctrl_c();
+  
+  return 0;
+}
+
+Object builtin_browser_show_global(a)
+    Instruction ** a;
+{
+  extern FCT(void, RefreshBrowserInfos,());
+  extern FCT(GlobalRec *, find_global,(char *));
+  char * name = (char *) Eval(a[0]);
+  GlobalRec * obj;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(name);
+#endif
+  
+  ignore_ctrl_c();
+  if (((obj = find_global(name)) != 0) && (obj->_hide & UserHide)) {
+    obj->_hide &= ~UserHide;
+    RefreshBrowserInfos();
+  }
+  accept_ctrl_c();
+  
+  return 0;
+}
+
+Object builtin_browser_hide_global(a)
+    Instruction ** a;
+{
+  extern FCT(void, RefreshBrowserInfos,());
+  extern FCT(GlobalRec *, find_global,(char *));
+  char * name = (char *) Eval(a[0]);
+  GlobalRec * obj;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(name);
+#endif
+  
+  ignore_ctrl_c();
+  if (((obj = find_global(name)) != 0) && (! (obj->_hide & UserHide))) {
+    obj->_hide |= UserHide;
+    RefreshBrowserInfos();
+  }
+  accept_ctrl_c();
+  
+  return 0;
+}
+
+Object builtin_browser_show_function(a)
+    Instruction ** a;
+{
+  extern FCT(void, RefreshBrowserInfos,());
+  extern FCT(ProcRec *, find_proc,(char *));
+  char * name = (char *) Eval(a[0]);
+  ProcRec * obj;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(name);
+#endif
+  
+  ignore_ctrl_c();
+  if (((obj = find_proc(name)) != 0) && (obj->_hide & UserHide)) {
+    obj->_hide &= ~UserHide;
+    RefreshBrowserInfos();
+  }
+  accept_ctrl_c();
+  
+  return 0;
+}
+
+Object builtin_browser_hide_function(a)
+    Instruction ** a;
+{
+  extern FCT(void, RefreshBrowserInfos,());
+  extern FCT(ProcRec *, find_proc,(char *));
+  char * name = (char *) Eval(a[0]);
+  ProcRec * obj;
+  
+#ifdef RUNTIMECHECK
+  check_strlen(name);
+#endif
+  
+  ignore_ctrl_c();
+  if (((obj = find_proc(name)) != 0) && (! (obj->_hide & UserHide))) {
+    obj->_hide |= UserHide;
+    RefreshBrowserInfos();
+  }
+  accept_ctrl_c();
+  
+  return 0;
+}
+#endif
 
 /* */
 
@@ -1996,6 +3000,119 @@ void Init_Smac_Xcoral()
   new_builtin("watch_off", Type_Void, 0, 0, builtin_watch_off);
   new_builtin("usleep", Type_Void, 1, &Type_Int, builtin_usleep);
   new_builtin("current_mode", Type_String, 0, 0, builtin_current_mode);
+  
+#ifndef LIONEL
+  {
+    static Type * pString;
+    static Type * pInt;
+    static Type * string_pint[2];
+    
+    pString = Type__Type(Type_String, T_Pointer, sizeof(char **));
+    pInt = Type__Type(Type_Int, T_Pointer, sizeof(int *));
+    string_pint[0] = Type_String;
+    string_pint[1] = pInt;
+    
+    new_builtin("browser_set_pp", Type_Void, 2, string_string_string,
+		builtin_browser_set_pp);
+    new_builtin("browser_set_pp_options", Type_Void, 2, string_string_string,
+		builtin_browser_set_pp_options);
+    HashTable__Add(builtin_fct_nbre_arg_min,
+		   new_builtin("browser_add", Type_Void, 2, 
+			       &string_string_int[1], builtin_browser_add),
+		   1 + 1);
+    HashTable__Add(builtin_fct_nbre_arg_min,
+		   new_builtin("browser_del", Type_Void, 1, 
+			       &Type_String, builtin_browser_del),
+		   0 + 1);
+    HashTable__Add(builtin_fct_nbre_arg_min,
+		   new_builtin("browser_class_file", Type_String, 2, 
+			       string_pint, builtin_browser_class_file),
+		   1 + 1);
+    new_builtin("browser_class_parents", pString, 1, &Type_String,
+		builtin_browser_class_parents);
+    new_builtin("browser_class_children", pString, 1, &Type_String,
+		builtin_browser_class_children);
+    new_builtin("browser_class_methods", pString, 1, &Type_String,
+		builtin_browser_class_methods);
+    new_builtin("browser_class_attributes", pString, 1, &Type_String,
+		builtin_browser_class_attributes);
+    new_builtin("browser_class_flags", Type_String, 1, &Type_String,
+		builtin_browser_class_flags); 
+    new_builtin("browser_functions", pString, 1, &Type_String,
+		builtin_browser_functions);
+    new_builtin("browser_globals", pString, 1, &Type_String,
+		builtin_browser_globals);
+    new_builtin("browser_select_class", Type_Int, 1, &Type_String,
+		builtin_browser_select_class); 
+    new_builtin("browser_select_method", Type_Int, 1, &Type_String,
+		builtin_browser_select_method); 
+    new_builtin("browser_select_attribute", Type_Int, 1, &Type_String,
+		builtin_browser_select_attribute); 
+    new_builtin("browser_select_function", Type_Int, 1, &Type_String,
+		builtin_browser_select_function); 
+    new_builtin("browser_select_global", Type_Int, 1, &Type_String,
+		builtin_browser_select_global); 
+    new_builtin("browser_edit", Type_Void, 0, 0, builtin_browser_edit); 
+    new_builtin("browser_class_entry", Type_Int, 1, &Type_String,
+		builtin_browser_class_entry); 
+    new_builtin("browser_function_entry", Type_Int, 1, &Type_String,
+		builtin_browser_function_entry); 
+    new_builtin("browser_global_entry", Type_Int, 1, &Type_String,
+		builtin_browser_global_entry); 
+    new_builtin("browser_file_entry", Type_Int, 1, &Type_String,
+		builtin_browser_file_entry); 
+    new_builtin("browser_class", Type_String, 1, &Type_Int,
+		builtin_browser_class); 
+    new_builtin("browser_function", Type_String, 1, &Type_Int,
+		builtin_browser_function); 
+    new_builtin("browser_global", Type_String, 1, &Type_Int,
+		builtin_browser_global); 
+    new_builtin("browser_file", Type_String, 1, &Type_Int,
+		builtin_browser_file); 
+    new_builtin("browser_dump", Type_Int, 1, &Type_String,
+		builtin_browser_dump); 
+    new_builtin("browser_restore", Type_Int, 1, &Type_String,
+		builtin_browser_restore); 
+    new_builtin("browser_show_all", Type_Void, 0, 0,
+		builtin_browser_show_all); 
+    new_builtin("browser_hide_private_members", Type_Void, 0, 0,
+		builtin_browser_hide_private_members); 
+    new_builtin("browser_hide_protected_and_private_members", Type_Void, 0, 0,
+		builtin_browser_hide_protected_and_private_members); 
+    new_builtin("browser_show_protected_and_private_members", Type_Void, 0, 0,
+		builtin_browser_show_protected_and_private_members); 
+    new_builtin("browser_hide_inherited_members", Type_Void, 0, 0,
+		builtin_browser_hide_inherited_members); 
+    new_builtin("browser_show_inherited_members", Type_Void, 0, 0,
+		builtin_browser_show_inherited_members); 
+    new_builtin("browser_hide_internal_types", Type_Void, 0, 0,
+		builtin_browser_hide_internal_types); 
+    new_builtin("browser_show_internal_types", Type_Void, 0, 0,
+		builtin_browser_show_internal_types); 
+    new_builtin("browser_hide_static_functions", Type_Void, 0, 0,
+		builtin_browser_hide_static_functions); 
+    new_builtin("browser_show_static_functions", Type_Void, 0, 0,
+		builtin_browser_show_static_functions); 
+    new_builtin("browser_hide_static_globals", Type_Void, 0, 0,
+		builtin_browser_hide_static_globals); 
+    new_builtin("browser_show_static_globals", Type_Void, 0, 0,
+		builtin_browser_show_static_globals); 
+    new_builtin("browser_hide_children_of", Type_Void, 1, &Type_String,
+		builtin_browser_hide_children_of); 
+    new_builtin("browser_show_children_of", Type_Void, 1, &Type_String,
+		builtin_browser_show_children_of); 
+    new_builtin("browser_hide_class", Type_Void, 1, &Type_String,
+		builtin_browser_hide_class); 
+    new_builtin("browser_show_class", Type_Void, 1, &Type_String,
+		builtin_browser_show_class); 
+    new_builtin("browser_hide_global", Type_Void, 1, &Type_String,
+		builtin_browser_hide_global); 
+    new_builtin("browser_show_global", Type_Void, 1, &Type_String,
+		builtin_browser_show_global); 
+    new_builtin("browser_hide_function", Type_Void, 1, &Type_String,
+		builtin_browser_hide_function); 
+    new_builtin("browser_show_function", Type_Void, 1, &Type_String,
+		builtin_browser_show_function); 
+}
+#endif
 }
-
-
