@@ -1,9 +1,3 @@
-#if ( hpux || _AIX || ultrix || __osf__)
-/* nothing */
-#else
-extern char *malloc(), *realloc();
-#endif
-
     /* This file is a transformation of James A. Roskind's C++ grammar */
 
 
