@@ -42,14 +42,24 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/cursorfont.h>
-#if defined (apollo) || defined (__FreeBSD__)
-#include <stdlib.h>
+
+#if STDC_HEADERS
+#  include <stdlib.h>
 #else
-#include <malloc.h>
+#  if HAVE_MALLOC_H
+#    include <malloc.h>
+#  endif
 #endif
+
+#if HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
+#endif
+
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <X11/Xatom.h>
 
 #include "options.h"
 #include "xcoral.h"
@@ -65,26 +75,37 @@
 #include "chars_cmds.h"
 #include "main_events.h"
 #include "new_window.h"
+#include "browser_eng.h"
+#include "input_str.h"
+#include "selection.h"
+#include "warn_box.h"
 
 extern Browser br;
 extern char *b_name[];
 int flag_visit = False;
 
 FCT (static void, AddFiles, () );
-FCT (static void, ButtonEdit, () );
+FCT (void, ButtonEdit, () );
 FCT (static void, ButtonInVisitWindow, (BWin *bwin, XButtonEvent *ev) );
-FCT (static void, ClassSelect, (BWin *bwin) );
+FCT (static int, ClassSelect, (BWin *bwin) );
 FCT (static void, EditFile, (BWin *bwin, char *name) );
-FCT (static void, FileSelect, (BWin *bwin) );
+FCT (static int, FileSelect, (BWin *bwin) );
 FCT (static void, ManageFiles, (int op) );
-FCT (static void, MethodSelect, (BWin *bwin) );
-FCT (static void, ParentChildSelect, (BWin *bwin) );
-FCT (static void, ProcSelect, (BWin *bwin) );
+FCT (static int, MethodSelect, (BWin *bwin) );
+FCT (static int, ParentChildSelect, (BWin *bwin) );
+FCT (static int, ProcSelect, (BWin *bwin) );
+FCT (static int, AttributeSelect, (BWin *bwin) );
+FCT (static int, GlobalSelect, (BWin *bwin) );
 FCT (static void, RefreshBrowserWindow, (BWin *bwin) );
-FCT (static void, RemoveFiles, (char *name) );
-FCT (static void, SelectItem, (BWin *bwin, int x, int y, int i) );
+FCT (void, RemoveFiles, (char *name) );
+FCT (void, SelectItem, (BWin *bwin, int x, int y, int i) );
+FCT (void, UpdateTitle, (char *name) );
+FCT (void, RefreshPpButton,() );
 FCT (static void, TitleVisit, (char *name) );
-FCT (static void, UpdateTitle, (char *name) );
+
+#if !HAVE_STRDUP
+extern FCT (char*, strdup, (char*) );
+#endif
 
 /*
 **	Function name : ExposeBrowser
@@ -102,8 +123,17 @@ int ExposeBrowser ( ev )
     if ( ev -> xexpose.window == br.title ) {
 	if ( br.title_name != 0 )
 	  UpdateTitle ( br.title_name );
-	else 
-	  Display3D ( dpy, br.title, br.ts, br.bs, 1, 0 );	
+	else {
+	  Display3D ( dpy, br.title, br.ts, br.bs, 1, 0 );
+/*
+	  if (strcmp ( br.mode -> name, "Java" ) == 0)
+	    UpdateTitle("Java mode");
+	  else if (strcmp ( br.mode -> name, "C++" ) == 0)
+	    UpdateTitle("C++ mode");
+	  else
+	    UpdateTitle("C mode");
+*/
+	}
 	return True;
     }
     
@@ -165,9 +195,33 @@ int ExposeBrowser ( ev )
 		     br.font -> ascent + (B_INTER / 2), "Edit", 4 );
 	return True;
     }
-    
+    if ( ev -> xexpose.window == br.pp ) {
+      RefreshPpButton();
+      return True;
+    }
+    if ( ev -> xexpose.window == br.dump ) {
+	Display3D ( dpy, ev -> xexpose.window, br.ts, br.bs, 1, 0 );
+	XDrawString ( dpy, ev -> xexpose.window, br.gc,
+		     ( br.button_width - XTextWidth ( br.font, "Dump", 4 )) / 2,
+		     br.font -> ascent + (B_INTER / 2), "Dump", 4 );
+	return True;
+    }
+    if ( ev -> xexpose.window == br.restore ) {
+	Display3D ( dpy, ev -> xexpose.window, br.ts, br.bs, 1, 0 );
+	XDrawString ( dpy, ev -> xexpose.window, br.gc,
+		     ( br.button_width - XTextWidth ( br.font, "Rest", 4 )) / 2,
+		     br.font -> ascent + (B_INTER / 2), "Rest", 4 );
+	return True;
+    }
+    if ( ev -> xexpose.window == br.hide ) {
+	Display3D ( dpy, ev -> xexpose.window, br.ts, br.bs, 1, 0 );
+	XDrawString ( dpy, ev -> xexpose.window, br.gc,
+		     ( br.button_width - XTextWidth ( br.font, "Hide", 4 )) / 2,
+		     br.font -> ascent + (B_INTER / 2), "Hide", 4 );
+	return True;
+    }
     expose_flag = False;
-    for ( i = 0; i < 7; i++ ) {
+    for ( i = 0; i < B_NB_WIN; i++ ) {
 	if ( ev -> xexpose.window == br.tbw[i].frame ) {
 	    Display3D ( dpy, ev -> xexpose.window, br.ts, br.bs, 1, 0 );
 	    expose_flag = True;
@@ -179,10 +233,18 @@ int ExposeBrowser ( ev )
 		TitleVisit ( GetFileName ( br.tbw[W_VISIT].text )); 
 	    }
 	    else {
+	      if ((i == 0) && (strcmp ( br.mode -> name, "Java" ) == 0)) {
+		width = XTextWidth ( br.font, "Class", strlen ("Class"));
+		XDrawString ( dpy, ev -> xexpose.window, br.gc, 
+			     (br.tbw[i].f_width - width)/2, br.font -> ascent + (B_INTER / 2),            
+			     "Class", strlen ("Class"));
+	      }
+	      else {
 		width = XTextWidth ( br.font, b_name [i], strlen (b_name [i]));
 		XDrawString ( dpy, ev -> xexpose.window, br.gc, 
 			     (br.tbw[i].f_width - width)/2, br.font -> ascent + (B_INTER / 2),            
-			     b_name [i], strlen (b_name [i]));                                                            
+			     b_name [i], strlen (b_name [i]));
+	      }
 	    }
 	    expose_flag = True;
 	    break;
@@ -211,6 +273,24 @@ int ExposeBrowser ( ev )
     return (expose_flag);
 }	
 
+void RefreshPpButton()
+{
+  if ( br.stat == B_UNMAP )
+    return;
+  
+  XClearWindow ( dpy, br.pp );
+  Display3D ( dpy, br.pp, br.ts, br.bs, 1, 0 );
+  if ( br.mode->pp_flag )
+    XDrawString ( dpy, br.pp, br.gc,
+		 ( br.button_width - XTextWidth ( br.font, "Prepr", 5 )) / 2,
+		 br.font -> ascent + (B_INTER / 2), "Prepr", 5 );
+  else 
+    XDrawString ( dpy, br.pp, br.gc, 
+		 ( br.button_width - XTextWidth ( br.font, "No pp", 5 )) / 2,
+		 br.font -> ascent + (B_INTER / 2), "No pp", 5 );
+}
+
+
 /*
 **	Function name : RefreshBrowserWindow
 **
@@ -239,6 +319,7 @@ static void RefreshBrowserWindow ( bwin )
 **	Input : L'event.
 **	Ouput : Vrai si le button press concerne le browser.
 */
+
 int ButtonBrowser ( ev )
     XButtonEvent *ev;
 {
@@ -255,7 +336,11 @@ int ButtonBrowser ( ev )
 	||  (w == br.rec) 
 	||  (w == br.add) 
 	|| ( w == br.del) 
-	|| ( w == br.dec) 	    	    
+	|| ( w == br.dec) 
+	|| ( w == br.pp) 
+	|| ( w == br.dump) 
+	|| ( w == br.restore) 
+	|| ( w == br.hide) 
 	|| ( w == br.edit ) ) {
 	Display3D ( dpy, w, br.ts, br.bs, 1, 1 );
 	XFlush ( dpy );
@@ -297,9 +382,58 @@ int ButtonBrowser ( ev )
 	  ManageFiles ( B_DEL );
 	else if ( w == br.close )
 	  UnmapBrowser ();
+	else if ( w == br.pp ) {
+	    br.mode->pp_flag ^= 1;
+	    XClearWindow ( dpy, w );
+	    if ( br.mode->pp_flag )
+	      XDrawString ( dpy, w,br.gc, 
+			   (br.button_width - XTextWidth ( br.font, "Prepr", 5 )) / 2,
+			   br.font -> ascent + (B_INTER / 2), "Prepr", 5 );
+	    else 
+	      XDrawString ( dpy, w, br.gc, 
+			   (br.button_width - XTextWidth ( br.font, "No pp", 5 )) / 2,
+			   br.font -> ascent + (B_INTER / 2), "No pp", 5 );
+	    Display3D ( dpy, w, br.ts, br.bs, 1, 0 );
+	    if (br.mode->pp_flag) {
+	      FCT (extern char *, GetStringFromDB, (char *, int) );
+	      char * opt;
+	      
+	      if (br.mode->pp_options) {
+		InputStrSave(br.mode->pp_options, strlen(br.mode->pp_options));
+		InputStrPrevious(0);
+	      }
+	      if ((opt = GetStringFromDB("preproc. options ? ", 0)) == 0) {
+		if (br.mode->pp_options)
+		  free(br.mode->pp_options);
+		br.mode->pp_options = 0;
+	      }
+	      else if (*opt != 7) {
+		if (br.mode->pp_options)
+		  free(br.mode->pp_options);
+		br.mode->pp_options = strdup(opt);
+	      }
+	    }
+	}
+	else if ( w == br.dump )
+	  MenuBrowserDump(0);
+	else if ( w == br.restore )
+	  MenuBrowserRestore(0);
+	else if ( w == br.hide ) {
+	  if (br.tbw[W_CLASS].select) {
+	    int len;
+	    char * class_name;
+	    
+	    class_name = (char *) GetCurrentLine ( br.tbw[W_CLASS].buf, &len);
+	    class_name [len] = '\0';
+	    
+	    BrowserHide(class_name);
+	  }
+	  else
+	    BrowserHide(0);
+	}
 	return True;
     }
-    for ( i = 0; i< 7; i++ ) {
+    for ( i = 0; i < B_NB_WIN; i++ ) {
 	if ( w == br.tbw[i].text -> window ) {
 	    if ( i != W_VISIT ) {
 		SelectItem ( &br.tbw[i], ev -> x, ev -> y, i );
@@ -447,7 +581,7 @@ static void AddFiles ()
 **	Input : Le nom du fichiers.
 **	Ouput :
 */
-static void RemoveFiles ( name )
+void RemoveFiles ( name )
     char *name;
 {
     char *tmp, *p;
@@ -539,28 +673,32 @@ void RefreshBrowserInfos ()
     int i;
     XEvent event;
     
-    SetInfos (); 	/* Classes, Procs, Files */
+    SetInfos (); 	/* Classes, Procs, Files, Attribute, Globals */
     br.tbw[W_CLASS].select = br.tbw[W_PROC].select = br.tbw[W_FILES].select = 0;
     br.tbw[W_PARENT].select = br.tbw[W_CHILD].select = br.tbw[W_METHOD].select = 0;
+    br.tbw[W_ATTRIBUTES].select = br.tbw[W_GLOBALS].select = 0;
     
-    if ( strcmp ( br.mode -> name, "C++mode" ) == 0 ) {
+    if ( (strcmp ( br.mode -> name, "C++mode" ) == 0)
+	|| (strcmp ( br.mode -> name, "Java" ) == 0)) {
 	i = 0;
     }
     else if ( strcmp ( br.mode -> name, "C-mode" ) == 0 ) {
-	i = 4;
+	i = C_FIRST_WIN;
     }
     else
       return;
     
     if ( br.stat == B_UNMAP )
       return;
-    
-    for ( ;i < 6; i++ ) {
+
+    for ( ;i < (B_NB_WIN - 1); i++ )
+      {
 	while ( XCheckWindowEvent ( dpy,
 				   br.tbw[i].text -> window , ExposureMask, &event ));
 	switch ( i ) {
 	case W_FILES:
 	case W_PROC:
+	case W_GLOBALS:
 	case W_CLASS:
 	  RefreshBrowserWindow ( &br.tbw[i] );
 	  FirstPage ( br.tbw[i].text );
@@ -568,6 +706,7 @@ void RefreshBrowserInfos ()
 	case W_PARENT:
 	case W_CHILD:
 	case W_METHOD:
+	case W_ATTRIBUTES:
 	  KillText ( dpy, br.tbw[i].text ); 
 	  break;
 	default:
@@ -575,6 +714,7 @@ void RefreshBrowserInfos ()
 	}
 	ConfigScroll ( &br.tbw[i] );			
     }
+    UpdateTitle ( " " );
 }
 
 /*
@@ -585,7 +725,7 @@ void RefreshBrowserInfos ()
 **	Input : 
 **	Ouput :
 */
-static void ButtonEdit ()
+void ButtonEdit ()
 {
     EdWin *ew;
     char *filename;
@@ -634,13 +774,14 @@ static void ButtonEdit ()
 **	Input : Le contexte, la position, le type.
 **	Ouput :
 */
-static void SelectItem ( bwin, x, y, i )
+void SelectItem ( bwin, x, y, i )
     BWin *bwin;
     int x, y;
     int i;
 {
     int n;
     int scroll = bwin -> text -> sl;
+    int object_flag = False;
     
     if ( ( bwin -> text -> lines_in_buf == 1 ) 
 	|| (MoveToXYinTextWindow ( bwin -> text,  x, y ) == -1 ))
@@ -661,26 +802,39 @@ static void SelectItem ( bwin, x, y, i )
     
     switch ( i ) {
     case W_CLASS:
-      ClassSelect ( bwin );
+      if (ClassSelect(bwin)==True)
+	object_flag = True;
       break;
     case W_PARENT:
     case W_CHILD:
-      ParentChildSelect ( &br.tbw[i] );
+      if (ParentChildSelect(&br.tbw[i])==True)
+	object_flag = True;
       break;
     case W_METHOD:
-      MethodSelect ( bwin );
+      if (MethodSelect(bwin)==True)
+	object_flag = True;
       break;
     case W_PROC:
-      ProcSelect ( bwin );
+      if (ProcSelect(bwin)==True)
+	object_flag = True;
       break;
     case W_FILES:
-      FileSelect ( bwin );
+      if (FileSelect(bwin)==True)
+	object_flag = True;
+      break;
+    case W_ATTRIBUTES:
+      if (AttributeSelect(bwin)==True)
+	object_flag = True;
+      break;
+    case W_GLOBALS:
+      if (GlobalSelect(bwin)==True)
+	object_flag = True;
       break;
     default:
       break;
     }
     br.old_click = br.click_time;
-    if ( br.visit_raise == True )
+    if ((br.visit_raise==True)&&(object_flag==True))
       XMapRaised ( dpy, br.tbw[W_VISIT].frame );
 }
 
@@ -691,63 +845,79 @@ static void SelectItem ( bwin, x, y, i )
 **	Input : Le contexte
 **	Ouput :
 */
-static void ClassSelect ( bwin )
+static int ClassSelect ( bwin )
     BWin *bwin;
 {
     char  *class_name;
-    char **t_class;
     int len;
     Position *infos;
     char **tmp;
     
     class_name = (char *) GetCurrentLine ( br.tbw[W_CLASS].buf, &len);
     class_name [len] = '\0';
-    UpdateTitle ( class_name );
-    infos = get_class_decl ( REMOVE_CLASS_INFO (class_name) );
+    UpdateTitle ( class_name + CLASS_PLENGTH);
+    class_name = REMOVE_CLASS_INFO (class_name);
+    infos = get_class_decl ( class_name );
     
-    if ( infos == 0 ) return;
+    if ( infos == 0 ) 
+      return False;
+    class_name = strdup(class_name);
+    if (! class_name) 
+      return False;
     
 /*    ClearBuffer ( br.tbw[W_VISIT].buf ); */
     EditFile ( &br.tbw[W_VISIT], infos -> file_name );
     AtLineDisplayPage ( br.tbw[W_VISIT].text, infos -> line_number );
-    if ( infos != 0 )
-      (void) free ( (char *) infos );
+    (void) free ( (char *) infos );
     
-    t_class = br.class_save + (bwin -> select -1);
+    if ( br.methods_save != 0 )
+      (void) free ( (char *) br.methods_save );
+    /*	(void) RefreshBrowserInfos (); */ /* Fixed by mtg@csd.uch.gr */
+    br.methods_save = ( char **)  get_methods_list ( class_name, 1);
+    if (br.methods_save == 0)
+      KillText(dpy, br.tbw[W_METHOD].text);
+    else
+      ExtractInfos ( br.methods_save, W_METHOD );
+    ConfigScroll ( &br.tbw[W_METHOD] ); 
+    br.tbw[W_METHOD].select = 0;
+    ClearPage ( br.tbw[W_METHOD].text );
+    FirstPage ( br.tbw[W_METHOD].text ); 
     
-    if ( t_class != 0 ) {
-	if ( br.methods_save != 0 )
-	  (void) free ( (char *) br.methods_save );
-/*	(void) RefreshBrowserInfos (); */ /* Fixed by mtg@csd.uch.gr */
-	br.methods_save = ( char **)  get_methods_list ( *t_class);
-	if (br.methods_save == 0)
-	  KillText(dpy, br.tbw[W_METHOD].text);
-	else
-	  ExtractInfos ( br.methods_save, W_METHOD );
-	ConfigScroll ( &br.tbw[W_METHOD] ); 
-	br.tbw[W_METHOD].select = 0;
-	ClearPage ( br.tbw[W_METHOD].text );
-	FirstPage ( br.tbw[W_METHOD].text ); 
-	
-	tmp = ( char **)  get_parents_list ( *t_class);
-	ExtractInfos ( tmp, W_PARENT );
-	if ( tmp != 0 )
-	  free ( (char *) tmp );
-	ConfigScroll ( &br.tbw[W_PARENT] ); 
-	br.tbw[W_PARENT].select = 0;
-	ClearPage ( br.tbw[W_PARENT].text );
-	FirstPage ( br.tbw[W_PARENT].text ); 
-	
-	FirstPage ( br.tbw[W_CHILD].text );
-	tmp = ( char **)  get_sons_list ( *t_class);
-	ExtractInfos ( tmp, W_CHILD );
-	if ( tmp != 0 )
-	  free ( (char *) tmp );
-	ConfigScroll ( &br.tbw[W_CHILD] );
-	br.tbw[W_CHILD].select = 0;
-	ClearPage ( br.tbw[W_CHILD].text );
-	FirstPage ( br.tbw[W_CHILD].text ); 
-    }
+    if ( br.attributes_save != 0 )
+      (void) free ( (char *) br.attributes_save );
+    br.attributes_save = ( char **)  get_attributes_list ( class_name, 1);
+    if (br.attributes_save == 0)
+      KillText(dpy, br.tbw[W_ATTRIBUTES].text);
+    else
+      ExtractInfos ( br.attributes_save, W_ATTRIBUTES );
+    ConfigScroll ( &br.tbw[W_ATTRIBUTES] ); 
+    br.tbw[W_ATTRIBUTES].select = 0;
+    ClearPage ( br.tbw[W_ATTRIBUTES].text );
+    FirstPage ( br.tbw[W_ATTRIBUTES].text ); 
+
+    tmp = ( char **)  get_parents_list ( class_name, 0);
+    KillText(dpy, br.tbw[W_PARENT].text);
+    ExtractInfos ( tmp, W_PARENT );
+    if ( tmp != 0 )
+      free ( (char *) tmp );
+    ConfigScroll ( &br.tbw[W_PARENT] ); 
+    br.tbw[W_PARENT].select = 0;
+    ClearPage ( br.tbw[W_PARENT].text );
+    FirstPage ( br.tbw[W_PARENT].text ); 
+
+    FirstPage ( br.tbw[W_CHILD].text );
+    tmp = ( char **)  get_sons_list ( class_name);
+    KillText(dpy, br.tbw[W_CHILD].text);
+    free(class_name);
+    ExtractInfos ( tmp, W_CHILD );
+    if ( tmp != 0 )
+      free ( (char *) tmp );
+    ConfigScroll ( &br.tbw[W_CHILD] );
+    br.tbw[W_CHILD].select = 0;
+    ClearPage ( br.tbw[W_CHILD].text );
+    FirstPage ( br.tbw[W_CHILD].text ); 
+    
+    return True;
 }
 
 /*
@@ -758,7 +928,7 @@ static void ClassSelect ( bwin )
 **	Input : Le titre
 **	Ouput :
 */
-static void UpdateTitle ( name )
+void UpdateTitle ( name )
     char *name;
 {
     if ( name == 0 )
@@ -785,7 +955,18 @@ static void UpdateTitle ( name )
 **	Input : Le contexte.
 **	Ouput :
 */
-static void ProcSelect ( bwin )
+
+static char * nth_visible(tbl, nth)
+    char ** tbl;
+    unsigned nth;
+    
+{
+  while (HIDED(*tbl) || nth--)
+    tbl += 1;
+  return *tbl;
+}
+
+static int ProcSelect ( bwin )
     BWin *bwin;
 {
     char  *proc_name;
@@ -794,18 +975,50 @@ static void ProcSelect ( bwin )
     
     proc_name = (char *) GetCurrentLine ( bwin -> buf, &len);
     proc_name [len] = '\0';
-    UpdateTitle ( proc_name );
-    infos = get_proc_impl ( proc_name );
+    UpdateTitle ( proc_name + PROC_PLENGTH);
+    infos = get_proc_impl ( nth_visible(br.procs_save, bwin->select - 1));
     
     if ( infos != 0 ) {
-/*	ClearBuffer ( br.tbw[W_VISIT].buf ); */
 	EditFile ( &br.tbw[W_VISIT], infos -> file_name );
 	AtLineDisplayPage ( br.tbw[W_VISIT].text, infos -> line_number );
+	(void) free ( (char *) infos );
+	return True;
     }
-    if ( infos != 0 )
-      (void) free ( (char *) infos );
+    else
+      return False;
 }
 
+
+/*
+**	Function name : GlobalSelect
+**
+**	Description : Selection d'une variable
+**	Input : Le contexte.
+**	Ouput :
+*/
+static int GlobalSelect ( bwin )
+    BWin *bwin;
+{
+    char  *global_name;
+    int len;
+    Position *infos;
+    
+    global_name = (char *) GetCurrentLine ( bwin -> buf, &len);
+    global_name [len] = '\0';
+    UpdateTitle ( global_name + GLOBAL_PLENGTH);
+    infos = get_global_impl (nth_visible(br.globals_save, bwin->select - 1));
+    
+    if ( infos != 0 ) {
+	EditFile ( &br.tbw[W_VISIT], infos -> file_name );
+	AtLineDisplayPage ( br.tbw[W_VISIT].text, infos -> line_number );
+	(void) free ( (char *) infos );
+	return True;
+    }
+    else
+      return False;
+}
+
+
 /*
 **	Function name : FileSelect
 **
@@ -813,7 +1026,7 @@ static void ProcSelect ( bwin )
 **	Input : Le contexte.
 **	Ouput :
 */
-static void FileSelect ( bwin )
+static int FileSelect ( bwin )
     BWin *bwin;
 {
     char *name, **t_name;
@@ -837,11 +1050,13 @@ static void FileSelect ( bwin )
     }
     
     if ( name != 0 ) {
-/*	ClearBuffer ( br.tbw[W_VISIT].buf ); */
 	EditFile ( &br.tbw[W_VISIT], *t_name );
 	FirstPage ( br.tbw[W_VISIT].text );
 	TextCursorOn ( br.tbw[W_VISIT].text );
+	return True;
     }
+    else
+      return False;
 }
 
 /*
@@ -851,48 +1066,98 @@ static void FileSelect ( bwin )
 **	Input : Le contexte.
 **	Ouput :
 */
-static void MethodSelect ( bwin )
+static int MethodSelect ( bwin )
     BWin *bwin;
 {
     char  *method_name;
-    char **t_methods;
+    char *t_methods;
     int len;
     Position *infos;
     char *buf;
     
     method_name = (char *) GetCurrentLine ( br.tbw[W_METHOD].buf, &len);
     method_name [len] = '\0';
-    t_methods = br.methods_save + ( bwin -> select - 1 );
+    t_methods = nth_visible(br.methods_save, bwin -> select - 1 );
 #ifndef lint
-    buf = (char *) malloc ( (unsigned) (strlen (GET_METHOD_CLASS(*t_methods))
-					+ strlen ( *t_methods ) + 4) );
-    (void) strcpy ( buf, GET_METHOD_CLASS(*t_methods) );
+    buf = (char *) malloc ( (unsigned) (strlen (GET_METHOD_CLASS(t_methods))
+					+ strlen ( t_methods ) + 4) );
+    (void) strcpy ( buf, GET_METHOD_CLASS(t_methods) );
 #else 
     buf = 0;
 #endif
     (void) strcat ( buf, "::" );
-    (void) strcat ( buf, *t_methods );
+    (void) strcat ( buf, t_methods );
     UpdateTitle ( buf );
     if ( buf != 0 )
       (void) free ( buf );
     
     if ( br.dec_imp_flag == DEC_MODE ) 
 #ifndef lint
-      infos =  get_method_decl ( GET_METHOD_CLASS(*t_methods), *t_methods );
+      infos =  get_method_decl ( GET_METHOD_CLASS(t_methods), t_methods );
     else
-      infos =  get_method_impl ( GET_METHOD_CLASS(*t_methods), *t_methods );
+      infos =  get_method_impl ( GET_METHOD_CLASS(t_methods), t_methods );
 #else
     infos = (Position*) t_methods;	/* affecte infos, et utilise t_methods */
 #endif
     if ( infos != 0 ) {
-/*	ClearBuffer ( br.tbw[W_VISIT].buf ); */
 	EditFile ( &br.tbw[W_VISIT], infos -> file_name );
 	AtLineDisplayPage ( br.tbw[W_VISIT].text, infos -> line_number );
+	(void) free ( (char *) infos );
+	return True;
     }
-    if ( infos != 0 )
-      (void) free ( (char *) infos );
+    else
+      return False;
 }
 
+
+/*
+**	Function name : AttributeSelect
+**
+**	Description : Selection d'un attribut
+**	Input : Le contexte.
+**	Ouput :
+*/
+static int AttributeSelect ( bwin )
+    BWin *bwin;
+{
+    char  *attr_name;
+    char *t_attrs;
+    int len;
+    Position *infos;
+    char *buf;
+    
+    attr_name = (char *) GetCurrentLine ( br.tbw[W_ATTRIBUTES].buf, &len);
+    attr_name [len] = '\0';
+    t_attrs = nth_visible(br.attributes_save, bwin -> select - 1 );
+#ifndef lint
+    buf = (char *) malloc ( (unsigned) (strlen (GET_ATTRIBUTE_CLASS(t_attrs))
+					+ strlen ( t_attrs ) + 4) );
+    (void) strcpy ( buf, GET_ATTRIBUTE_CLASS(t_attrs) );
+#else 
+    buf = 0;
+#endif
+    (void) strcat ( buf, "::" );
+    (void) strcat ( buf, t_attrs );
+    UpdateTitle ( buf );
+    if ( buf != 0 )
+      (void) free ( buf );
+    
+#ifndef lint
+    infos =  get_attribute_decl ( GET_ATTRIBUTE_CLASS(t_attrs), t_attrs );
+#else
+    infos = (Position*) t_attrs;	/* affecte infos, et utilise t_attrs */
+#endif
+    if ( infos != 0 ) {
+	EditFile ( &br.tbw[W_VISIT], infos -> file_name );
+	AtLineDisplayPage ( br.tbw[W_VISIT].text, infos -> line_number );
+	(void) free ( (char *) infos );
+	return True;
+    }
+    else
+      return False;
+}
+
+
 /*
 **	Function name : PrentChildSelect
 **
@@ -900,7 +1165,7 @@ static void MethodSelect ( bwin )
 **	Input : Le contexte.
 **	Ouput :
 */
-static void ParentChildSelect ( bwin )
+static int ParentChildSelect ( bwin )
     BWin *bwin;
 {
     char *class_name;
@@ -918,16 +1183,21 @@ static void ParentChildSelect ( bwin )
     t_class = br.class_save;
     
     while ( t_class != 0 ) {
-	if ( (*t_class == 0) || (strcmp(class_name, *t_class) == 0) ) 
+      if (*t_class == 0)
+	break;
+      
+      if (! HIDED(*t_class)) {
+	if (strcmp(class_name, *t_class) == 0)
 	  break;
-	t_class ++;
-	i++;
+	i += 1;
+      }
+      t_class += 1;
     }
     if ( *t_class == 0 ) {
 	UpdateTextItem ( br.tbw[W_PARENT].text, br.tbw[W_PARENT].select );
 	UpdateTitle ( " " );
 	br.tbw[W_PARENT].select = 0;
-	return;
+	return False;
     }
     /* i = no ligne dans la liste des classes */
     UpdateTextItem ( b_class -> text, b_class -> select );
@@ -935,6 +1205,7 @@ static void ParentChildSelect ( bwin )
     CurrentLineToTop ( b_class -> text );
     UpdateTextItem ( b_class -> text, b_class -> select );
     SelectItem ( b_class, 5, 5, W_CLASS );
+    return True;
 }
 
 
@@ -999,33 +1270,113 @@ static void ButtonInVisitWindow ( bwin, ev )
     BWin *bwin;
     XButtonEvent *ev;
 {
-    char *msg;
-    
-    switch ( ev -> button ) {
-    case Button1:
-      TextCursorOff ( bwin -> text );
-      (void) MoveToXYinTextWindow ( bwin -> text, ev -> x, ev -> y );
-      TextCursorOn ( bwin -> text );
-      break;
-    case Button2:
-      msg = (char *) ie_call_function ( (Text *) bwin -> text, "color_buffer", 0, 0);
-      if(msg)
-	DisplayWMessage (msg, "Color buffer failed", 0);
-      else {
-	  ClipOn(bwin -> text, 0);
-	  RefreshPage(bwin -> text);
-	  ClipOff(bwin -> text);
-      }
-      break;
-    case Button3:
-      TextCursorOff ( bwin -> text );
-      XSync ( dpy, False );
-      StoreBytesInCutBuffer ( bwin -> text, ev -> x, ev -> y );
-      TextCursorOn ( bwin -> text );
-      break;
-    default:
-      break;
+  char *msg;
+  extern Window w_sel;
+  EdWin **t;
+  
+  switch ( ev -> button ) {
+  case Button1:
+    /*      TextCursorOff ( bwin -> text );
+       (void) MoveToXYinTextWindow ( bwin -> text, ev -> x, ev -> y );
+       TextCursorOn ( bwin -> text );
+    */
+    TextCursorOff ( bwin -> text );
+    StorePosition ( bwin -> text );
+    if ( MoveToXYinTextWindow ( bwin -> text,
+			       ev -> x, ev -> y ) == -1 ) {
+      HoleToRight ( bwin -> text -> buf );
+      GotoLineNumber ( bwin -> text, bwin -> text -> lines_in_buf );
+      SetPosition ( bwin -> text );
+      UpdatePage ( bwin -> text );
     }
+    TextCursorOn ( bwin -> text );
+    (void) HandleSelection (bwin -> text, ev);
+    if (bwin -> text -> selection -> select == True) {
+      /* Il faut effacer la derniere selection si elle existe */
+      for ( t = TWin; t < TWin + MAXWIN; t++ ) {
+	if ( *t == 0 ) continue;
+	if ((*t) -> text -> selection -> select == True) {
+	  /*	  if ((*t) -> text != edwin -> text) { */
+	  (void) ClearSelection ((*t) -> text, True);
+	  SetCurrentLine ((*t) -> text);
+	  TextCursorOn((*t) -> text);
+	  FreezeTextCursor((*t) -> text);
+	  break;
+	  /*	  } */
+	}
+      }
+      w_sel = ev -> window;
+      XSetSelectionOwner ( dpy, XA_PRIMARY, w_sel, ev -> time);
+      if ( w_sel == XGetSelectionOwner ( dpy, XA_PRIMARY )) {
+#ifdef DEBUG
+	(void) fprintf (stderr, "Selection owner OK\n");
+#endif /* DEBUG */	      
+      }
+      else {
+#ifdef DEBUG
+	(void) fprintf (stderr, "Selection owner failed\n");
+#endif /* DEBUG */
+	w_sel = None;
+      }
+    }
+    break;
+  case Button2:
+    StorePosition ( bwin -> text );
+    msg = (char *) ie_call_function ( (Text *) bwin -> text, "color_buffer", 0, 0);
+    if(msg)
+      DisplayWMessage (msg, "Color buffer failed", 0);
+    else {
+      SetCurrentLine (bwin -> text);
+      ClipOn(bwin -> text, 0);
+      RefreshPage(bwin -> text);
+      ClipOff(bwin -> text);
+    }
+    break;
+  case Button3:
+    /*
+       TextCursorOff ( bwin -> text );
+       XSync ( dpy, False );
+       StoreBytesInCutBuffer ( bwin -> text, ev -> x, ev -> y );
+       TextCursorOn ( bwin -> text );
+    */
+    TextCursorOff ( bwin -> text );
+    XSync ( dpy, False );
+    if (bwin -> text -> selection -> select == True) {
+      StorePosition ( bwin -> text );
+      if ( MoveToXYinTextWindow ( bwin -> text,
+				 ev -> x, ev -> y ) == -1 ) {
+	HoleToRight ( bwin -> text -> buf );
+	GotoLineNumber ( bwin -> text, bwin -> text -> lines_in_buf );
+	SetPosition ( bwin -> text );
+	UpdatePage ( bwin -> text );
+      }
+      (void) UpdateSelection(bwin -> text);
+    }
+    else
+      StoreBytesInCutBuffer ( bwin -> text, ev -> x, ev -> y );
+
+    TextCursorOn ( bwin -> text );
+    RefreshSelection(bwin -> text);
+    if (bwin -> text -> selection -> select == True) {
+      /* Il faut effacer la derniere selection si elle existe */
+      for ( t = TWin; t < TWin + MAXWIN; t++ ) {
+	if ( *t == 0 ) continue;
+	if ((*t) -> text -> selection -> select == True) {
+/*	  if ((*t) -> text != edwin -> text) { */
+	    (void) ClearSelection ((*t) -> text, True);
+	    SetCurrentLine ((*t) -> text);
+	    TextCursorOn((*t) -> text);
+	    FreezeTextCursor((*t) -> text);
+	    break;
+/*	  } */
+	}
+      }
+      w_sel = ev -> window;
+      XSetSelectionOwner ( dpy, XA_PRIMARY, w_sel, ev -> time);
+      if ( w_sel != XGetSelectionOwner ( dpy, XA_PRIMARY ))
+	w_sel = None;
+    }
+  }
 }
 
 /*
@@ -1043,7 +1394,7 @@ int KeyPressInBrowser ( ev )
 {
     int i;
     
-    for ( i = 0; i < 7; i++ ) {
+    for ( i = 0; i < B_NB_WIN; i++ ) {
 	if ( ev -> window == br.tbw [i].text -> window ) {
 #ifdef DEBUG				
 	    fprintf ( stderr, "Key Press In Browser text\n");
@@ -1125,3 +1476,22 @@ static void HandleKeyPressInBrowser(ev, bw)
   }
 }
 
+/*
+**	Function name : BrowserClearSelection
+**
+**	Description :
+**	Input :
+**	Output :
+*/
+void BrowserClearSelection()
+{
+  Text *text = br.tbw[W_VISIT].text;
+
+  if ( text -> selection -> select == True) {
+    ClearSelection(text, True);
+    SetCurrentLine (text);
+    TextCursorOn(text);
+    FreezeTextCursor(text);
+  }
+}
+
