@@ -52,22 +52,40 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 
-#ifdef apollo
-#include <sys/dir.h>
-#define dirent direct
+#if HAVE_DIRENT_H
+#  include <dirent.h>
+#  define NAMLEN(dirent) strlen((dirent)->d_name)
 #else
-#include <dirent.h>
+#  define dirent direct
+#  define NAMLEN(dirent) (dirent)->d_namlen
+#  if HAVE_SYS_NDIR_H
+#    include <sys/ndir.h>
+#  endif
+#  if HAVE_SYS_DIR_H
+#    include <sys/dir.h>
+#  endif
+#  if HAVE_NDIR_H
+#    include <ndir.h>
+#  endif
 #endif
 
+#if HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
+#endif
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/param.h>
-#ifndef __FreeBSD__
-#include <malloc.h>
-#endif
-#include <stdlib.h>
 
+#if STDC_HEADERS
+#  include <stdlib.h>
+#else
+#  if HAVE_MALLOC_H
+#    include <malloc.h>
+#  endif
+#endif
 
 #include "options.h"
 #include "mode.h"
@@ -75,12 +93,15 @@
 #include "browser_eve.h"
 #include "parse.h"
 #include "page.h"
+#include "main_events.h"
 
 extern Display  	*dpy;
 Browser		br;
 
 char *b_name[] = {
-	"Classes", 	"Parents",	"Children", "Methods",
+	"Types",
+ 	"Parents",	"Children", "Methods",
+	"Attributes", "Globals",
 	"Functions", "Files","Visit", 0
 };
 
@@ -95,6 +116,10 @@ FCT (static void, ConfigChild, () );
 FCT (static void, ConfigFiles, () );
 FCT (static void, ConfigMethod, () );
 FCT (static void, ConfigProc, () );
+FCT (static void, ConfigAttributes, () );
+FCT (static void, ConfigGlobals, () );
+
+unsigned Hide_Bits;
 
 static int already_map = False;
 static  Atom wm_delwin;
@@ -137,6 +162,8 @@ void InitBrowser ()
     br.font = LoadFont ( dpy, B_FONT );
     br.methods_save = br.class_save = br.files_save = 0;
     br.dec_imp_flag = DEC_MODE;
+    br.attributes_save = br.globals_save = br.procs_save = 0;
+    Hide_Bits = 0;
     br.mode = (Mode *) GetMode ("C++mode");
     br.title_name = 0;
     
@@ -149,7 +176,7 @@ void InitBrowser ()
     
     br.title_height = br.font -> ascent + br.font-> descent + B_INTER;
     br.button_height = br.font -> ascent + br.font-> descent + B_INTER;
-    br.button_width = XTextWidth ( br.font, B_SEARCH, strlen (B_SEARCH) ) + (2 * B_INTER);
+    br.button_width = XTextWidth ( br.font, "Close", strlen("Close")) + (B_INTER/2) +8;
 /*    br.width = (DisplayWidth ( dpy,DefaultScreen ( dpy )) * 6 )  / 11;
     br.height = (DisplayHeight ( dpy,DefaultScreen ( dpy )) * 6 ) / 8; */
     br.width = GetOpGeo( OP_B_WIDTH );
@@ -190,8 +217,8 @@ void InitBrowser ()
       sizehints.min_width = 500;
       sizehints.min_height = 450;
     }
-    sizehints.max_width = (DisplayWidth ( dpy,DefaultScreen ( dpy )) * 9) / 10;
-    sizehints.max_height = (DisplayHeight ( dpy,DefaultScreen ( dpy )) * 9) / 10;
+    sizehints.max_width = DisplayWidth(dpy,DefaultScreen(dpy));
+    sizehints.max_height = DisplayHeight(dpy,DefaultScreen(dpy));
     XSetStandardProperties ( dpy, br.frame, "Browser","Browser",
 			    None, 0, 0, &sizehints );
     
@@ -200,22 +227,14 @@ void InitBrowser ()
     
     (void) BrowserButtons ();
     init_browser (); /* Le moteur */
-/*    
-    br_start = (char *) getenv ( "XCORAL_PARSE_AT_STARTUP" );
-    if ( (br_start != 0) && (strcmp("True", br_start ) == 0) ){
-	(void) getcwd ( pathname, MAXPATHLEN );
-	br_start = (char *) getenv ( "XCORAL_PARSE_PATH" );
-	if ( br_start == 0 )
-	  (void) LoadDir ( pathname );
-	else {
-	    ParseDir ( br_start, pathname );
-	}
-    }
-*/
     br.parse_flag = NO_RECURSIVE;
     br_start = (char *) getenv ( "XCORAL_PARSE_PATH" );
     if ( br_start != 0 ) {
+#if HAVE_GETCWD
 	(void) getcwd ( pathname, MAXPATHLEN );
+#else
+	(void) getwd  ( pathname );
+#endif
 	br.parse_flag = RECURSIVE;
 	ParseDir ( br_start, pathname );
     }
@@ -226,20 +245,6 @@ void InitBrowser ()
     
     visit_raise = (char *) getenv ( "XCORAL_VISIT_RAISE" );
     br.visit_raise = ( visit_raise == 0 || strcmp ( "True", visit_raise ) == 0 ) ? True : False;
-/*    
-    sizehints.min_height /= 3; 
-    sizehints.min_width /= 3;
-    sizehints.x = GetOpGeo( OP_V_X );
-    sizehints.y = GetOpGeo( OP_V_Y );
-    if (sizehints.x >= 0 && sizehints.y >= 0)
-      sizehints.flags |= USPosition;
-    else
-      sizehints.flags &= ~USPosition;
-    sizehints.width = GetOpGeo( OP_V_WIDTH );
-    sizehints.height = GetOpGeo( OP_V_HEIGHT );
-    XSetStandardProperties ( dpy, br.tbw[W_VISIT].frame, "Visit Window","Visit Window",
-			    None, 0, 0, &sizehints );
-*/			    
     br.old_click = 0;
     br.click_time = 0;
 }
@@ -383,14 +388,25 @@ static void BrowserButtons ()
 				    br.button_width, br.button_height, 0, black, br.bg );
     br.rec = XCreateSimpleWindow (dpy, br.main, 0, 0,
 				    br.button_width, br.button_height, 0, black, br.bg );
+    br.pp = XCreateSimpleWindow (dpy, br.main, 0, 0,
+				    br.button_width, br.button_height, 0, black, br.bg );
+    br.dump = XCreateSimpleWindow (dpy, br.main, 0, 0,
+				    br.button_width, br.button_height, 0, black, br.bg );
+    br.restore = XCreateSimpleWindow (dpy, br.main, 0, 0,
+				    br.button_width, br.button_height, 0, black, br.bg );
+    br.hide = XCreateSimpleWindow (dpy, br.main, 0, 0,
+				   br.button_width, br.button_height, 0, black, br.bg );
     
     XSelectInput ( dpy, br.dec, ExposureMask | ButtonPressMask | ButtonReleaseMask );
     XSelectInput ( dpy, br.add, ExposureMask | ButtonPressMask | ButtonReleaseMask);
     XSelectInput ( dpy, br.del, ExposureMask | ButtonPressMask | ButtonReleaseMask);
-    XSelectInput ( dpy, br.edit, ExposureMask | ButtonPressMask | ButtonReleaseMask);	
-    XSelectInput ( dpy, br.close, ExposureMask | ButtonPressMask | ButtonReleaseMask);	
-    XSelectInput ( dpy, br.rec, ExposureMask | ButtonPressMask | ButtonReleaseMask);	
-    
+    XSelectInput ( dpy, br.edit, ExposureMask | ButtonPressMask | ButtonReleaseMask);
+    XSelectInput ( dpy, br.close, ExposureMask | ButtonPressMask | ButtonReleaseMask);
+    XSelectInput ( dpy, br.rec, ExposureMask | ButtonPressMask | ButtonReleaseMask);
+    XSelectInput ( dpy, br.pp, ExposureMask | ButtonPressMask | ButtonReleaseMask);
+    XSelectInput ( dpy, br.dump, ExposureMask | ButtonPressMask | ButtonReleaseMask);
+    XSelectInput ( dpy, br.restore, ExposureMask | ButtonPressMask | ButtonReleaseMask);
+    XSelectInput ( dpy, br.hide, ExposureMask | ButtonPressMask | ButtonReleaseMask);
 }
 
 /*
@@ -423,7 +439,7 @@ void LoadDir ( dir )
 	    buf = dp->d_name;			
 	}
 	else {
-	    buf = (char *) malloc ( (unsigned) (strlen ( dir ) + strlen ( dp->d_name ) + 4) );
+	    buf = (char *) malloc ( (unsigned) (strlen (dir) + NAMLEN(dp) + 4) );
 	    (void) sprintf ( buf, "%s/%s", dir, dp->d_name );
 	}
 	(void) stat ( buf , &st );
@@ -466,8 +482,8 @@ static void BrowserTextWindows ()
     screen = DefaultScreen ( dpy );
     root = RootWindow ( dpy, screen );
     black = BlackPixel ( dpy, DefaultScreen ( dpy ));
-    
-    for ( i = 0; i < 7; i++ ) {	
+
+    for ( i = 0; i < B_NB_WIN; i++ ) {	
 	if ( i == W_VISIT ) {
 	    frame = XCreateSimpleWindow (dpy, root, GetOpGeo( OP_V_X ),
 					 GetOpGeo( OP_V_Y ),
@@ -550,17 +566,19 @@ static void BrowserTextWindows ()
 */
 void SetInfos ()
 {
-    char **tmp;
-    
     if ( br.class_save != 0 )
       (void) free ( (char *) br.class_save );
     br.class_save = ( char **)  get_classes_list ();
     ExtractInfos ( br.class_save, W_CLASS );
-    tmp = ( char **)  get_procs_list ();
+    if (br.procs_save)
+      (void) free ((char *) br.procs_save);
+    br.procs_save = ( char **)  get_procs_list (0);
+    ExtractInfos (br.procs_save, W_PROC ); 
     
-    ExtractInfos ( tmp, W_PROC ); 
-    if ( tmp != 0 )
-      (void) free ( (char *) tmp );
+    if (br.globals_save)
+      (void) free ((char *) br.globals_save);
+    br.globals_save = ( char **)  get_globals_list (0);
+    ExtractInfos ( br.globals_save, W_GLOBALS ); 
     if ( br.files_save != 0 )
       (void) free ( (char *) br.files_save );
     br.files_save = (char **)  get_files_list ();
@@ -589,6 +607,7 @@ void ExtractInfos ( names, type )
     if ( names != (char **) Null )  {
 	ClearBuffer ( bwin -> buf );
 	while ( *names != (char *) Null ) {
+	  if ((type == W_FILES) || (! HIDED(*names))) {
 	    switch ( type ) {
 	    case W_CLASS:
 	    case W_PARENT: 
@@ -600,15 +619,21 @@ void ExtractInfos ( names, type )
 	      break;
 	    case W_PROC:
 	      infos_name = GET_PROCEDURE_INFO(*names);
-	      infos_name = *names;
 	      break;
 	    case W_FILES:
 	      infos_name = *names;
-	      break;	
+	      break;
+	    case W_ATTRIBUTES:
+	      infos_name = GET_ATTRIBUTE_INFO(*names);
+	      break;
+	    case W_GLOBALS:
+	      infos_name = GET_GLOBAL_INFO(*names);
+	      break;
 	    }
 	    InsertNchar ( bwin -> buf, infos_name, strlen ( infos_name ));
 	    InsertNchar ( bwin -> buf, "\n", 1 );
-	    names ++;
+	  }
+	  names ++;
 	}
 	if ( type == W_FILES )
 	  SetFileBuf ( bwin -> buf );
@@ -733,12 +758,15 @@ void ConfigBrowser ( width, height )
     XResizeWindow ( dpy, br.main, br. width, br.height - br.title_height );
     
     ConfigFiles (); 
-    if ( strcmp ( br.mode -> name, "C++mode" ) == 0 ) {
+    if ( (strcmp ( br.mode -> name, "C++mode" ) == 0)
+	|| (strcmp ( br.mode -> name, "Java" ) == 0)) {
 	ConfigClass ();
 	ConfigParent ();
 	ConfigChild ();
 	ConfigMethod ();
 	ConfigProc ();
+	ConfigAttributes ();
+	ConfigGlobals ();
     }
     else if ( strcmp ( br.mode -> name, "C-mode" ) == 0 ) {
 	ConfigProc ();
@@ -752,11 +780,15 @@ void ConfigBrowser ( width, height )
     y = GetBWheight(W_FILES) + ( 2 * B_INTER);
     
     XMoveWindow ( dpy, br.close, B_INTER, y );
-    XMoveWindow ( dpy, br.rec, B_INTER + 2 + br.button_width, y );
-    XMoveWindow ( dpy, br.add, B_INTER + ( 2 * (2 + br.button_width)), y );
-    XMoveWindow ( dpy, br.del, B_INTER + ( 3 * (2 + br.button_width)), y );
-    XMoveWindow ( dpy, br.dec,B_INTER + ( 4 * (2 + br.button_width)) , y );
-    XMoveWindow ( dpy, br.edit,B_INTER + ( 5 * (2 + br.button_width)) , y );
+    XMoveWindow ( dpy, br.dump, B_INTER + 2 + br.button_width, y );
+    XMoveWindow ( dpy, br.restore, B_INTER + ( 2 * (2 + br.button_width)), y );
+    XMoveWindow ( dpy, br.rec, B_INTER + ( 3 * (2 + br.button_width)), y );
+    XMoveWindow ( dpy, br.add, B_INTER + ( 4 * (2 + br.button_width)), y );
+    XMoveWindow ( dpy, br.del, B_INTER + ( 5 * (2 + br.button_width)), y );
+    XMoveWindow ( dpy, br.dec,B_INTER + ( 6 * (2 + br.button_width)) , y );
+    XMoveWindow ( dpy, br.pp,B_INTER + ( 7 * (2 + br.button_width)) , y );
+    XMoveWindow ( dpy, br.hide,B_INTER + ( 8 * (2 + br.button_width)) , y );
+    XMoveWindow ( dpy, br.edit,B_INTER + ( 9 * (2 + br.button_width)) , y );
 }
 
 
@@ -806,7 +838,7 @@ static void ConfigClass ()
     bwin -> x = B_INTER;
     bwin -> y = B_INTER;
     bwin -> f_width = (br.width - ( 4*B_INTER )) / 3;
-    bwin -> f_height = (br.height - ( 4*B_INTER ) - (2*br.title_height)) / 2;
+    bwin -> f_height = GetBWheight(W_FILES);
     
     XResizeWindow ( dpy, bwin -> frame, bwin -> f_width, bwin -> f_height );
     XResizeWindow ( dpy, bwin -> title, bwin -> f_width, br.title_height ); 
@@ -890,9 +922,17 @@ static void ConfigFiles ()
 {
     BWin *bwin = &br.tbw[W_FILES];
     
-    if ( strcmp ( br.mode -> name, "C++mode" ) == 0 ) {
+    if (strcmp ( br.mode -> name, "C++mode" ) == 0){
 	bwin -> f_width = (br.width - ( 4*B_INTER )) / 3;
-	bwin -> f_height = (br.height - ( 4*B_INTER ) - (2*br.title_height)) / 2;
+	bwin -> f_height = 
+	  (br.height - 5 * B_INTER - br.title_height - br.button_height) / 3;
+	bwin -> x = ( 3 * B_INTER ) + ( (br.width - ( 4*B_INTER )) *2/ 3);
+	bwin -> y = B_INTER;
+    }
+    else if ( (strcmp ( br.mode -> name, "Java" ) == 0)) {
+	bwin -> f_width = (br.width - ( 4*B_INTER )) / 3;
+	bwin -> f_height = 
+	  (br.height - (4 * B_INTER) - br.title_height - br.button_height) / 2;
 	bwin -> x = ( 3 * B_INTER ) + ( (br.width - ( 4*B_INTER )) *2/ 3);
 	bwin -> y = B_INTER;
     }
@@ -934,8 +974,8 @@ static void ConfigMethod ()
     bwin -> x = B_INTER;
     bwin -> y = GetBWy(W_CLASS) + GetBWheight(W_CLASS)
       + br.button_height + ( 2 * B_INTER );
-    bwin -> f_width = ( br.width - (3 * B_INTER)) / 2;
-    bwin -> f_height = GetBWheight(W_CLASS);
+    bwin -> f_width = br.width - (3 * B_INTER) - GetBWwidth(W_FILES);
+    bwin -> f_height = GetBWheight(W_FILES);
     
     XMoveResizeWindow ( dpy, bwin -> frame, 
 		       bwin -> x, bwin -> y,
@@ -963,13 +1003,13 @@ static void ConfigProc ()
 {
     BWin *bwin = &br.tbw[W_PROC];
     
-    if ( strcmp ( br.mode -> name, "C++mode" ) == 0 ) {
-	bwin -> x = (2 *B_INTER) + GetBWwidth(W_METHOD);
-	bwin -> y = GetBWy(W_CLASS) + GetBWheight(W_CLASS)
-	  + br.button_height + ( 2 * B_INTER );
+    if ( (strcmp ( br.mode -> name, "C++mode" ) == 0)
+	|| (strcmp ( br.mode -> name, "Java" ) == 0)) {
+	bwin -> x = B_INTER;
+	bwin -> y = GetBWy(W_METHOD) + GetBWheight(W_METHOD) + B_INTER;
 	bwin -> f_width = GetBWwidth(W_METHOD);
-	bwin -> f_height = GetBWheight(W_METHOD);
-	bwin -> m_width = GetBWmwidth(W_METHOD);
+	bwin -> f_height = GetBWheight(W_FILES);
+	bwin -> m_width = bwin -> f_width - ( 2 * B_SPACE );
     }
     else if ( strcmp ( br.mode -> name, "C-mode" ) == 0 ){
 	bwin -> x = (2 *B_INTER) + GetBWwidth(W_FILES);
@@ -995,6 +1035,67 @@ static void ConfigProc ()
 }
 
 
+/*
+**	Function name : ConfigAttributes
+**
+**	Description : Postionne la geopmetrie de la fenetre
+**		des attributs.
+**	Input : Le contexte.
+**	Ouput :
+*/
+static void ConfigAttributes ()
+{
+    BWin *bwin = &br.tbw[W_ATTRIBUTES];
+    
+    bwin -> x = GetBWx(W_FILES);
+    bwin -> y = GetBWy(W_METHOD);
+    bwin -> f_width = GetBWwidth(W_FILES);
+    bwin -> f_height = GetBWheight(W_METHOD);
+    bwin -> m_width = bwin -> f_width - ( 2 * B_SPACE );
+    
+    XMoveResizeWindow ( dpy, bwin -> frame,
+		       bwin -> x, bwin -> y,
+		       bwin -> f_width, bwin -> f_height );
+    
+    XResizeWindow ( dpy, bwin -> title, bwin -> f_width, br.title_height ); 
+    
+    bwin -> m_height = bwin -> f_height - br.title_height - (2*B_SPACE);
+    XResizeWindow ( dpy, bwin -> main, bwin -> m_width, bwin -> m_height );
+    
+    ConfigScroll ( &br.tbw[W_ATTRIBUTES] );
+}
+
+/*
+**	Function name : ConfigGlobals
+**
+**	Description : Postionne la geopmetrie de la fenetre
+**		des attributs.
+**	Input : Le contexte.
+**	Ouput :
+*/
+static void ConfigGlobals ()
+{
+    BWin *bwin = &br.tbw[W_GLOBALS];
+    
+    bwin -> x = GetBWx(W_ATTRIBUTES);
+    bwin -> y = GetBWy(W_PROC);
+    bwin -> f_width = GetBWwidth(W_ATTRIBUTES);
+    bwin -> f_height = GetBWheight(W_PROC);
+    bwin -> m_width = bwin -> f_width - ( 2 * B_SPACE );
+    
+    XMoveResizeWindow ( dpy, bwin -> frame,
+		       bwin -> x, bwin -> y,
+		       bwin -> f_width, bwin -> f_height );
+    
+    XResizeWindow ( dpy, bwin -> title, bwin -> f_width, br.title_height ); 
+    
+    bwin -> m_height = bwin -> f_height - br.title_height - (2*B_SPACE);
+    XResizeWindow ( dpy, bwin -> main, bwin -> m_width, bwin -> m_height );
+    
+    ConfigScroll ( &br.tbw[W_GLOBALS] );
+}
+
+
 /*
 **	Function name : Config Scroll
 **
@@ -1037,7 +1138,8 @@ void DisplayBrowser()
     if ( strcmp ( br.mode -> name, "default" ) == 0 )
       br.mode = (Mode *) GetMode ("C++mode");
     
-    if ( strcmp ( br.mode -> name, "C++mode" ) == 0 ) {
+    if ( (strcmp ( br.mode -> name, "C++mode" ) == 0)
+	|| (strcmp ( br.mode -> name, "Java" ) == 0)) {
 	i = 0;
     }
     else if ( strcmp ( br.mode -> name, "C-mode" ) == 0 ){
@@ -1054,30 +1156,27 @@ void DisplayBrowser()
     XMapWindow ( dpy, br.del );
     XMapWindow ( dpy, br.dec );      
     XMapWindow ( dpy, br.edit );
+    XMapWindow ( dpy, br.pp );
+    XMapWindow ( dpy, br.dump );
+    XMapWindow ( dpy, br.restore );
+    XMapWindow ( dpy, br.hide );
     XMapRaised ( dpy, br.frame );
     
     for ( ;i < B_NB_WIN; i++ ) { 
-	XMapSubwindows ( dpy, br.tbw[i].main ); 
-	XMapSubwindows ( dpy, br.tbw[i].frame ); 
-	XMapWindow ( dpy, br.tbw[i].main ); 
-	XMapWindow ( dpy, br.tbw[i].frame ); 
-	/*
-	   * Plus tard les explications de ce 'FirstPage'
-	*/
-	if ( (br.tbw[i].text -> lines_in_buf > 1) &&(already_map == True)) {
-	    FirstPage ( br.tbw[i].text );
-	}	
-	/*
-	   XMapWindow ( dpy, br.main );
-	   XMapWindow ( dpy, br.title );
-	   XMapWindow ( dpy, br.close );
-	   XMapWindow ( dpy, br.dec );
-	   XMapWindow ( dpy, br.add );
-	   XMapWindow ( dpy, br.del );
-	   XMapWindow ( dpy, br.dec );      
-	   XMapWindow ( dpy, br.edit );
-	   XMapWindow ( dpy, br.frame );
-	*/    
+      if ((strcmp ( br.mode -> name, "Java" ) == 0)) {
+	if ((i == W_GLOBALS) || (i == W_PROC))
+	  continue;
+      }
+      XMapSubwindows ( dpy, br.tbw[i].main ); 
+      XMapSubwindows ( dpy, br.tbw[i].frame ); 
+      XMapWindow ( dpy, br.tbw[i].main ); 
+      XMapWindow ( dpy, br.tbw[i].frame ); 
+      /*
+	 * Plus tard les explications de ce 'FirstPage'
+      */
+      if ( (br.tbw[i].text -> lines_in_buf > 1) &&(already_map == True)) {
+	FirstPage ( br.tbw[i].text );
+      }	
     }
   
     br.stat = B_MAP;
@@ -1085,6 +1184,15 @@ void DisplayBrowser()
       already_map = True;	
 }
 
+void DisplayBrowserIfNeverMap()
+{
+  if (! already_map) {
+    DisplayBrowser();
+    WaitForMapped ( br.frame, False );
+  }
+}
+
+
 /*
 **	Function name : UnmapBrowser
 **
@@ -1101,7 +1209,6 @@ void UnmapBrowser ()
     XUnmapWindow ( dpy, br.main );
     XUnmapSubwindows ( dpy, br.frame );
     XUnmapWindow ( dpy, br.frame );
-    
     XUnmapWindow ( dpy, br.tbw[W_VISIT].frame ); 
     br.stat = B_UNMAP;
 }
@@ -1118,7 +1225,8 @@ void SetBrowserMode ( mode )
     Mode *mode;
 {
     if ( (strcmp (mode -> name, "C-mode") == 0 )
-	|| (strcmp (mode -> name, "C++mode") == 0 ))
+	|| (strcmp (mode -> name, "C++mode") == 0 )
+	|| (strcmp (mode -> name, "Java") == 0 ))
       br.mode = mode;
 }
 
