@@ -88,6 +88,20 @@
   }                                                                    \
 }
 
+#define assoc_Rec(name, TypeRec, head, current)  {                     \
+  current = *head;                                                     \
+  while (current != Null) {                                            \
+    if (current->_name == name)                                        \
+      break;                                                           \
+    else                                                               \
+      current = current->_next;                                        \
+  }                                                                    \
+}
+
+
+#define iter_Rec(name, TypeRec, head, current)                         \
+  for (current = *head; current != Null; current = current->_next)     \
+    if (strcmp(current->_name, name) == 0)
 
 #define extract_Rec(name, TypeRec, head, current)  {                   \
   TypeRec* previous;                                                   \
