@@ -55,9 +55,10 @@
 #include "mark_reg.h"
 #include "page.h"
 #include "ctr_version.h"
+#include "main_events.h"
 
 char *menu_names[] = { 
-	"File", "Window", "Modes", "Search", "Region", "Version", "Font", "Misc", 0
+	"File", "Window", "Edit", "Modes", "Search", "Region", "Font", "Misc", 0
 };
 
 static char *file_menu [] = {
@@ -66,6 +67,8 @@ static char *file_menu [] = {
 	"Insert file     ^x i",
 	"Save file       ^x^s",
 	"Write file as   ^x^w",
+	"Browser dump    ^xd",
+	"Browser restore ^xr",
 	"Quit            ^x^c",
 	0
 };
@@ -73,22 +76,34 @@ static char *file_menu [] = {
 static char *window_menu [] = {
 	"New text window     ^x n",
 	"Delete text window  ^x^c",
-	"Display kill buffer",
-	"Display open files  ^x b",
-	"Display browser",
-	"Display manual",
+	"Open files window   ^x b",
+	"Browser window",
+	"Directory window    ^x e",
+	"Manual window",
 	0
 };
 
+static char *edit_menu [] = {
+  "Undo           ^u",
+  "Redo           ^t",
+  "Cut selection  ^Btn2",
+  "Paste selection Bnt2",
+  "Display kill buffer",
+  0
+};
+
 static char *mode_menu [] = {
 	"Select default mode",
 	"Select C mode",
 	"Select C++ mode",
+	"Select Java mode",
 	"Select Latex mode",
 	"Select Html mode",
-	"Select Perl mode",
+
+        "Select Perl mode",
 	"Select Ada mode",
 	"Select Fortran mode",
+	
 	"Select shell-script mode",
 	"Execute a sub Shell",
 	"Select others ...",
@@ -112,12 +127,12 @@ static char *mark_menu [] = {
 	"Set mark              ^space",
 	"Goto the mark           ^x m",
 	"Exchange point mark     ^x^x",
-	"Kill region               ^w",
+	"Delete region             ^w",
 	"Copy region            Esc w",
 	"Paste region              ^y",
 	"Indent region          Esc i",
 	"Eval region            Esc e",
-	"Color buffer                ",
+	"Color buffer            ^x a",
 	"Color region                ",
 	"Current line to top    Esc t",
 	0
@@ -157,46 +172,45 @@ static char *font_menu [] = {
 };
 
 static char *font_names [] = {
-	"-adobe-courier-bold-r-normal--*-100-*-*-m-*-iso8859-1",  
-	"-adobe-courier-bold-r-normal--*-120-*-*-m-*-iso8859-1",
-	"-adobe-courier-bold-r-normal--*-140-*-*-m-*-iso8859-1",
+        "-adobe-courier-bold-r-normal--10-100-75-75-m-60-iso8859-1",  
+        "-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1",
+        "-adobe-courier-bold-r-normal--14-140-75-75-m-90-iso8859-1",
 	"-adobe-courier-medium-r-normal--10-100-75-75-m-60-iso8859-1",
-	"-adobe-courier-medium-r-normal--*-120-*-*-m-*-iso8859-1",
-	"-adobe-courier-medium-r-normal--*-140-*-*-m-*-iso8859-1",
-	"-adobe-courier-medium-r-normal--*-180-*-*-m-*-iso8859-1",
-	"-adobe-helvetica-medium-r-normal--*-100-*-*-p-*-iso8859-1",
-	"-adobe-helvetica-medium-r-normal--*-140-*-*-p-*-iso8859-1",
-	"-adobe-times-bold-r-normal--*-140-*-*-p-*-iso8859-1",
-	"-adobe-times-medium-r-normal--*-180-*-*-p-*-iso8859-1",
-	"-adobe-times-medium-r-normal--*-200-*-*-p-*-iso8859-1",
-	"-adobe-times-medium-r-normal--*-240-*-*-p-*-iso8859-1",
-	"-adobe-new century schoolbook-medium-r-normal--*-180-*-*-p-*-iso8859-1",
+        "-adobe-courier-medium-r-normal--12-120-75-75-m-70-iso8859-1",
+        "-adobe-courier-medium-r-normal--14-140-75-75-m-90-iso8859-1",
+        "-adobe-courier-medium-r-normal--18-180-75-75-m-110-iso8859-1",
+        "-adobe-helvetica-medium-r-normal--14-100-100-100-p-76-iso8859-1",
+        "-adobe-helvetica-medium-r-normal--20-140-100-100-p-100-iso8859-1",
+        "-adobe-times-bold-r-normal--14-140-75-75-p-77-iso8859-1",
+        "-adobe-times-medium-r-normal--18-180-75-75-p-94-iso8859-1",
+        "-adobe-times-medium-r-normal--20-140-100-100-p-100-iso8859-1",
+        "-adobe-times-medium-r-normal--24-240-75-75-p-132-iso8859-1",
+        "-adobe-new century schoolbook-medium-r-normal--18-180-75-75-p-103-iso8859-1",
 	0
 };
 
 static char *misc_menu [] = {
-	"New C++ class",
-	"New C++ method",
-	"New C function",
-	"New include header",
-	"Latex style and macros",
-	"Html macros",
-	"Misc commands",
-	"User commands",
-	"Load and eval file",
-	"Eval expression ^x^e",
-	"Xcoral release",
-	"Help",
-	0
+  "Programming util...",
+  "Rcs commands...",
+  "Latex commands...",
+  "Html commands...",
+  "Misc commands...",
+  "User commands...",
+  "Load and eval file",
+  "Eval expression ^x^e",
+  "Hanoi towers demo",
+  "Xcoral release",
+  "Help",
+0
 };
 
 char **item_names [] = {
 	file_menu,
 	window_menu,
+	edit_menu,
 	mode_menu,
 	search_menu,
 	mark_menu,
-	version_menu,
 	font_menu,
 	misc_menu,
 	0
@@ -208,6 +222,8 @@ static void (*f_file []) () = {
 	MenuInsertFile,
 	MenuSaveFile,
 	MenuWriteFile,
+	MenuBrowserDump,
+	MenuBrowserRestore,
 	(void (*)()) DeleteWindow, 
 	0
 };
@@ -215,22 +231,34 @@ static void (*f_file []) () = {
 static void (*f_window []) () = { 
 	NewWindow,
 	(void (*)()) DeleteWindow,
-	DisplayKillBuffer,
 	DisplayOpenFiles,
 	DisplayBrowser,
+	EdirWindow,
 	DisplayManBox,
 	0
 };
 
+static void (*f_edit []) () = {
+  DoUndo,
+  DoUndoUndo,
+  DeleteBytesFromCutBuffer,
+  PasteSelection,
+  DisplayKillBuffer,
+  0
+};
+
 static void (*f_mode []) () = {
 	SetDefaultMode,
 	SetCMode,
 	SetCCMode,
+	SetJavaMode,
 	SetLatexMode,
 	SetHtmlMode,
-	SetPerlMode,
+
+        SetPerlMode,
 	SetAdaMode,
 	SetFortranMode,
+	
 	SetshellMode,
 	SetShellMode,
 	LoadMode,
@@ -281,27 +309,30 @@ static void (*f_version []) () = {
 };
 
 static void (*f_font []) () = { 
-	0
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0
 };
 
 static void (*f_misc []) () = {
-	ClassHeader,
-	MethodHeader,
-	FunctionHeader,
-	IncludeHeader,
-	LatexMacros,
-	HtmlMacros,
-	MiscCommands,
-	UserCommands,
-	LoadAndEvalFile,
-	EvalExpressionFromMenu,
-	Version,
-	DisplayManBox,
-	0
+  ProgrammingUtil,
+  RcsCommands,
+  LatexMacros,
+  HtmlMacros,
+  MiscCommands,
+  UserCommands,
+  LoadAndEvalFile,
+  EvalExpressionFromMenu,
+  Hanoi,  
+  Version,
+  DisplayManBox,
+  0
 };
 
 void (**func_names []) () = {
-	f_file, f_window, f_mode, f_search, f_mark, f_version, f_font, f_misc, 0
+/*  
+	f_file, f_window, f_edit, f_mode, f_search, f_mark, f_version, f_font, f_misc, 0
+*/
+	f_file, f_window, f_edit, f_mode, f_search, f_mark, f_font, f_misc, 0
+  
 };
 
 #define M_FONTS 6
