@@ -45,16 +45,41 @@
 #include <X11/Xatom.h>
 #include <X11/keysym.h>
 #include <ctype.h>
-#include <string.h>
-#include <sys/time.h>
 #include <signal.h>
+#include <sys/types.h>
 
-#ifndef apollo
-#ifdef __FreeBSD__
-#include <stdlib.h>
-#else /* !__FreeBSD__ */
-#include <malloc.h>
+#if HAVE_STRING_H
+#include <string.h>
+#else
+#include <strings.h>
 #endif
+
+#if STDC_HEADERS
+#  include <stdlib.h>
+#else
+#  if HAVE_MALLOC_H
+#    include <malloc.h>
+#  endif
+#endif
+
+#if HAVE_UNISTD_H
+#  include <unistd.h>
+#endif
+
+#if HAVE_POLL
+#  include <stropts.h>
+#  include <poll.h>
+#endif
+
+#if TIME_WITH_SYS_TIME
+#  include <sys/time.h>
+#  include <time.h>
+#else
+#  if HAVE_SYS_TIME_H
+#    include <sys/time.h>
+#  else
+#    include <time.h>
+#  endif
 #endif
 
 #include "main_text.h"
@@ -66,12 +91,16 @@
 #include "list_box.h"
 #include "ie_func.h"
 #include "get_file.h"
+#include "selection.h"
 
 extern Display *dpy;
 static int toggle_del = 0;
 extern char *ie_call_function();
 
-FCT (static void, donothing, () );
+FCT (static RETSIGTYPE, donothing, () );
+
+static char *cut_buffer = 0;
+static int n_cb = 0;
 
 /*
 **	Function name : CheckModifAndFilename
@@ -104,6 +133,9 @@ void f_impc ( text, ch )
     int ch;
 {
   char c = (char) ch;
+
+  if (text -> selection -> select == True)
+    ClearSelection(text, True);
   
   TextCursorOff ( text );
   (void) UpdateMark ( text, 1, 0 );
@@ -134,8 +166,10 @@ void f_impc ( text, ch )
 void f_tab ( text )
     Text *text;
 {
+  if (text -> selection -> select == True)
+    ClearSelection(text, True);
+
   TextCursorOff ( text );
-/*  CheckModifAndFilename ( text ); */
   
   UpdateMark ( text, 1, 0 );	
   InsertNchar ( text -> buf, "\t", 1 );
@@ -165,8 +199,10 @@ void f_tab ( text )
 void f_return ( text )
     Text *text;
 {
+  if (text -> selection -> select == True)
+    ClearSelection(text, True);
+
   TextCursorOff ( text );
-/*  CheckModifAndFilename ( text ); */
   
   if ( text -> n2 == 0 ) 
     CurrentLineToMiddle ( text );
@@ -234,6 +270,15 @@ void f_delete ( text )
     klaxon ();
     return;
   }
+/*  
+  if (text -> selection -> select == True)
+    ClearSelection(text, True);
+*/
+  if (text -> selection -> select == True) {
+     DeleteBytesFromCutBuffer (text);
+     return;
+  }
+  
   TextCursorOff ( text );
   GetPrevChar ( text -> buf, &c );
   
@@ -325,6 +370,9 @@ void Control_K ( text, n )
     return;
   }
   
+  if (text -> selection -> select == True)
+    ClearSelection(text, True);
+
   mark = UpdateMark ( text, 0, -(n -1) );
   p = (char *) DeleteLines ( text -> buf, n, &len, &dn );
   if ( mark ) 
@@ -375,6 +423,9 @@ void Control_Y ( text, i )
   char *p;
   int len, dn;
   
+  if (text -> selection -> select == True)
+    ClearSelection(text, True);
+
   TextCursorOff ( text );
   p = (char *) RestoreKillBuf ( i, &len, &dn );
   
@@ -461,8 +512,11 @@ void GetBytesFromCutBuffer ( text )
   char *s;
   int nbytes;
   
-  s = XFetchBuffer ( dpy, &nbytes, 0 );
+  s = Fetch_bytes (&nbytes);
   
+  if (text -> selection -> select == True)
+    ClearSelection(text, True);
+
   if ( (s != 0) && (nbytes != 0 )) {
     TextCursorOff ( text );
     ClipOn ( text, 0 );
@@ -542,39 +596,35 @@ void StoreBytesInCutBuffer ( text, x, y )
     Text *text;
     int x, y;
 {
-  char *old_s = 0;
-  int old_n = 0;
-  int old_x = text -> x_pos;
-  int old_y = text -> y_pos;
   char *old_left = (char *) LeftBuf ( text -> buf );
   char *old_right = (char *) RightBuf ( text -> buf );
   char *new_left, *new_right;
+
+  StartSelection(text);
+
+  StorePosition ( text );
+  if ( MoveToXYinTextWindow ( text, x, y ) == -1 ) {
+    HoleToRight ( text -> buf );
+    GotoLineNumber ( text, text -> lines_in_buf );
+    SetPosition ( text );
+    UpdatePage ( text );
+  }
   
-  (void) MoveToXYinTextWindow ( text, x, y );
+/*  (void) MoveToXYinTextWindow ( text, x, y ); */
+
   TextCursorOn ( text );
   XSync ( dpy, False );
-  
-  /* Juste pour voir si ya deja qqch. 
-     Si oui, on libere */
-  old_s = XFetchBytes ( dpy, &old_n );
-  if ( old_s != 0 )
-    (void) XFree ( old_s );
-  
+  EndSelection(text);
+
   new_left = (char *) LeftBuf ( text -> buf );
   new_right = (char *) RightBuf ( text -> buf );
 
   if ( new_left > old_left )  { /* Deplacement vers la droite du curseur */
-	XStoreBytes ( dpy, old_left +1, new_left - old_left );
+    Store_bytes ( old_left +1, new_left - old_left );
   }
   else {
-	XStoreBytes ( dpy, new_right, old_right - new_right );
+    Store_bytes ( new_right, old_right - new_right );
   }
-  toggle_del = False;
-  SmallTime ( (long) 200000 );	/* 0.20 secondes */
-  TextCursorOff ( text );
-  (void) MoveToXYinTextWindow ( text, old_x, old_y );
-  
-/*  XSetSelectionOwner (dpy, XA_PRIMARY, text -> window, CurrentTime ); */
 }
 
 
@@ -592,12 +642,55 @@ void DeleteBytesFromCutBuffer ( text )
   char *s, *tmp = 0;
   int mark = 0;
   int nbytes;
+  int markline = 0, markpos = 0;
   
-  s = XFetchBytes ( dpy, &nbytes );
+  s = Fetch_bytes (&nbytes);
+
+  if (text -> selection -> select == True)
+    ClearSelection(text, True);
+  else
+    return;
   
-  if ( (s != 0) && (nbytes != 0 ) && (toggle_del == False) ) {
-    if ( (text -> modif == False) && (strcmp(text -> filename,"NoName")==0) ) {
+  if ( (s != 0) && (nbytes != 0 )) {
+    if ( (text -> modif == False) && (strcmp(text -> filename,"NoName")==0) )
       return;
+
+    /* On sauve la marque si yen a une */
+    if (text -> markline) {
+      markline = text -> markline;
+      markpos = text -> markpos;
+    }
+    
+    /* On positionne une marque sur le debut de la selection */
+    if (text -> selection -> start_line < text -> selection -> end_line) {
+      text -> markline = text -> selection -> start_line;
+      text -> markpos = text -> selection -> ncfl_sl;
+    }
+    else if (text -> selection -> start_line > text -> selection -> end_line){
+      text -> markline = text -> selection -> end_line;
+      text -> markpos = text -> selection -> ncfl_el;
+    }
+    else {
+      /* selection sur une ligne; */
+      if (text -> selection -> ncfl_sl < text -> selection -> ncfl_el) {
+	text -> markline = text -> selection -> start_line;
+	text -> markpos = text -> selection -> ncfl_sl;
+      }
+      else {
+	text -> markline = text -> selection -> start_line;
+	text -> markpos = text -> selection -> ncfl_el;
+      }
+    }
+    
+    (void) GotoTheMark (text);
+
+    if(markline) {
+      text -> markline = markline;
+      text -> markpos = markpos;
+    }
+    else {
+      text -> markline = 0;
+      text -> markpos = 0;
     }
 
     if ( (RightBuf ( text -> buf ) + nbytes) > BottomBuf ( text -> buf ) ) {
@@ -650,7 +743,7 @@ void DeleteBytesFromCutBuffer ( text )
   }
 }
 static int timer_expire = False;
-static void donothing () { timer_expire = True; }
+static RETSIGTYPE donothing () { timer_expire = True; }
 
 
 /*
@@ -663,37 +756,28 @@ static void donothing () { timer_expire = True; }
 void SmallTime ( t )
     long t; /* micros-secondes */
 {
-  struct itimerval value;
-#if defined(SVR4) || defined(UNIXWARE)  
+#if HAVE_USLEEP
   usleep(t);
-#else
-#ifdef NOTDEF
-  value.it_interval.tv_sec = 0;
-  value.it_interval.tv_usec = 0;
-  value.it_value.tv_sec = 0;
-  value.it_value.tv_usec = t;
 
-  timer_expire = False;
-  (void) signal ( SIGALRM, (void (*) ()) donothing );
-  (void) setitimer ( ITIMER_REAL, &value, (struct itimerval *) 0 );
-  while(1) {
-    sigblock(sigmask(SIGALRM));
-    if ( timer_expire )
-      break;
-    else
-      sigpause(0);
-  }
-  sigblock(0);
-  (void) signal ( SIGALRM, SIG_IGN );
-#else
-	/* best way to do it... portable, too... */
-  struct timeval timeout;
+#elif HAVE_SELECT
+  struct timeval delay;
 
-  timeout.tv_sec = 0;
-  timeout.tv_usec = t;
+  delay.tv_sec  = 0;
+  delay.tv_usec = t;
+  select(0, NULL, NULL, NULL, &delay);
+
+#elif HAVE_NANOSLEEP
+  struct timespec rqtp;
+
+  rqtp.tv_sec  = t / (unsigned long)  1000000;
+  rqtp.tv_nsec = (t % (unsigned long) 1000000) * 1000 ;
+  nanosleep(&rqtp, NULL);
+
+#else  /* Use default poll */
+  struct pollfd unused;
+
+  poll(&unused,0,(t / 1000)); 
 
-  select(0, NULL, NULL, NULL, &timeout);
-#endif
 #endif
 }
 
@@ -783,6 +867,59 @@ void MethodHeader ( text )
   ie_redisplay ( text );
 }
 
+/*
+**	Function name : ProgrammingUtil
+**
+**	Description :
+**	Input :
+**	Output :
+*/
+void ProgrammingUtil(text)
+    Text *text;
+{
+  char *msg;
+  int nw = text->win_id;
+
+  StorePosition ( text );
+  msg = (char *) ie_call_function ( text, "add_header", 0, 0) ;
+  text = update_cwd(nw);
+  if ( msg != 0 )
+    DisplayMessage ( text -> mwin, msg );
+  ie_redisplay ( text );
+  if(TextInBuf(text)==False){
+     TextCursorOff(text);
+     ClearPage(text);
+     TextCursorOn(text);
+  }  
+}
+
+/*
+**	Function name : RcsCommands
+**
+**	Description :
+**	Input :
+**	Output :
+*/
+void RcsCommands(text)
+    Text *text;
+{
+  char *msg;
+  int nw = text->win_id;
+
+  StorePosition ( text );
+  msg = (char *) ie_call_function ( text, "rcs_commands", 0, 0) ;
+  text = update_cwd(nw);
+  if ( msg != 0 )
+    DisplayMessage ( text -> mwin, msg );
+  ie_redisplay ( text );
+  if(TextInBuf(text)==False){
+     TextCursorOff(text);
+     ClearPage(text);
+     TextCursorOn(text);
+  }  
+}
+
+
 /*
 **	Function name : LatexMacros
 **
@@ -802,6 +939,11 @@ void LatexMacros(text)
   if ( msg != 0 )
     DisplayMessage ( text -> mwin, msg );
   ie_redisplay ( text );
+  if(TextInBuf(text)==False){
+     TextCursorOff(text);
+     ClearPage(text);
+     TextCursorOn(text);
+  }  
 }
 
 /*
@@ -823,6 +965,11 @@ void HtmlMacros(text)
   if ( msg != 0 )
     DisplayMessage ( text -> mwin, msg );
   ie_redisplay ( text );
+  if(TextInBuf(text)==False){
+     TextCursorOff(text);
+     ClearPage(text);
+     TextCursorOn(text);
+  }  
 }
 
 /*
@@ -843,7 +990,12 @@ void MiscCommands(text)
   text = update_cwd(nw);
   if ( msg != 0 )
     DisplayMessage ( text -> mwin, msg );
-  ie_redisplay ( text );
+  ie_redisplay (text);
+  if(TextInBuf(text)==False){
+     TextCursorOff(text);
+     ClearPage(text);
+     TextCursorOn(text);
+  }  
 }
 
 /*
@@ -865,6 +1017,37 @@ void UserCommands(text)
   if ( msg != 0 )
     DisplayMessage ( text -> mwin, msg );
   ie_redisplay ( text );
+  if(TextInBuf(text)==False){
+     TextCursorOff(text);
+     ClearPage(text);
+     TextCursorOn(text);
+  }  
+}
+
+/*
+**	Function name : EdirWindow
+**
+**	Description :
+**	Input :
+**	Output :
+*/
+void EdirWindow(text)
+    Text *text;
+{
+  char *msg;
+  int nw = text->win_id;
+
+  StorePosition ( text );
+  msg = (char *) ie_call_function ( text, "edir", 0, 0) ;
+  text = update_cwd(nw);
+  if ( msg != 0 )
+    DisplayMessage ( text -> mwin, msg );
+  ie_redisplay ( text );
+  if(TextInBuf(text)==False){
+     TextCursorOff(text);
+     ClearPage(text);
+     TextCursorOn(text);
+  }  
 }
 
 
@@ -936,72 +1119,54 @@ void QuotedChar( text )
 	}
     }
 }
-#ifdef SVR4
+
 /*
- *      usleep(delay)  --
- *
- *      Possible usleep replacement. Delay in microseconds.
- *      Another possiblity is to use poll(2). On Solaris
- *      2.x, select is just a wrapper for poll, so you
- *      are better off using it directly.  If you use,
- *      poll, note that it uses millisecond resolution,
- *      and is not affected by the O_NDELAY and O_NONBLOCK 
- *      flags.
- *
- *      Note that using nanosleep has portability implications,
- *      even across different versions of Solaris 2.x. In
- *      particular, only Solaris 2.3 has libposix4, and 
- *      hence nanosleep. Select (or poll) is a better option if 
- *      you need portability across those versions.
- *
- *      If you define USE_NANOSLEEP, be sure to link with -lposix4
- *
- *      This function is taken from the Solaris 2.x Porting FAQ
- */
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <errno.h>
-#include <time.h>
-#include <sys/time.h>
-#include <sys/param.h>
-#include <sys/types.h>
-#ifdef  USE_POLL
-#include <stropts.h>
-#include <poll.h>
-#endif  /* USE_POLL */
-
-int usleep(useconds)
-    unsigned long int useconds;
+**	Function name : Store_bytes
+**
+**	Description :
+**	Input :
+**	Output :
+*/
+void Store_bytes(s, n)
+    char *s; int n;
 {
-#ifdef  USE_NANOSLEEP
-  struct timespec rqtp;
-
-  rqtp.tv_sec  = useconds / (unsigned long)  1000000;
-  rqtp.tv_nsec = (useconds % (unsigned long) 1000000) * 1000 ;
-
-  if (nanosleep(&rqtp,  (struct  timespec *) NULL) == -1)
-    perror("nanosleep");
-  return (0);
-
-#elif   USE_POLL
-  struct pollfd unused;
-
-  if (poll(&unused,0,(useconds/1000)) == -1) 
-    perror("poll");
-  return(0);
-#else  /* default to select */
-  struct timeval delay;
-
-  delay.tv_sec = 0;
-  delay.tv_usec = useconds;
-  if (select(0,
-             (fd_set *) NULL,
-             (fd_set *) NULL,
-             (fd_set *) NULL,
-             &delay) == -1)
-    perror("select");
-  return (0);
-#endif /* USE_NANOSLEEP */
+  Clear_bytes();
+  
+  cut_buffer = (char *) malloc (n+2);
+  bzero (cut_buffer, n+2);
+  strncpy (cut_buffer, s, n);
+  n_cb = n;
 }
-#endif /* SVR4 */
+
+/*
+**	Function name : Fetch_bytes
+**
+**	Description :
+**	Input :
+**	Output :
+*/
+char *Fetch_bytes(n)
+    int *n;
+{
+  *n = n_cb;
+  return cut_buffer;
+}
+
+/*
+**	Function name : Clear_bytes
+**
+**	Description :
+**	Input :
+**	Output :
+*/
+void Clear_bytes()
+{
+  if (cut_buffer) {
+    free (cut_buffer);
+    n_cb = 0;
+    cut_buffer = 0;
+  }
+}
+
+
+
