@@ -38,12 +38,16 @@
    ######################################################################## */
 
 
+#if HAVE_STRING_H
+#include <string.h>
+#else
+#include <strings.h>
+#endif
+
 #include "result_types.h"
 #include "file_dict.h"
 #include "class_dict.h"
 #include "browser_util.h"
-#include <string.h>
-#include <stdio.h>
 
 
 /*------------------------------------------------------------------------------
@@ -58,14 +62,15 @@ int        class_count = 0;
 
 /*------------------------------------------------------------------------------
 */
-ClassRec* create_class (class_name)
+ClassRec* create_class (class_name, decl)
   char* class_name;
+  unsigned decl;
 {
-	ClassRec**  head;
+  ClassRec**  head;
   ClassRec*   current_class;
   int         x_size;
-
-	get_head_Rec(class_name, class_dict, CLASS_DICT_SIZE, head);
+  
+  get_head_Rec(class_name, class_dict, CLASS_DICT_SIZE, head);
   search_Rec(class_name, ClassRec, head, current_class);
   if (current_class == Null) {
     x_size = sizeof(ClassRec) + CLASS_PLENGTH + strlen(class_name) + 1;
@@ -73,14 +78,17 @@ ClassRec* create_class (class_name)
     if (current_class != Null) {
       create_Rec(class_name, ClassRec, head, current_class, CLASS_PREFIX, CLASS_PLENGTH);
       current_class->_decl_file     = Null;
-	    current_class->_decl_line     = 0;
-	    current_class->_parents_list  = Null;
-	    current_class->_parents_count = 0;
-	    current_class->_methods_list  = Null;
-	    current_class->_next_marked   = Null;
-		 	class_count++;
-		}
-	}
+      current_class->_decl_line     = 0;
+      current_class->_parents_list  = Null;
+      current_class->_parents_count = 0;
+      current_class->_methods_list  = Null;
+      current_class->_attributes_list  = Null;
+      current_class->_decl = decl;
+      current_class->_hide = 0;
+      current_class->_next_marked   = Null;
+      class_count++;
+    }
+  }
   return(current_class);
 }
 
@@ -115,6 +123,7 @@ void class_eraze_file(file_name)
   FileRec*   current_file;
   ClassRec*  current_class;
   MethodRec* current_method;
+  AttributeRec* current_attr;
   ParentRec* current_parent;
   int        index;
 
@@ -148,6 +157,15 @@ void class_eraze_file(file_name)
             (*erazed_slice) |= ((0x00000001L) << (index & LAST_BIT));
           }
           current_method = current_method->_next;
+        }
+	current_attr = current_class->_attributes_list;
+        while (current_attr != Null) {
+          if (current_attr->_decl_file == current_file) {
+            current_attr->_decl_file = Null;
+            current_attr->_decl_line = 0;
+            (*erazed_slice) |= ((0x00000001L) << (index & LAST_BIT));
+          }
+          current_attr = current_attr->_next;
         }
 	      current_class = current_class->_next;
       }
@@ -169,6 +187,9 @@ void garbage_class()
   MethodRec* previous_method;
   MethodRec* current_method;
   MethodRec* next_method;
+  AttributeRec* previous_attr;
+  AttributeRec* current_attr;
+  AttributeRec* next_attr;
   ParentRec* previous_parent;
   ParentRec* current_parent;
   ParentRec* next_parent;
@@ -203,6 +224,23 @@ void garbage_class()
               current_method  = current_method->_next;
             }
           }
+	  previous_attr = Null;
+          current_attr  = current_class->_attributes_list;
+          while (current_attr != Null) {
+            if (current_attr->_decl_file == Null) {
+              next_attr = current_attr->_next;
+              if (previous_attr == Null)
+                current_class->_attributes_list = next_attr;
+              else
+                previous_attr->_next = next_attr;
+              free(current_attr);
+              current_attr = next_attr;
+            }
+            else {
+              previous_attr = current_attr;
+              current_attr  = current_attr->_next;
+            }
+          }
           previous_parent = Null;
           current_parent  = current_class->_parents_list;
           while (current_parent != Null) {
@@ -221,6 +259,7 @@ void garbage_class()
             }
           }
           if (   (current_class->_methods_list == Null)
+	      && (current_class->_attributes_list == Null)
               && (current_class->_decl_file    == Null)) {
             next_class = current_class->_next;
             if (previous_class == Null)
