@@ -58,6 +58,7 @@ typedef enum ScopeType ScopeType;
 struct ParentRec {
   char*             _name;             /*  Le nom de la classe parent   */
   ScopeType         _scope;            /*  La nature de l'heritage      */
+  LineNumber        _decl_line;        /*  La ligne dans le fichier  */  
   struct ParentRec* _next;             /*  La classe suivante           */
 };
 
@@ -66,15 +67,20 @@ typedef struct ParentRec ParentRec;
 
 struct ClassRec;
 
+#define METHOD_ISA_CONSTRUCTOR(decl) (decl & 0x10)
+#define METHOD_IS_VIRTUAL(decl) (decl & 0x100)
+#define METHOD_IS_INLINE(decl) (decl & 0x8)
+
 struct MethodRec {
   char*             _name;             /*  Le de la methode                    */
   char*             _class_name;       /*  Le nom de la classe                 */
   ScopeType         _scope;            /*  La portee de la methode             */
-  int               _virtual_decl;     /*  La methode est virtuelle ou non     */
+  int		    _decl;
   FileRec*          _decl_file;        /*  Le fichier de declaration           */
   LineNumber        _decl_line;        /*  La ligne dans le fichier precedent  */
   FileRec*          _impl_file;        /*  Le fichier d'implementation         */
   LineNumber        _impl_line;        /*  La ligne d'implementation           */
+  unsigned	    _hide;
   struct MethodRec* _next_marked;      /*  Le pointeur de marquage             */
   struct MethodRec* _next;             /*  La methode suivante                 */
 };
@@ -82,6 +88,23 @@ struct MethodRec {
 typedef struct MethodRec MethodRec;
 
 
+#define ATTRIBUT_IS_STATIC(decl) (decl & 0x8)
+
+struct AttributeRec {
+  char*             _name;             /*  Le de la methode                    */
+  char*             _class_name;       /*  Le nom de la classe                 */
+  ScopeType         _scope;            /*  La portee de l'attribut             */
+  int		    _decl;
+  FileRec*          _decl_file;        /*  Le fichier de declaration           */
+  LineNumber        _decl_line;        /*  La ligne dans le fichier precedent  */
+  unsigned	    _hide;
+  struct AttributeRec* _next_marked;   /*  Le pointeur de marquage             */
+  struct AttributeRec* _next;          /*  L'attribut suivant                  */
+};
+
+typedef struct AttributeRec AttributeRec;
+
+
 /*------------------------------------------------------------------------------
 */
 struct ClassRec {
@@ -91,11 +114,17 @@ struct ClassRec {
   ParentRec*       _parents_list;      /*  La liste des parents                */
   int              _parents_count;     /*  Le nombre de parents                */
   MethodRec*       _methods_list;      /*  La liste des methodes               */
+  AttributeRec*    _attributes_list;   /*  La liste des methodes               */
+  unsigned	   _decl;
+  unsigned	   _hide;
   struct ClassRec* _next_marked;       /*  Le pointeur de marquage             */
   struct ClassRec* _next;              /*  La classe suivante                  */
 };
 typedef struct ClassRec ClassRec;
 
+#define TYPEDEF_TYPE 1
+#define TEMPLATE_TYPE 2
+#define ISA_TYPEDEF(x)(x & TYPEDEF_TYPE)
 
 /*------------------------------------------------------------------------------
 //                         Le dictionnaire des classes
