@@ -41,17 +41,27 @@
 #include <stdio.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
-#ifdef __FreeBSD__
-#include <stdlib.h>
-#else /* !__FreeBSD__ */
-#include <malloc.h>
+
+#if STDC_HEADERS
+#  include <stdlib.h>
+#else
+#  if HAVE_MALLOC_H
+#    include <malloc.h>
+#  endif
 #endif
+
+#if HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
+#endif
 
 #include "main_text.h"
 #include "color_area.h"
+#include "ie_func.h"
 
 extern Display *dpy;
+
 /*
 #define DEBUG2
 #define DEBUG
@@ -115,7 +125,7 @@ void SetColorArea ( text, start, end, colorname )
 	tmp1 = tmp;
 #ifdef DEBUG    
 	(void) fprintf (stderr,"tmp = %d\n", tmp);
-#endif /* DEBUG */    
+#endif	 /* DEBUG */    
 	if ((start >= tmp -> pos) && (start <= (tmp -> pos + tmp -> len -1))) {
 	    /* On est a l'interieur d'un element */
 	    InsertInsideElement ( text, new_ce, tmp );
@@ -164,6 +174,7 @@ static ColorElement *CreateColorElement ( start, end, color )
     ColorElement *new_ce;
     
     new_ce = (ColorElement *) malloc ( (unsigned) sizeof(ColorElement) + 2);
+    (void) bzero ((char *) new_ce, sizeof(ColorElement) + 2);
     new_ce -> pos = start;
     new_ce -> len = end - start + 1;
     new_ce -> color = color;
@@ -322,7 +333,7 @@ static void CheckColorList ( text, new_ce )
     ColorElement *tmp = new_ce;
     ColorElement *previous;
     
-    while (tmp = tmp -> next) {
+    while ((tmp = tmp -> next) != 0) {
 	previous = tmp -> previous;
 	if ( (previous -> pos +  previous -> len -1) < tmp -> pos )
 	  break;
@@ -561,7 +572,7 @@ static void DecrColorList (text, nbytes)
     Text *text;
     int nbytes; /* > 0 */
 {
-    ColorElement *ce, *next, *previous;
+    ColorElement *ce;
     ColorElement *tmp = text -> last_ce;
     int pos_before, pos_after;
   
@@ -628,7 +639,7 @@ static void DecrColorList (text, nbytes)
 	    
 	    /* On enleve l'element et on met les liens a jour. */
 	    ce = tmp -> previous;
-	    if (text -> current_ce = tmp)
+	    if ((text -> current_ce = tmp) != 0)
 	      text -> current_ce = tmp -> previous;
 	    (void) free (tmp);
 	    
@@ -702,7 +713,6 @@ static void DecrColorList (text, nbytes)
 #ifdef DEBUG    
 /*    DisplayColorList (text); */
 #endif /* DEBUG */    
-#undef DEBUG    
 }
 
 /*
@@ -732,7 +742,9 @@ void ColorLineInPage ( text, x, y, line, s, n )
 
 #ifdef DEBUG
     (void) fprintf (stderr,"LIGNE %d n1 = %d\n", line, text -> n1);
-#endif DEBUG    
+    (void) write (1,s,n);
+    (void) write (1,"\n",1);
+#endif /* DEBUG */
 
     if ( line == text -> n1 + 1 ) {
 	/*
@@ -744,6 +756,13 @@ void ColorLineInPage ( text, x, y, line, s, n )
 	   ligne scrollee horizontalement;
 	*/
 	pos = ie_beginning_of_line(text) + text -> page.hs_delta;
+
+	if (ie_the_char(text,pos) != *s) {
+	  /* on est apres un tab, il faut donc se placer apres */
+	  while (ie_the_char(text,pos) != '\t') {
+	    pos++;
+	  }
+	}
 	/*
 	   Si ya des tab au debut de la ligne ,
 	   il ne faut pas en tenir compte
@@ -864,13 +883,12 @@ void ColorLineInPage ( text, x, y, line, s, n )
 #endif /* DEBUG */
 	}
     }
-#undef DEBUG
     
     if ( tmp == 0 ) {
 	XDrawImageString ( dpy, text -> window, text -> Cgc, x, y, s, n );
 #ifdef DEBUG
 	(void) fprintf (stderr,"\tPas d'elements de couleur\n");
-#endif DEBUG    
+#endif /* DEBUG */
 	return;
     }
     text -> current_ce = tmp;
@@ -879,14 +897,10 @@ void ColorLineInPage ( text, x, y, line, s, n )
        a afficher ou bien il se trouve a l'interieur de celle-ci,
        partiellement ou totalement.
     */
-#ifdef DEBUG
-    (void) fprintf (stderr,"LIGNE %d\n", line);
-#endif DEBUG    
-
     while ( n > 0 ) {
 #ifdef DEBUG
     (void) fprintf (stderr,"tmp = %d pos = %d\n", tmp, pos);
-#endif DEBUG    
+#endif /* DEBUG */
 	if (pos == tmp -> pos) {
 	    /* La position courante est sur un element de couleur */
 	    if (n >= tmp -> len)
