@@ -43,24 +43,29 @@
 #include <X11/Xutil.h>
 #include <X11/Xresource.h>
 
-#ifndef apollo
-#ifdef __FreeBSD__
-#include <stdlib.h>
-#else /* !__FreeBSD__ */
-#include <malloc.h>
-#endif
+#if STDC_HEADERS
+#  include <stdlib.h>
+#else
+#  if HAVE_MALLOC_H
+#    include <malloc.h>
+#  endif
 #endif
+
+#if HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
+#endif
 
 #include "main_text.h"
 #include "handle_key.h"
 #include "shadow.h"
-#include "button.bm"
+#include "parse.h"
+#include "page.h"
+#include "buttons_bits.h"
 
 ButtonWindow bw [NB_BUTTONS];
 static ResourcesButton rs;
-extern unsigned long 	PixelValue ();
-extern void NextPage(), PreviousPage(),ScrollDownCont(),ScrollUpCont(),LastPage(),FirstPage();
 extern Display *dpy;
 
 FCT (static void, MakeButtons, (MWin *mwin,unsigned long fg, unsigned long bg) );
@@ -81,7 +86,14 @@ void InitControlRes ( fg, bg, ts, bs )
     XGCValues 	gcv;
     Window		root;
     int 		depth;
-    
+    int result;
+    unsigned long pfg, pbg;
+
+    pfg = BlackPixel ( dpy, DefaultScreen (dpy));
+    pbg = PixelValue ( dpy, "lightgrey", &result );
+    if ( result == False )
+      pbg = WhitePixel ( dpy, DefaultScreen (dpy));
+
     rs.fg = fg; 
     rs.bg = bg;
     
@@ -92,7 +104,7 @@ void InitControlRes ( fg, bg, ts, bs )
     
     XSetState ( dpy, rs.mess_gc, fg, bg, GXcopy, AllPlanes );
     rs.font = XLoadQueryFont ( dpy,
-			      "-adobe-courier-bold-r-normal--*-140-*-*-m-*-iso8859-1");
+			      "-adobe-courier-bold-r-normal--14-140-75-75-m-90-iso8859-1");
     if ( rs.font != 0 ) {
 	XSetFont ( dpy, rs.mess_gc, rs.font -> fid );
     }
@@ -102,17 +114,17 @@ void InitControlRes ( fg, bg, ts, bs )
 	XSetFont ( dpy, rs.mess_gc, rs.font -> fid );
     }
     rs.up_pix = XCreatePixmapFromBitmapData ( dpy, root, (char *) up_bits,
-					     BUTTON_W, BUTTON_W, fg, bg, depth );
+					     BUTTON_W, BUTTON_W, pfg, pbg, depth );
     rs.down_pix = XCreatePixmapFromBitmapData ( dpy, root, (char *) down_bits,
-					       BUTTON_W, BUTTON_W, fg, bg, depth );
+					       BUTTON_W, BUTTON_W, pfg, pbg, depth );
     rs.np_pix = XCreatePixmapFromBitmapData ( dpy, root, (char *) np_bits,
-					     BUTTON_W, BUTTON_W, fg, bg, depth );
+					     BUTTON_W, BUTTON_W, pfg, pbg, depth );
     rs.pp_pix = XCreatePixmapFromBitmapData ( dpy, root, (char *) pp_bits,
-					     BUTTON_W, BUTTON_W, fg, bg, depth );
+					     BUTTON_W, BUTTON_W, pfg, pbg, depth );
     rs.queue_pix = XCreatePixmapFromBitmapData ( dpy, root, (char *) q_bits,
-						BUTTON_W, BUTTON_W, fg, bg, depth );
+						BUTTON_W, BUTTON_W, pfg, pbg, depth );
     rs.head_pix = XCreatePixmapFromBitmapData ( dpy, root, (char *) head_bits,
-					       BUTTON_W, BUTTON_W, fg, bg ,depth );
+					       BUTTON_W, BUTTON_W, pfg, pbg ,depth );
     
     rs.top_shadow = ts;
     rs.bot_shadow = bs;
@@ -144,67 +156,67 @@ void SetCtrMode ( mwin, mode )
 MWin *MakeControlPanel ( w )
     Window w;
 {
-    MWin *mwin;
-    int result;
-    char *tmp_color;
-    extern char *getenv ();
-    
-    mwin = ( MWin * ) malloc ( sizeof ( MWin ));	
-    
-    mwin -> frame = XCreateSimpleWindow ( dpy, w, 0,
-					 0, DEFAULT_SIZE, DEFAULT_SIZE, 0, rs.fg, rs.bg );
-    
-    mwin -> w_stat = XCreateSimpleWindow ( dpy, mwin -> frame,
-					  1, 1, BUTTON_W, BUTTON_W, 0, rs.fg, rs.bg );
-    XSelectInput ( dpy, mwin -> w_stat, ExposureMask );
-    
-    mwin -> mess = XCreateSimpleWindow ( dpy, mwin -> frame,
-					BUTTON_W + 1, 1, DEFAULT_SIZE, DEFAULT_SIZE, 0, rs.fg, rs.bg ); 
-    XSelectInput ( dpy, mwin -> mess, ExposureMask | KeyPressMask );
-    
-    mwin -> w_mode = XCreateSimpleWindow ( dpy, mwin -> frame, 0,
-					  1, ( 4 * BUTTON_W ), BUTTON_W, 0, rs.fg, rs.bg );					      
-    XSelectInput ( dpy, mwin -> w_mode, ExposureMask );
-    
-    mwin -> mess_gc = rs.mess_gc;
-    mwin -> fg = rs.fg;
-    mwin -> bg = rs.bg;
-    mwin -> font = rs.font;
-    mwin -> top_sh = rs.top_shadow;
-    mwin -> bot_sh = rs.bot_shadow;
-    mwin -> stat = False;
-    mwin -> twidth = DEFAULT_SIZE;
-    mwin -> theight = BUTTON_W;
-    
-    if ( UseColor () == False ) {
-	
-	if ( mwin -> bg == BlackPixel ( dpy, DefaultScreen ( dpy ))) {
-	    mwin -> red = WhitePixel ( dpy, DefaultScreen ( dpy ));
-	    mwin -> green = BlackPixel ( dpy, DefaultScreen ( dpy ));
-	}
-	else {
-	    mwin -> red = BlackPixel ( dpy, DefaultScreen ( dpy ));
-	    mwin -> green = WhitePixel ( dpy, DefaultScreen ( dpy ));
-	}
+  MWin *mwin;
+  int result;
+  char *tmp_color;
+  extern char *getenv ();
+  
+  mwin = ( MWin * ) malloc ( sizeof ( MWin ));	
+  
+  mwin -> frame = XCreateSimpleWindow ( dpy, w, 0,
+				       0, DEFAULT_SIZE, DEFAULT_SIZE, 0, rs.fg, rs.bg );
+  
+  mwin -> w_stat = XCreateSimpleWindow ( dpy, mwin -> frame,
+					1, 1, BUTTON_W, BUTTON_W, 0, rs.fg, rs.bg );
+  XSelectInput ( dpy, mwin -> w_stat, ExposureMask );
+  
+  mwin -> mess = XCreateSimpleWindow ( dpy, mwin -> frame,
+				      BUTTON_W + 1, 1, DEFAULT_SIZE, DEFAULT_SIZE, 0, rs.fg, rs.bg ); 
+  XSelectInput ( dpy, mwin -> mess, ExposureMask | KeyPressMask );
+  
+  mwin -> w_mode = XCreateSimpleWindow ( dpy, mwin -> frame, 0,
+					1, ( 4 * BUTTON_W ), BUTTON_W, 0, rs.fg, rs.bg );					      
+  XSelectInput ( dpy, mwin -> w_mode, ExposureMask );
+  
+  mwin -> mess_gc = rs.mess_gc;
+  mwin -> fg = rs.fg;
+  mwin -> bg = rs.bg;
+  mwin -> font = rs.font;
+  mwin -> top_sh = rs.top_shadow;
+  mwin -> bot_sh = rs.bot_shadow;
+  mwin -> stat = False;
+  mwin -> twidth = DEFAULT_SIZE;
+  mwin -> theight = BUTTON_W;
+  
+  if ( UseColor () == False ) {
+    if ( mwin -> bg == BlackPixel ( dpy, DefaultScreen ( dpy ))) {
+      mwin -> red = WhitePixel ( dpy, DefaultScreen ( dpy ));
+      mwin -> green = BlackPixel ( dpy, DefaultScreen ( dpy ));
     }
     else {
-      if ( (tmp_color = (char *) getenv ( "XCORAL_MODIF_COLOR" )) == 0 )
-	  mwin -> red = PixelValue ( dpy, "red", &result );
-	else 
-	  mwin -> red = PixelValue ( dpy, tmp_color, &result );
-	if ( result == False )
-	  mwin -> red = BlackPixel ( dpy, DefaultScreen ( dpy ));
-	
-	if ( (tmp_color = (char *) getenv ( "XCORAL_SAVE_COLOR" )) == 0 )
-	  mwin -> green = 0;
-	else
-	  mwin -> green = PixelValue ( dpy, tmp_color, &result );
-	if ( result == False )
-	  mwin -> green = 0;
-	  
+      mwin -> red = BlackPixel ( dpy, DefaultScreen ( dpy ));
+      mwin -> green = WhitePixel ( dpy, DefaultScreen ( dpy ));
     }
-    (void) MakeButtons ( mwin, rs.fg, rs.bg ); 
-    return ( mwin );
+  }
+  else {
+    if ( (tmp_color = (char *) getenv ( "XCORAL_MODIF_COLOR" )) == 0 )
+      mwin -> red = PixelValue ( dpy, "red", &result );
+    else 
+      mwin -> red = PixelValue ( dpy, tmp_color, &result );
+
+    if ( result == False )
+      mwin -> red = BlackPixel ( dpy, DefaultScreen ( dpy ));
+    
+    if ( (tmp_color = (char *) getenv ( "XCORAL_SAVE_COLOR" )) == 0 )
+      mwin -> green = 0;
+    else
+      mwin -> green = PixelValue ( dpy, tmp_color, &result );
+    
+    if ( result == False )
+      mwin -> green = 0;
+  }
+  (void) MakeButtons ( mwin, rs.fg, rs.bg ); 
+  return ( mwin );
 }
 
 
@@ -409,11 +421,7 @@ void ShowControlPanel ( mwin, width, height )
 */
 void DisplayMessage ( mwin, s ) 
     MWin *mwin;
-#ifdef __FreeBSD__
-    const char *s;
-#else
     char *s;
-#endif
 {
     if ( (s == 0) || (mwin == 0) )
       return;
