@@ -38,11 +38,16 @@
    ######################################################################## */
 
 
+#if HAVE_STRING_H
+#include <string.h>
+#else
+#include <strings.h>
+#endif
+#include <stdio.h>
+
 #include "result_types.h"
 #include "file_dict.h"
 #include "browser_util.h"
-#include <string.h>
-#include <stdio.h>
 
 /*------------------------------------------------------------------------------
 //                         Le dictionnaire des fichiers
