@@ -39,10 +39,12 @@
 
 
 #include <stdio.h>
-#ifdef UNIXWARE
-#include <unistd.h>
-#include <filehdr.h>
-#endif 
+
+#if HAVE_UNISTD_H
+#  include <sys/types.h>
+#  include <unistd.h>
+#endif
+
 #include <X11/Xlib.h>
 #include <X11/cursorfont.h>
 #include <X11/Xutil.h>
@@ -52,17 +54,18 @@
 #include <sys/file.h>
 #include <sys/param.h>
 
-#ifndef apollo
-#ifdef __FreeBSD__
-#include <stdlib.h>
-#else /* !__FreeBSD__ */
-#include <malloc.h>
+#if STDC_HEADERS
+#  include <stdlib.h>
+#else
+#  if HAVE_MALLOC_H
+#    include <malloc.h>
+#  endif
 #endif
-#endif
-#include <string.h> 
-#ifdef SVR4
-#include <unistd.h>
-#include <sys/exechdr.h>
+
+#if HAVE_STRING_H
+#include <string.h>
+#else
+#include <strings.h>
 #endif
 
 #include <errno.h>
@@ -79,90 +82,13 @@
 #include "input_str.h"
 #include "dial_box.h"
 #include "warn_box.h"
+#include "ie_func.h"
+#include "selection.h"
 
 FCT (static int, CheckFile, (char *path, int *len, Text *text) );
 FCT (static int, IsBinary, (FILE *fd, Text *text, int len) );
 FCT (static void, KillCurrentBuffer, (Text *text, int from) );
-     
-#ifdef sun
-#define _SUN_OS
-#include <sys/exec.h>
-#define ISMAG(x) (((x) == OMAGIC) || ((x) == NMAGIC) || ((x) == ZMAGIC))
-#endif
-
-#ifdef apollo
-#define _APOLLO
-#include <filehdr.h>
-#define ISMAG(x) ISCOFF(x)
-#endif
-
-#if  (m68k && _AUX_SOURCE)
-#define _APPLE_A_UX
-#include <filehdr.h>
-#define ISMAG(x) (((x)==MC68MAGIC) || ((x)==MC68TVMAGIC) || ((x)==M68MAGIC) || ((x)==M68TVMAGIC) \
-|| ((x)==M68NSMAGIC))
-#endif
-
-#ifdef hpux
-#define _HPUX
-#include <filehdr.h>
-#define ISMAG(x) (((x) == RELOC_MAGIC) || ((x) == EXEC_MAGIC) || ((x) == SHARE_MAGIC) \
-	|| ((x) == DEMAND_MAGIC) || ((x) == DL_MAGIC) || ((x) == SHL_MAGIC))
-#endif
-
-#if ((__osf__) && (__alpha))
-#define _DEC_OSF
-#include <sys/exec.h>
-#define ISMAG(x) (((x) == OMAGIC) || ((x) == NMAGIC) || ((x) == ZMAGIC))
-#endif
-
-#if ((ultrix) && (mips))
-#define _DEC_ULTRIX
-#include <sys/exec.h>
-#define ISMAG(x) (((x) == OMAGIC) || ((x) == NMAGIC) || ((x) == ZMAGIC))
-#endif
-
-#ifdef _AIX
-#include <filehdr.h>
-#define ISMAG(x) (((x) == U802WRMAGIC) || ((x) == U802ROMAGIC) || \
-	((x) == U802TOCMAGIC) || ((x) == U800WRMAGIC) || \
-	((x) == U800ROMAGIC)  || ((x) == U800TOCMAGIC))
-#endif
-
-#if ((sgi) || (__sgi))
-#define _SGI
-#include <filehdr.h>
-#define ISMAG(x) ISCOFF(x)
-#endif
-
-#ifdef linux
-#define _LINUX
-#include <unistd.h>
-#include <a.out.h>
-#define ISMAG(x) (((x)==OMAGIC)||((x)==NMAGIC)||((x)==ZMAGIC))
-#endif
-
-#if defined (__NetBSD__) || defined (__FreeBSD__)
-#define _NETBSD
-#include <a.out.h>
-#define ISMAG(x) (((x) == OMAGIC) || ((x) == NMAGIC) || ((x) == ZMAGIC))
-#endif
-
-#ifdef __bsdi__
-#define _BSDI
-#include <a.out.h>
-#define ISMAG(x) (((x) == OMAGIC) || ((x) == NMAGIC) || ((x) == ZMAGIC))
-#endif
-
-#ifdef UNIXWARE
-#include <sys/x.out.h>
-#define ISMAG(x) (((x) == OMAGIC) || ((x) == NMAGIC) || ((x) == ZMAGIC))
-#endif
-
-#ifndef ISMAG
-#define _BAD_SYS
-#define ISMAG(x) (((x) == OMAGIC) || ((x) == NMAGIC) || ((x) == ZMAGIC))
-#endif
+FCT (static char*, MyStrerror,  ( int err ) );
 
 extern 	Display	*dpy;
 
@@ -197,6 +123,11 @@ void KbdReadFile ( text )
     }
     if ( *str == '~' )
       str = ExpandTildeName ( str );
+
+    if (text -> selection -> select == True) {
+      SetCurrentLine (text);
+      ClearSelection(text, True);
+    }
     
     TextCursorOff ( text );
     if ( LoadFile ( text, str, NEW ) != -1 ) {
@@ -231,7 +162,7 @@ void KbdInsertFile ( text )
     char c = '\007'; /* ^G */
     int nbl, olen;
     int x_len;
-    
+
     ClearMessageWindow ( text -> mwin );
     
     str = GetString ( text, "Insert File : ", 0 );
@@ -243,6 +174,11 @@ void KbdInsertFile ( text )
     if ( *str == '~' )
       str = ExpandTildeName ( str );
     
+    if (text -> selection -> select == True) {
+      SetCurrentLine (text);
+      ClearSelection(text, True);
+    }
+    
     TextCursorOff ( text );
  
     nbl = text -> lines_in_buf;
@@ -260,13 +196,22 @@ void KbdInsertFile ( text )
     
 	/* Pour la couleur */
 	if ( text -> current_ce != 0 ) {
+	  if(x_len != 0)
+	    MoveHole ( text -> buf, x_len );
 	  (void) UpdateColorList ( text, x_len );
+	  if(x_len != 0)
+	    MoveHole ( text -> buf, -x_len );
 	  ClipOn ( text, 0 );
 	  RefreshPage ( text );
 	  ClipOff ( text );
 	}
     }
     TextCursorOn ( text );
+    if ( text -> stat & LASTPAGE ) {
+      MCLEAR ( text -> stat, LASTPAGE );
+      MCLEAR ( text -> stat, MESSAGE );
+      MCLEAR ( text -> stat, FIRSTPAGE );
+    }
 }
 
 
@@ -286,7 +231,6 @@ int LoadFile ( text, s, type )
     static int len;
     int n;
     Mode *mode;
-    char *msg;
     FILE *fd;
     extern flag_visit;
     
@@ -388,12 +332,11 @@ void SetDirAndFilename ( text, name )
 	  
 	*p1 = 0;
 	chdir(text -> filename);
-#ifdef linux
-	if ( getwd ( pathname ) == 0 ) {
-	    (void) fprintf ( stderr, "Can't getwd\n" );			
-	}
-#else	
+
+#if HAVE_GETCWD
 	(void) getcwd ( pathname, MAXPATHLEN );
+#else
+	(void) getwd  ( pathname );
 #endif
 	(void) strcpy( text -> current_dir, pathname);
 	if ( strcmp ( text -> current_dir, "/" ) == 0 )
@@ -409,13 +352,12 @@ void SetDirAndFilename ( text, name )
 	 * A cause de l'automount et des liens symboliques il faut
 	 * de nouveau verifier le path.
 	 */
-#ifdef linux
-	if ( getwd ( pathname ) == 0 ) {
-	    (void) fprintf ( stderr, "Can't getwd\n" );			
-	}
-#else	
+#if HAVE_GETCWD
 	(void) getcwd ( pathname, MAXPATHLEN );
+#else
+	(void) getwd  ( pathname );
 #endif
+
 	if ( strcmp ( pathname, text -> current_dir ) != 0 ) {
 	  (void) strcpy ( text -> current_dir, pathname );
 	  if (! strcmp(text -> current_dir, "/"))
@@ -497,15 +439,12 @@ void WriteFile ( text )
 {
     FILE *fd;
     int size;
-#ifndef __FreeBSD__
-    extern char *sys_errlist[];
-#endif
-    char buf [128];
+    char buf [2*MAXPATHLEN];
     char *name;
     
     write_flag = True;
     if ( (fd = fopen ( text -> filename, "w" )) == 0 ) {
-	DisplayMessage ( text -> mwin, sys_errlist[errno] );
+	DisplayMessage ( text -> mwin, MyStrerror(errno) );
 	write_flag = False;
 	return;
     }
@@ -515,6 +454,8 @@ void WriteFile ( text )
     if ( size == -1 )  {
 	(void) sprintf ( buf, "Write error" );
 	DisplayMessage ( text -> mwin, buf );
+	DisplayWMessage ("Warning: Write error\nfile system full, may be",
+			 "Write File", True );
 	(void) fclose ( fd );
 	write_flag = False;
 	return;
@@ -522,10 +463,8 @@ void WriteFile ( text )
     else {
 	name = 0;
 	if ( (name = (char * ) strrchr ( text -> filename, '/' )) != 0 )
-/*	  (void) sprintf ( buf, "Write %s, %d bytes.\n", ++name, size ); */
 	  (void) sprintf ( buf, "Write %s, %d bytes.", ++name, size );
 	else {
-/*	    (void) sprintf ( buf, "Write %s, %d bytes.\n", text -> filename, size ); */
 	  (void) sprintf ( buf, "Write %s, %d bytes.", text -> filename, size );
 	}
 	DisplayMessage ( text -> mwin, buf );
@@ -557,10 +496,10 @@ void KbdWriteFile ( text )
     Text *text;
 {
     char *s;
-    char buf [128];
+    char buf [256];
     char c = '\007';
     char *name;
-    char tmp_name [1024];
+    char tmp_name [MAXPATHLEN];
     
 #ifdef DEBUG
     (void) fprintf ( stderr, "Write File from key\n" );
@@ -654,97 +593,7 @@ static int IsBinary ( fd, text, len )
     Text *text;
     int len;
 {
-    unsigned short magic;
-#ifdef _SUN_OS
-    struct exec head;
-#endif
-#ifdef _HPUX
-    struct header head;
-#endif
-#ifdef _DEC_OSF
-    struct exec head;
-#endif
-#ifdef _DEC_ULTRIX
-    struct exec head;
-#endif
-#ifdef _APPLE_A_UX
-    struct filehdr head;
-#endif
-#ifdef _AIX
-    struct filehdr head;
-#endif
-#ifdef _SGI
-    struct filehdr head;
-#endif
-#ifdef _APOLLO
-    struct filehdr head;
-#endif
-#ifdef _LINUX
-    struct exec head;
-#endif
-
-#ifdef _NETBSD
-        struct exec head;
-#endif
-
-#ifdef _BSDI
-	struct exec head;
-#endif
-    
-#ifdef _BAD_SYS
-    struct exec head;
-#endif
-
-#ifdef UNIXWARE
-    struct filehdr head;
-#endif
-    
-    if ( len < sizeof (head) )	/* Un peu court ... */
-      return False;
-    if (fread((char *) &head, sizeof (head), 1, fd) != 1) {
-	(void) fprintf ( stderr, "Header read error\n" );
-	return False;
-    }
-#ifdef _SUN_OS
-    magic = head.a_magic;
-#endif
-#ifdef _DEC_ULTRIX
-    magic = head.ex_o.magic;
-#endif
-#ifdef _DEC_OSF
-    magic = head.ex_o.magic;
-#endif
-#ifdef _HPUX
-    magic = head.a_magic;
-#endif
-#ifdef _APPLE_A_UX
-    magic = head.f_magic;
-#endif
-#ifdef _SGI
-    magic = head.f_magic;
-#endif
-#ifdef _AIX
-    magic = head.f_magic;
-#endif
-#ifdef _APOLLO
-    magic = head.f_magic;
-#endif
-#ifdef _LINUX
-    magic = (short) head.a_info;
-#endif
-#ifdef _BSDI
-	magic = head.a_magic;
-#endif
-#ifdef _NETBSD
-        magic = N_GETMAGIC (head);
-#endif
-
-    if ( ISMAG (magic) 
-	|| (!strncmp ( (char *) &head, (char *) "!<arch>", 6)) ) {
-	DisplayMessage  ( text -> mwin, "%@#7&^/_!...brrr..." );
-	return True;
-    }
-    return False ;
+  return False;
 }
 
 
@@ -776,6 +625,11 @@ void MenuReadFile ( text )
 	return;
     }
 
+    if (text -> selection -> select == True) {
+      SetCurrentLine (text);
+      ClearSelection(text, True);
+    }
+    
     TextCursorOff ( text );
 
     if ( LoadFile ( text, str, NEW ) != -1 ) {
@@ -810,13 +664,19 @@ void MenuInsertFile ( text )
     char c = '\007'; /* ^G */
     int nbl, olen;
     int x_len;
-    
+
+    if (text -> selection -> select == True){
+      SetCurrentLine (text);
+      ClearSelection(text, True);
+    }
+
     ClearMessageWindow ( text -> mwin );
     str = (char *) SelectFileFromBox ( "Insert File" );
     if ( (str == 0) || (strncmp(str, &c, 1)) == 0 ) {
 	DisplayMessage ( text -> mwin, "Abort" );
 	return;
     }
+
     TextCursorOff ( text );
     nbl = text -> lines_in_buf;
     olen = BottomBuf (text -> buf) - RightBuf (text -> buf);
@@ -832,13 +692,22 @@ void MenuInsertFile ( text )
 		     text -> lines_in_buf - nbl, U_INSERT );
 	/* Pour la couleur */
 	if ( text -> current_ce != 0 ) {
+	  if(x_len != 0)
+	    MoveHole ( text -> buf, x_len );
 	  (void) UpdateColorList ( text, x_len );
+	  if(x_len != 0)
+	    MoveHole ( text -> buf, -x_len );
 	  ClipOn ( text, 0 );
 	  RefreshPage ( text );
 	  ClipOff ( text );
 	}
     }
     TextCursorOn ( text );
+    if ( text -> stat & LASTPAGE ) {
+      MCLEAR ( text -> stat, LASTPAGE );
+      MCLEAR ( text -> stat, MESSAGE );
+      MCLEAR ( text -> stat, FIRSTPAGE );
+    }
 }
 
 
@@ -855,6 +724,11 @@ void MenuSaveFile ( text )
     char   *name;
     char c = '\007';
     
+    if (text -> selection -> select == True) {
+      SetCurrentLine (text);
+      ClearSelection(text, True);
+    }
+
     if ( strcmp (text -> filename, "NoName") == 0 ) {
 	name = (char *) GetStringFromDB ( "Write file : ", False );
 	
@@ -904,14 +778,18 @@ void MenuWriteFile ( text )
     Text *text;
 {
     char *s;
-    char buf [128];
+    char buf [256];
     char c = '\007';
     char *name;
-    char tmp_name [1024];
+    char tmp_name [MAXPATHLEN];
     
 #ifdef DEBUG
     (void) fprintf ( stderr, "Write File from menu\n" );
 #endif
+    if (text -> selection -> select == True) {
+      SetCurrentLine (text);
+      ClearSelection(text, True);
+    }
     
     if ( (name = (char * ) strrchr ( text -> filename, '/' )) != 0 ) {
       name ++; /* On vire le slash */
@@ -967,8 +845,12 @@ int SaveCurrentBuffer ( text, from )
     char *str = 0;
     char c = '\007'; /* ^G */
     char *buf, *name;
-    char tmp_name [1024];
-    extern char *sys_errlist[];
+    char tmp_name [MAXPATHLEN];
+
+    if (text -> selection -> select == True) {
+      SetCurrentLine (text);
+      ClearSelection(text, True);
+    }
     
     buf = (char *) malloc ( (unsigned) strlen ( text -> filename ) + 64 );
     
@@ -992,7 +874,7 @@ int SaveCurrentBuffer ( text, from )
 	if ( (str == 0) || ((strncmp(str,"y",1) == 0)&&(strlen(str)==1)) ) {
 	    WriteFile ( text );
 	    if (write_flag != True) {
-	      DisplayWMessage ( sys_errlist[errno], "Save current buffer", True );
+	      DisplayWMessage ( MyStrerror(errno), "Save current buffer", True );
 	      if ( buf != 0 )
 		(void) free ( buf );
 	      return False;
@@ -1033,6 +915,11 @@ void MenuGotoLine ( text )
     char *str;
     char c = '\007';
     
+    if (text -> selection -> select == True) {
+      SetCurrentLine (text);
+      ClearSelection(text, True);
+    }
+
     str = (char * ) GetStringFromDB ( "Line number : ", False );
     if ( (str == 0) || (strncmp (str, &c, 1 ) == 0) || ((n = atoi ( str )) == 0) )
       DisplayMessage ( text -> mwin, "Abort" );
@@ -1090,9 +977,14 @@ static void KillCurrentBuffer ( text, from )
 void KillBuffer ( text )
     Text *text;
 {
-    TextCursorOff ( text );
-    KillCurrentBuffer ( text, F_KEY );
-    TextCursorOn ( text );
+  if (text -> selection -> select == True){
+    SetCurrentLine (text);    
+    ClearSelection(text, True);
+  }
+    
+  TextCursorOff ( text );
+  KillCurrentBuffer ( text, F_KEY );
+  TextCursorOn ( text );
 }
 
 /*
@@ -1124,11 +1016,13 @@ void LoadAndEvalFile ( text )
     char *msg = 0;
     char *s = 0;
     char c = '\007';
-    char buf[128];
- 
+
+    if (text -> selection -> select == True){
+      SetCurrentLine (text);
+      ClearSelection(text, True);
+    }
+
     StorePosition ( text );
-    (void) sprintf ( buf, "Load and eval [.xcoralrc]  : " );
-    
     s = SelectFileFromBox ( "Load and eval file" );
     if ( (s == 0) || (strncmp(s, &c, 1)) == 0 ) {
 	DisplayMessage ( text -> mwin, "Abort" );
@@ -1191,3 +1085,152 @@ Text * update_cwd(win_id)
   return TWin [win_id]->text;
 }
 
+FCT (extern int, browser_dump,(char *));
+FCT (extern int, browser_restore,(char *));
+
+void MenuBrowserDump ( text )
+    Text *text;
+{
+    char *str = 0;
+    
+    if (text){
+      if (text -> selection -> select == True){
+	SetCurrentLine (text);
+	ClearSelection(text, True);
+      }
+      ClearMessageWindow ( text -> mwin );
+    }
+
+    str = (char *) SelectFileFromBox ( "Browser dump file" );
+    
+    if ((str == 0) || (*str == 7)) {
+	if (text)
+	  DisplayMessage ( text -> mwin, "Abort" );
+	return;
+    }
+
+    if (text)
+      TextCursorOff ( text );
+
+    if (! browser_dump(str)) {
+      XBell ( dpy, 10 );
+      if (text)
+	DisplayMessage ( text -> mwin, "dump error" );
+    }
+    if (text)
+      TextCursorOn ( text );
+}
+
+void MenuBrowserRestore ( text )
+    Text *text;
+{
+    char *str;
+    
+    if (text) {
+      ClearMessageWindow ( text -> mwin );
+      if (text -> selection -> select == True){
+	SetCurrentLine (text);
+	ClearSelection(text, True);
+      }
+    }
+
+    str = (char *) SelectFileFromBox ( "Browser restore file" );
+    
+    if ((str == 0) || (*str == 7)) {
+	if (text)
+	  DisplayMessage ( text -> mwin, "Abort" );
+	return;
+    }
+
+    if (text)
+      TextCursorOff ( text );
+
+    if (! browser_restore(str)) {
+      XBell ( dpy, 10 );
+      if (text)
+	DisplayMessage ( text -> mwin, "restore error" );
+    }
+    if (text)
+      TextCursorOn ( text );
+}
+
+void KbdBrowserDump ( text )
+    Text *text;
+{
+    char *str;
+    
+    ClearMessageWindow ( text -> mwin );
+    
+    str = GetString ( text, "Browser dump File : ", 0 );
+    
+    if ((str == 0) || (*str == 7)) {
+	DisplayMessage ( text -> mwin, "Abort" );
+	return;
+    }
+    if ( *str == '~' )
+      str = ExpandTildeName ( str );
+    
+    TextCursorOff ( text );
+
+    if (! browser_dump(str)) {
+      XBell ( dpy, 10 );
+      DisplayMessage ( text -> mwin, "dump error" );
+    }
+    TextCursorOn ( text );
+}
+
+void KbdBrowserRestore ( text )
+    Text *text;
+{
+    char *str;
+    
+    ClearMessageWindow ( text -> mwin );
+    
+    str = GetString ( text, "Restore File : ", 0 );
+    
+    if ((str == 0) || (*str == 7)) {
+	DisplayMessage ( text -> mwin, "Abort" );
+	return;
+    }
+    if ( *str == '~' )
+      str = ExpandTildeName ( str );
+    
+    TextCursorOff ( text );
+
+    if (! browser_restore(str)) {
+      XBell ( dpy, 10 );
+      DisplayMessage ( text -> mwin, "restore error" );
+    }
+    TextCursorOn ( text );
+}
+
+
+/*---------------------------------------------------------------------
+*/
+static char*
+MyStrerror ( err )
+    int err;
+{
+#if HAVE_STRERROR
+   return strerror ( err );
+#elif HAVE_SYSERR || HAVE_STDIO_SYSERR
+#  if !HAVE_STDIO_SYSERR
+   extern char *sys_errlist[];
+   extern int sys_nerr;
+#  endif
+   static char buffer[32];
+   if (!err)
+     return "";
+ 
+   if ((err > 0) && (err < sys_nerr))
+     return (char*) sys_errlist[err];
+ 
+   sprintf (buffer, "Unknown error : %d", err);
+   return buffer;
+#else
+#if HAVE_PERROR
+   perror("System error in xcoral excution");
+#endif
+   return("<<< Error message unavailble >>>");
+#endif
+}
