@@ -44,7 +44,8 @@
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
 #include <ctype.h>
-#if defined(SYSV) || defined(UNIXWARE)
+
+#if HAVE_STRING_H
 #include <string.h>
 #else
 #include <strings.h>
@@ -284,7 +285,8 @@ ST *automate ( text, event, current_st )
 #ifdef DEBUG
     printf ( "n_bytes = %d %X\n", n_bytes, ksym );
 #endif
-    (void) GetInfosKey ( text, ksym, buf [0], &infos );
+
+    (void) GetInfosKey ( text, ksym, (unsigned char) buf [0], &infos );
 
     deletewindow = False;
     initial_stat_return = False;
@@ -382,7 +384,6 @@ static void GetInfosAsciiKey ( text, c, infos )
     int c;
     InfosKey *infos;
 {
-    
   if ( c >= 0 && c <= CtrZ && n_bytes != 0 ) {
     switch ( c ) {
 	case CtrX	:
@@ -450,7 +451,6 @@ static void f_ascii ( text, infos )
 	if ( (type == EXTERN_FUNC) && (func != 0 )) {
 	  int nw = text->win_id;
 	  
-/*	  CheckModifAndFilename ( text ); */
 	  if ( InsideMacro ( text ) == True )
 	    LoadMacro ( text, (FCT(void,(*),(void *)))ie_call_function,
 		       (void*) func, repeat_num );
@@ -480,7 +480,6 @@ static void f_ascii ( text, infos )
 	  if ( (type == EXTERN_FUNC) && (func != 0 )) {
 	    int nw = text->win_id;
 	    
-/*	    CheckModifAndFilename ( text ); */
 	    if ( InsideMacro ( text ) == True )
 	      LoadMacro ( text, (FCT(void,(*),(void *)))ie_call_function,
 			 (void*) func, repeat_num );
@@ -496,7 +495,6 @@ static void f_ascii ( text, infos )
 	}
 	else if ( ( infos -> ch < 256 /* pas XK */) && isprint ( infos -> ch ) ) {
 	  int repeat = repeat_num;
-	  
 	  f_impc ( text, (int) infos -> ch );
 	  if ( InsideMacro ( text ) == True )
 	    LoadMacro ( text, (void(*)()) f_impc, (void *) infos -> ch, repeat );
@@ -550,7 +548,6 @@ static void f_ctrx ( text, infos )
 	if ( (type == EXTERN_FUNC) && (func != 0 )) {
 	    int nw = text->win_id;
 	    
-/*	    CheckModifAndFilename ( text ); */
 	    if ( InsideMacro ( text ) == True )
 	      LoadMacro ( text, (FCT(void,(*),(void *)))ie_call_function,
 			 (void*) func, repeat_num );
@@ -606,7 +603,6 @@ static void f_ctr ( text, infos )
       if ( (type == EXTERN_FUNC) && (func != 0 )) {
 	int nw = text->win_id;
 	
-/*	CheckModifAndFilename ( text ); */
 	if ( InsideMacro ( text ) == True )
 	  LoadMacro ( text, (FCT(void,(*),(void *)))ie_call_function,
 		     (void*) func, repeat_num );
@@ -656,7 +652,6 @@ static void f_escape ( text, infos )
     if ( (type == EXTERN_FUNC) && (func != 0 )) {
       int nw = text->win_id;
       
-/*      CheckModifAndFilename ( text ); */
       if ( InsideMacro ( text ) == True )
 	LoadMacro ( text, (FCT(void,(*),(void *)))ie_call_function,
 		   (void*) func, repeat_num );
@@ -882,23 +877,23 @@ void DisplayLineNumber( text )
 **      Input : the XK name
 **      Output : the XK psuedo code (>= 256) or 0
 */
-int Xk_Code( name )
-    char * name;
+unsigned Xk_Code( name )
+    unsigned char * name;
 {
-  int i;
+  unsigned i;
   
   for (i = 0; XKInfos[i].name; i += 1)
-    if (! strcasecmp(XKInfos[i].name, name))
+    if (! strcasecmp(XKInfos[i].name, (char *) name))
       /* appelle XK_ksym_to_code pour gerer les ksym egaux */
       return XK_ksym_to_code(XKInfos[i].xk_code);
     
   return 0;
 }
 
-int XK_ksym_to_code(ksym)
+unsigned XK_ksym_to_code(ksym)
     KeySym ksym;
 {
-  int i;
+  unsigned i;
   
   for (i = 0; XKInfos[i].name; i += 1)
     if (XKInfos[i].xk_code == ksym)
@@ -906,3 +901,4 @@ int XK_ksym_to_code(ksym)
     
   return 0;
 }
+
