@@ -44,6 +44,7 @@
 
 #include "init_menus.h"
 #include "handle_menus.h"
+#include "shadow.h"
 
 extern XContext	SwContext;
 static int button_press;
@@ -58,8 +59,6 @@ FCT (static void, show_item, (Display *display, XYMenu *menu, int i, int n) );
 FCT (static void, clear_menu, (Display *display, XYMenu *menu, int n) );
 FCT (static void, show_menu, (Display *display, XYMenu *menu, int n) );
 
-extern void Display3D ();
-
 /*
 **	Function name : HandleMenu
 **
