@@ -41,7 +41,12 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+
+#if HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
+#endif
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -62,6 +67,7 @@
 #include "mark_reg.h"
 #include "chars_cmds.h"
 #include "warn_box.h"
+#include "selection.h"
 
 FCT (static void, MatchedAt, (Text *text, int c, int lines, int pos) );
 
@@ -378,6 +384,9 @@ void ie_insert_char ( text, c )
 {
     char the_c = (char) c;
     
+    if (text -> selection -> select == True)
+      ClearSelection(text, True);
+
     if ( text -> shell_id != 0) {
 	if ((char) c == '\n') {
 	    shell_return ( text );
@@ -421,8 +430,11 @@ void ie_delete_char (text )
     Text *text;
 {
     char c;
-    int mark = 0;;
-    
+    int mark = 0;
+
+    if (text -> selection -> select == True)
+      ClearSelection(text, True);
+		 
     if ( ie_at_end_of_file ( text ) == True )
       return;
     
@@ -662,6 +674,9 @@ int ie_cmd_shell ( text, str )
 	return (-1);
     }
 
+    if (text -> selection -> select == True)
+      ClearSelection(text, True);
+    
     while (fgets ( tmp,BIG_LINE, fd ) != 0 ) {
 	len = strlen(tmp);
 	modif_flag = True;
@@ -797,18 +812,8 @@ void ie_set_mode_font ( text, mode_name, font_name )
     SetModeFont ( mode, font_name );
 }
 
-/*
-**	Function name : ie_key_def
-**
-**	Description : Pour les liasons de cles.
-**             Pas triste la fonction.
-**	Input : Le text courant, le mode, la liaison de cles
-**         la fonction associee.
-**	Ouput :
-*/
-
-int key_code( keys )
-    char *keys;
+unsigned key_code( keys )
+    unsigned char *keys;
 {
   if (keys[0] == '^')
     switch (keys[1]) {
@@ -824,21 +829,32 @@ int key_code( keys )
 	return keys[1] - 'a' + 1;
       if ((keys[1] >= 'A') && (keys[1] <= 'Z'))
 	return keys[1] - 'A' + 1;
-      return -1;
+      return ~0;
   }
   {
-    int xk = Xk_Code(keys);
+    unsigned xk = Xk_Code(keys);
     
     return (xk) ? xk : keys[0];
   }
 }
 
+/*
+**	Function name : ie_key_def
+**
+**	Description : Pour les liasons de cles.
+**             Pas triste la fonction.
+**	Input : Le text courant, le mode, la liaison de cles
+**         la fonction associee.
+**	Ouput :
+*/
 void ie_key_def ( text, mode_name, keys, func_name )
 	Text *text;
-	char *mode_name, *keys, *func_name;
+	char *mode_name;
+	unsigned char *keys;
+	char *func_name;
 {
     Mode *mode;
-    int c;
+    unsigned c;
     void (* f) ();
 
     if ( (mode_name == 0) || (keys == 0) )
@@ -866,14 +882,14 @@ void ie_key_def ( text, mode_name, keys, func_name )
     */
 
     if ( keys [0] == '^' ) {
-	if ( strlen (keys) < 2 )
+	if ( strlen ((char *)keys) < 2 )
 	  return;
 	switch ( keys [1] ) {
 	case '[':
 	  /* Escape .. */
-	  if ( strlen (keys) < 3 )
+	  if ( strlen ((char *)keys) < 3 )
 	    return;
-	  if ((c = key_code(&keys[2])) == -1)
+	  if ((c = key_code(&keys[2])) == ~0)
 	    return;
 	  /* La fonction est-elle built_in ou extern ? */
 	  if ( func_name == 0 ) {
@@ -893,9 +909,9 @@ void ie_key_def ( text, mode_name, keys, func_name )
 	case 'X':
 	case 'x':
 	  /* Controle X .. */
-	  if ( strlen (keys) < 3 )
+	  if ( strlen ((char *)keys) < 3 )
 	    return;
-	  if ((c = key_code(&keys[2])) == -1)
+	  if ((c = key_code(&keys[2])) == ~0)
 	    return;
 	  /* La fonction est-elle built_in ou extern ? */
 	  if ( func_name == 0 ) {
@@ -914,9 +930,9 @@ void ie_key_def ( text, mode_name, keys, func_name )
 	  break;
 	default:
 	  /* ------Controle Key------ */
-	  if ( strlen ( keys ) != 2 )
+	  if ( strlen ( (char *)keys ) != 2 )
     	    return;
-	  if ((c = key_code(&keys[0])) == -1)
+	  if ((c = key_code(&keys[0])) == ~0)
 	    return;
 	  	  
 	  /* La fonction est-elle built_in ou extern */
@@ -938,12 +954,12 @@ void ie_key_def ( text, mode_name, keys, func_name )
     }
     else {
 	/* ------Caractere simple------ */
-	if ( strlen ( keys ) != 1 ) {
+	if ( strlen ( (char *)keys ) != 1 ) {
 	  if ((c = Xk_Code(keys)) == 0)
 	    return;
 	}
 	else
-	  c = (int) keys [0];
+	  c = keys [0];
 	
 	/* La fonction est-elle built_in ou extern ? */
 	if ( func_name == 0 ) {
@@ -1187,6 +1203,9 @@ int ie_insert_file ( text, filename )
     int nbl, olen;
     extern Display *dpy;
     int x_len;
+
+    if (text -> selection -> select == True)
+      ClearSelection(text, True);
     
     TextCursorOff ( text );
     nbl = text -> lines_in_buf;
@@ -1230,6 +1249,11 @@ char *ie_file_select ( text )
     char *str;
     char c = '\007'; /* ^G */
     
+    if (text -> selection -> select == True){
+      SetCurrentLine (text);
+      ClearSelection(text, True);
+    }
+
     str = (char *) SelectFileFromBox ( "Select file from Smac." );
     if ( (str == 0) || (strncmp(str, &c, 1)) == 0 ) {
 	if ( str != 0 )
@@ -1825,7 +1849,7 @@ void ie_color_area ( text, start, end, colorname )
 }
 
 /*
-**	Function name : ie_past_region
+**	Function name : ie_paste_region
 **
 **	Description :
 **	Input :
@@ -1834,7 +1858,7 @@ void ie_color_area ( text, start, end, colorname )
 void ie_paste_region ( text )
     Text *text;
 {
-  
+  smac_paste_region(text);
 }
 
 /*
@@ -1847,7 +1871,7 @@ void ie_paste_region ( text )
 void ie_copy_region ( text )
     Text *text;
 {
-  
+  smac_copy_region(text);
 }
 
 /*
@@ -1860,7 +1884,7 @@ void ie_copy_region ( text )
 void ie_cut_region ( text )
     Text *text;
 {
-  
+  smac_kill_region(text);
 }
 
 /*
