@@ -41,15 +41,18 @@
 #include <stdio.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
-#ifndef apollo
-#ifdef __FreeBSD__
-#include <stdlib.h>
-#else /* !__FreeBSD__ */
-#include <malloc.h>
-#endif
+
+#if STDC_HEADERS
+#  include <stdlib.h>
+#else
+#  if HAVE_MALLOC_H
+#    include <malloc.h>
+#  endif
 #endif
 
 #include "init_menus.h"
+#include "shadow.h"
+#include "parse.h"
 
 XContext		SwContext;
 XContext		TiContext;
@@ -61,8 +64,6 @@ FCT (static char *, getmaxstr, (char **tstr) );
 FCT (static void, show_bar, (Display *display, Window w, XYMenu *menu) );
 FCT (static VMenu *, Vmenu, (Display *display, Window window,XYMenu *menu, char **item, int x_title, void (**fnt) (), unsigned long bg) );
 
-extern void Display3D ();
-
 /*
 **	Function name : InitMenuRes
 **
@@ -171,7 +172,7 @@ XYMenu *MakeMenus ( display, w, title, item, fnt )
   menu -> y_menu = menu -> bar_height + 2;		
   menu -> titlename 	= title;
   menu -> itemname 	= item;
-  menu -> y_title = rm.font -> ascent +  1;
+  menu -> y_title = rm.font -> ascent + 3;
   menu -> parent = w;
   menu -> n_last_unmapped = -1;
   menu -> hmax_menu = 0;
