@@ -39,19 +39,37 @@
 
 
 #include <stdio.h>
-#ifdef __NetBSD__
 #include <pwd.h>
-#include <dirent.h>
-#else
-#include <dirent.h>
-#include <pwd.h>
-#endif
+
+#if HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
+#endif
+
+#if HAVE_DIRENT_H
+#  include <dirent.h>
+#  define NAMLEN(dirent) strlen((dirent)->d_name)
+#else
+#  define dirent direct
+#  define NAMLEN(dirent) (dirent)->d_namlen
+#  if HAVE_SYS_NDIR_H
+#    include <sys/ndir.h>
+#  endif
+#  if HAVE_SYS_DIR_H
+#    include <sys/dir.h>
+#  endif
+#  if HAVE_NDIR_H
+#    include <ndir.h>
+#  endif
+#endif
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
+#include <X11/Xatom.h>
 #include <stdlib.h>
 #include <sys/stat.h>
+#include <sys/param.h>
 
 #include "main_text.h"
 #include "handle_key.h"
@@ -94,10 +112,13 @@ FCT (static int, CommonLength, (char *s, int ns, int r) );
 FCT (static void, ConfigInputString, () );
 FCT (static char *, ExpandFileName, (char *s, int *n, int *common) );
 FCT (static void, ExposeInputString, () );
-FCT (static void, InputStrSave, (char *s, int len) );
+FCT (void, InputStrSave, (char *s, int len) );
 FCT (static void, InputStrUnmap, () );
 FCT (static void, PrintMatchedNames, (char *s, int combien) );
 
+extern Atom prop_sel;
+extern Window w_sel;
+
 /*
 **	Function name : InitInputString
 **
@@ -145,17 +166,17 @@ void InitInputString ()
 	input_str -> current_mode -> esc_key [i].type = 0;
     }
     
-    ie_key_def ( input_str,"input_str" , "^A", "goto_beginning_of_line" );
-    ie_key_def ( input_str,"input_str" , "^B", "goto_previous_char" );
-    ie_key_def ( input_str,"input_str" , "^D", "delete_char" );
-    ie_key_def ( input_str,"input_str" , "^E", "goto_end_of_line" );
-    ie_key_def ( input_str,"input_str" , "^F", "goto_next_char" );
-    ie_key_def ( input_str,"input_str" , "^G", "str_abort" );
-    ie_key_def ( input_str,"input_str" , "^H", "delete" );
-    ie_key_def ( input_str,"input_str" , "^K", "kill_line" );
-    ie_key_def ( input_str,"input_str" , "^M", "str_return" );
-    ie_key_def ( input_str,"input_str" , "^I", "str_tab" );
-    ie_key_def ( input_str,"input_str" , "^P", "str_previous" );
+    ie_key_def ( input_str,"input_str" , (unsigned char *)"^A", "goto_beginning_of_line" );
+    ie_key_def ( input_str,"input_str" , (unsigned char *)"^B", "goto_previous_char" );
+    ie_key_def ( input_str,"input_str" , (unsigned char *)"^D", "delete_char" );
+    ie_key_def ( input_str,"input_str" , (unsigned char *)"^E", "goto_end_of_line" );
+    ie_key_def ( input_str,"input_str" , (unsigned char *)"^F", "goto_next_char" );
+    ie_key_def ( input_str,"input_str" , (unsigned char *)"^G", "str_abort" );
+    ie_key_def ( input_str,"input_str" , (unsigned char *)"^H", "delete" );
+    ie_key_def ( input_str,"input_str" , (unsigned char *)"^K", "kill_line" );
+    ie_key_def ( input_str,"input_str" , (unsigned char *)"^M", "str_return" );
+    ie_key_def ( input_str,"input_str" , (unsigned char *)"^I", "str_tab" );
+    ie_key_def ( input_str,"input_str" , (unsigned char *)"^P", "str_previous" );
 
     input_str -> current_mode -> key [DELETE].func = ie_WR_delete;
     
@@ -184,8 +205,8 @@ void InitInputString ()
 **	Input : 
 **	Ouput :
 */
-char *InputString ( parent, gc, font, prompt, reply )
-    Window parent;
+char *InputString ( parent, raise, gc, font, prompt, reply )
+    Window parent, raise;
     GC gc;
     XFontStruct *font;
     char *prompt;
@@ -245,6 +266,12 @@ char *InputString ( parent, gc, font, prompt, reply )
 	}
 	XNextEvent ( dpy, &ev );
 	switch ( ev.type ) {
+	case VisibilityNotify:
+	  if (ev.xvisibility.state == VisibilityUnobscured) {
+	    if (raise)
+	      XMapRaised (dpy, raise);
+	  }
+	  break;
 	case Expose:
 	  if ( ev.xany.window == parent ) {
 	    XDrawImageString ( dpy, parent , gc, 10,
@@ -266,6 +293,11 @@ char *InputString ( parent, gc, font, prompt, reply )
 	      HandleExpose ();
 	  }
 	  break;
+	case SelectionClear:
+	      XPutBackEvent ( dpy, &ev );
+	      XNextEvent ( dpy, &event );
+	      HandleSelectionClear();
+	  break;
 	case ConfigureNotify:
 	      XPutBackEvent ( dpy, &ev );
 	      XNextEvent ( dpy, &event );
@@ -289,7 +321,8 @@ char *InputString ( parent, gc, font, prompt, reply )
 	  case Button2:
 	    if ( ev.xany.window == input_str -> window ) {
 		if ( ev.xbutton.state == ControlMask ) {
-		    s = XFetchBuffer ( dpy, &nbytes, 0 );
+/*		    s = XFetchBuffer ( dpy, &nbytes, 0 ); */
+		  s = Fetch_bytes (&nbytes);
 		    if ( (s != 0) && (nbytes != 0 ) && (input_toggle_del == False) ) {
 			if ( (RightBuf ( input_str -> buf ) + nbytes) > BottomBuf ( input_str -> buf ) ) {
 			    nbytes = BottomBuf ( input_str -> buf ) - RightBuf ( input_str -> buf );
@@ -305,16 +338,24 @@ char *InputString ( parent, gc, font, prompt, reply )
 		    input_toggle_del = True;
 		}
 		else {
-		    s = XFetchBuffer ( dpy, &nbytes, 0 );
+/*		    s = XFetchBuffer ( dpy, &nbytes, 0 ); */
+		  if ( w_sel != None ) { /* xcoral a la selection */
+		    s = Fetch_bytes (&nbytes);
 		    if ((s != 0) && (nbytes != 0 )) {
-			lines = GetNewLine ( s, nbytes);
-			if ( lines == 0 )
-			  GetBytesFromCutBuffer ( input_str );
-			else
-			  klaxon ();
+		      lines = GetNewLine ( s, nbytes);
+		      if ( lines == 0 )
+			GetBytesFromCutBuffer ( input_str );
+		      else
+			klaxon ();
 		    }
 		    else
 		      klaxon ();
+		  }
+		  else {
+		    /* Allons chercher les donnees de l'autre client */
+		    XConvertSelection (dpy, XA_PRIMARY, XA_STRING, prop_sel,
+				       ev.xbutton.window, ev.xbutton.time);
+		  }
 		}
 	    }
 	    else {
@@ -335,6 +376,41 @@ char *InputString ( parent, gc, font, prompt, reply )
 			    parent_y + input_str -> font -> ascent,
 			    prompt, strlen (prompt));
 	  break;
+	case SelectionNotify:
+	  if (ev.xselection.property == None){
+	    klaxon();
+	    break;
+	  }
+
+	  if ( ev.xselection.selection == XA_PRIMARY ) {
+	    Atom type_req;
+	    int format_lu;
+	    unsigned long nb_lus, octets_restants;
+	    unsigned char *prop_lue;
+	    
+	    XGetWindowProperty (dpy, ev.xselection.requestor,
+				ev.xselection.property, 0, 8192, False,
+				ev.xselection.target, &type_req, 
+				&format_lu, &nb_lus, &octets_restants, &prop_lue);
+	    if (nb_lus > 0) {
+	      Store_bytes((char *)prop_lue, nb_lus);
+	      XFree(prop_lue);
+	      XDeleteProperty (dpy, ev.xselection.requestor,
+			       ev.xselection.property);
+	      
+	      s = Fetch_bytes (&nbytes);
+	      if ((s != 0) && (nbytes != 0 )) {
+		lines = GetNewLine ( s, nbytes);
+		if ( lines == 0 )
+		  GetBytesFromCutBuffer ( input_str );
+		else
+		  klaxon ();
+	      }
+	      else
+		klaxon ();
+	    }
+	  }
+	  break;
 	case EnterNotify:
 	  if ( ev.xany.window == input_str -> window ) {
 	      MouseIn ( input_str );
@@ -429,6 +505,11 @@ char *GetString ( text, prompt, reply )
     int len = XTextWidth (input_str -> font,
 			  prompt, strlen (prompt)) + 10;
     
+    if (text -> selection -> select == True) {
+      SetCurrentLine (text);
+      ClearSelection(text, True);
+    }
+
     /* On verifie qu'on peut afficher le mini-buffer */
     if ( (text -> mwin -> twidth - len) < 100 ) {
         /* 100 pixels, c'est le minimum, 7,8,9 caracteres
@@ -438,7 +519,7 @@ char *GetString ( text, prompt, reply )
     else {
         /* Mini-Buffer */
 	return ( (char *)
-		InputString ( text -> mwin -> mess,
+		InputString ( text -> mwin -> mess, 0,
 			     text -> mwin -> mess_gc,
 			     text -> mwin -> font, prompt, reply ));
     }
@@ -545,7 +626,7 @@ static void ExposeInputString ()
 **	Input : 
 **	Ouput :
 */
-static void InputStrSave ( s, len )
+void InputStrSave ( s, len )
     char *s;
     int len;
 {
@@ -628,7 +709,7 @@ static char * ExpandFileName (s, n, common)
     *n = 0;
     
     {
-	char *tmp, *fsdirname, *newdir;
+	char *fsdirname, *newdir;
 	char *slash;
 	
 	fsdirname = (char *) GetFsBoxInternalDir();
@@ -685,7 +766,7 @@ static char * ExpandFileName (s, n, common)
     while ((entry = readdir(dp)))
       if (strstr(entry->d_name, s) == entry->d_name) {
 	char * nresult =
-	  (char *) malloc((unsigned int) rlength + strlen(entry->d_name) + 1 + 1);
+	  (char *) malloc((unsigned int) rlength + NAMLEN(entry) + 1 + 1);
 	
 	if (! nresult) {
 	    (void) closedir (dp);
@@ -695,11 +776,11 @@ static char * ExpandFileName (s, n, common)
 	
 	bcopy(result, nresult, rlength);
 	(void) strcpy(nresult + rlength, entry->d_name);
-	rlength += strlen(entry->d_name) + 1;
+	rlength += NAMLEN(entry) + 1;
 	*n += 1;
 	{
 	  struct stat st;
-	  char * f = malloc(strlen(current_dir) + 1 + strlen(entry->d_name) + 1);
+	  char * f = malloc(strlen(current_dir) + 1 + NAMLEN(entry) + 1);
 	  
 	  if (f) {
 	    sprintf(f, "%s/%s", current_dir, entry->d_name);
@@ -858,13 +939,13 @@ char *ExpandTildeName ( name )
     struct passwd *pwd;
     char *home;
     char *p = name + 1;
-    static char  buf [256];
+    static char  buf_dir_tmp [MAXPATHLEN];
     
     if ( name [1] == '/' || name [1] == '\0') {   /* c'est moi */
 	if ( (home = (char *) getenv ( "HOME" )) == 0 )
 	  return name;
-	(void) strcpy ( buf, home );
-	(void) strcat ( buf, name + 1 );
+	(void) strcpy ( buf_dir_tmp, home );
+	(void) strcat ( buf_dir_tmp, name + 1 );
     }
     else { /* c'est un autre */
 	int save;
@@ -875,13 +956,13 @@ char *ExpandTildeName ( name )
 	pwd = getpwnam ( name + 1 );
 	*p = save;
 	if (  pwd ) {
-	    (void) strcpy ( buf, pwd ->  pw_dir);
-	    (void) strcat ( buf, p );
+	    (void) strcpy ( buf_dir_tmp, pwd ->  pw_dir);
+	    (void) strcat ( buf_dir_tmp, p );
 	}
 	else
 	  return name;
     }
-    return buf;
+    return buf_dir_tmp;
 }
 
 /*
