@@ -38,15 +38,21 @@
    ######################################################################## */
 
 
-#include <stdio.h>
+#if HAVE_STRING_H
 #include <string.h>
-#include <X11/Xlib.h>
-#ifndef apollo
-#ifdef __FreeBSD__
-#include <stdlib.h>
-#else /* !__FreeBSD__ */
-#include <malloc.h>
+#else
+#include <strings.h>
 #endif
+
+#include <stdio.h>
+#include <X11/Xlib.h>
+
+#if STDC_HEADERS
+#  include <stdlib.h>
+#else
+#  if HAVE_MALLOC_H
+#    include <malloc.h>
+#  endif
 #endif
 
 #include "main_text.h"
