@@ -44,7 +44,7 @@
 #include <sys/types.h>
 #include <fcntl.h>
 #include <sys/time.h>
-#ifdef _AIX
+#ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
 
@@ -52,6 +52,7 @@
 #include <X11/cursorfont.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
+#include <X11/Xatom.h>
 
 #include "xcoral.h"
 #include "browser_init.h"
@@ -70,6 +71,7 @@
 #include "chars_cmds.h"
 #include "cb_names.h"
 #include "get_file.h"
+#include "selection.h"
 
 EdWin	*CreateWindow ();
 Window	HandleMenu ();
@@ -86,14 +88,16 @@ XEvent event;
 static XEvent *ev = &event;
 
 extern void exit ();
-extern void ScrollNLine ();
+extern int ToolBarButton();
 
 FCT (static void, ButtonPressInControl, (XEvent *ev, Text *text, int i) );
 FCT (static void, HandleKeyPress, () );
 FCT (static void, HandleKeyRelease, () );
 FCT (static void, HandleMap, () );
 FCT (static void, HandleMessage, () );
-FCT (static void, HandleSelection, () );
+FCT (static void, HandleSelectionRequest, () );
+FCT (static void, HandleSelectionNotify, () );
+FCT (static void, HandlePropertyNotify, () );
 FCT (static void, IgnoreEvent, () );
 FCT (static void, ButtonUserFunc, (Text *text, int x, int y, char *func_name) );
 
@@ -102,12 +106,20 @@ void (* events_Handler[MAX_EVENT]) ();
 static void IgnoreEvent () {}
 static int lock_menus = False;
 static Text *lock_buttons = 0;
-
+Window w_sel = None;
+Atom prop_sel;
 jmp_buf br_env;
-
-fd_set readfds, save_readfds; 
 int width_fd;
 
+fd_set readfds, save_readfds;
+
+#if defined(fd_set)
+#define FD_SET(n, p)    ((*p) |= (1 << (n)))
+#define FD_CLR(n, p)    ((*p) &= ~(1 << (n)))
+#define FD_ISSET(n, p)  ((*p) & (1 << (n)))
+#define FD_ZERO(p)      ((*p) = 0)
+#endif
+
 /*
 **	Function name : InitEvent
 **
@@ -130,7 +142,12 @@ void InitEvent ()
     events_Handler [Expose] = HandleExpose;
     events_Handler [MappingNotify] = HandleMap;
     events_Handler [ClientMessage] = HandleMessage;
-    events_Handler [SelectionRequest] = HandleSelection;
+    events_Handler [SelectionRequest] = HandleSelectionRequest;
+    events_Handler [SelectionNotify] = HandleSelectionNotify;
+    events_Handler [SelectionClear] = HandleSelectionClear;
+    events_Handler [PropertyNotify] = HandlePropertyNotify;
+    
+    prop_sel = XInternAtom (dpy, "xcoral_prop", False);
 }
 
 /*
@@ -143,7 +160,7 @@ void InitEvent ()
 */
 void WaitForEvent ()
 {
-#if defined(SVR4) || defined(UNIXWARE)
+#if HAVE_SIGPROCMASK
 	sigset_t block, oblock;
 #else
     int old_mask;
@@ -158,7 +175,7 @@ void WaitForEvent ()
     for (;;) {
 	while ( XPending ( dpy ) == 0 ) {
 	    /* Pour un ^C eventuel */
-#if defined(SVR4) || defined(UNIXWARE)
+#if HAVE_SIGPROCMASK
 		(void)sigemptyset(&block);
 		(void)sigaddset(&block, SIGINT);
 		if (sigprocmask(SIG_BLOCK, &block, &oblock) < 0)
@@ -191,7 +208,7 @@ void WaitForEvent ()
 	      }
 	      break;
 	    }
-#if defined(SVR4) || defined(UNIXWARE)
+#if HAVE_SIGPROCMASK
 		(void)sigprocmask(SIG_SETMASK, &oblock, (sigset_t *)NULL);
 #else
 		(void) sigsetmask(old_mask);
@@ -276,7 +293,7 @@ void WaitForMapped ( win, istext )
 **	Input :
 **	Output :
 */
-WaitForXserver()
+void WaitForXserver()
 {
     XSync ( dpy, False );
 
@@ -331,18 +348,18 @@ static void ButtonPressInControl ( ev, text, i )
     
     DownButton ( ev -> xbutton.window );
 
-    XGrabPointer ( dpy, ev -> xbutton.window, True,
-		  ButtonPressMask, GrabModeAsync,GrabModeAsync,
-		  ev -> xbutton.window, None, CurrentTime );
+/*    XGrabPointer ( dpy, ev -> xbutton.window, True, */
+/*		  ButtonPressMask, GrabModeAsync,GrabModeAsync, */
+/*		  ev -> xbutton.window, None, CurrentTime ); */
 
     ExecButtonFunction ( text, i );
     
     if ( (i==2) || (i==3) ) {
 	UpButton ( ev -> xbutton.window );
-	XUngrabPointer ( dpy, CurrentTime );
+/*	XUngrabPointer ( dpy, CurrentTime ); */
 	return;
     }
-    XUngrabPointer ( dpy, CurrentTime );
+/*    XUngrabPointer ( dpy, CurrentTime ); */
     UpButton ( ev -> xbutton.window );
 }
 
@@ -394,19 +411,140 @@ static void HandleMap ()
 }
 
 /*
-**	Function name : HandleSelection
+**	Function name : HandleSelectionRequest
 **
 **	Description :
 **	Input :
 **	Output :
 */
-static void HandleSelection()
+static void HandleSelectionRequest()
 {
+  XEvent ev_not;
+  int nbytes = 0;
+  char *s;
+  
 #ifdef DEBUG
   (void) fprintf ( stderr, "SelectionRequest\n" );
 #endif
+
+  s = Fetch_bytes (&nbytes);
+
+  if (s == 0 || nbytes == 0) {
+#ifdef DEBUG
+  (void) fprintf ( stderr, "SelectionRequest null\n" );
+#endif
+    return;
+  }
+  
+  ev_not.type = SelectionNotify;
+  ev_not.xselection.requestor = ev -> xselectionrequest.requestor;
+  ev_not.xselection.selection = ev -> xselectionrequest.selection;
+  ev_not.xselection.target = ev -> xselectionrequest.target;
+  ev_not.xselection.time = ev -> xselectionrequest.time;
+  ev_not.xselection.property = ev -> xselectionrequest.property;
+
+  if ((ev -> xselectionrequest.selection == XA_PRIMARY)
+      &&(ev -> xselectionrequest.target)
+      &&(ev -> xselectionrequest.property)) {
+    XChangeProperty (dpy, ev_not.xselection.requestor,
+		   ev_not.xselection.property,
+		   ev_not.xselection.target,
+		     8,
+		     PropModeReplace,
+		     s,
+		     nbytes );
+  }
+  else {
+    ev_not.xselection.property = None;
+  }
+  
+  XSendEvent (dpy, ev_not.xselection.requestor, False, 0, &ev_not);
 }
 
+/*
+**	Function name : HandleSelectionNotify
+**
+**	Description :
+**	Input :
+**	Output :
+*/
+static void HandleSelectionNotify()
+{
+#ifdef DEBUG
+  (void) fprintf ( stderr, "SelectionNotify\n" );
+#endif
+
+  if (ev -> xselection.property == None){
+    klaxon();
+    return;
+  }
+  
+  if ( ev -> xselection.selection == XA_PRIMARY ) {
+    Atom type_req;
+    int format_lu;
+    unsigned long nb_lus, octets_restants;
+    unsigned char *prop_lue;
+    
+    XGetWindowProperty (dpy, ev -> xselection.requestor,
+			ev -> xselection.property, 0, 8192, False,
+			ev -> xselection.target, &type_req, 
+			&format_lu, &nb_lus, &octets_restants, &prop_lue);
+    if (nb_lus > 0) {
+      Store_bytes((char *)prop_lue, nb_lus);
+      XFree(prop_lue);
+      XDeleteProperty (dpy, ev -> xselection.requestor,
+		       ev -> xselection.property);
+      
+      if ( strcmp( edwin ->text -> current_mode -> name, "Shell") == 0)
+	SetPosInShell (edwin ->text);
+      GetBytesFromCutBuffer ( edwin -> text );
+    }
+  }
+}
+
+/*
+**	Function name : HandlePropertyNotify
+**
+**	Description :
+**	Input :
+**	Output :
+*/
+static void HandlePropertyNotify()
+{
+#ifdef DEBUG
+  (void) fprintf ( stderr, "PropertyNotify\n" );
+#endif
+}
+
+/*
+**	Function name : HandleSelectionClear
+**
+**	Description :
+**	Input :
+**	Output :
+*/
+void HandleSelectionClear()
+{
+  EdWin	**t;
+#ifdef DEBUG
+  (void) fprintf ( stderr, "SelectionClear\n" );
+#endif
+/*  Clear_bytes(); */
+  w_sel = None;
+
+  /* Efface la selection courante si elle existe */
+  for ( t = TWin; t < TWin + MAXWIN; t++ ) {
+    if ( *t == 0 ) continue;
+    if ((*t) -> text -> selection -> select == True) {
+      SetCurrentLine ((*t) -> text);
+      (void) ClearSelection ((*t) -> text, True);
+      SetCurrentLine ((*t) -> text);
+      TextCursorOn((*t) -> text);
+      FreezeTextCursor((*t) -> text);
+      break;
+    }
+  }
+}
 
 /*
 **	Function name : HandleConfigure
@@ -492,7 +630,7 @@ void HandleEnter ()
 {
     EdWin **t;
     int trouve = False;
-    
+
 #ifdef DEBUG
     ( void ) fprintf ( stderr, "Enter Notify %d mode = %d\n", 
 		      ev -> xcrossing.window,
@@ -618,7 +756,7 @@ static void HandleKeyPress ()
     }
     if ( tmp == 0 )
       return;
-    
+
     stat = (ST *) automate ( edwin -> text, ( XKeyEvent *) ev, edwin -> stat );
     if ( (int) stat == -2 ) {
 	/* Ya eu un changement de contexte */
@@ -662,6 +800,7 @@ void HandleButtonPress ()
     int vm, item;
     Window w_stat;
     int result;
+    EdWin **t;
     
 #ifdef DEBUG
     ( void ) fprintf ( stderr, "ButtonPress %d\n", ev -> xbutton.window );
@@ -686,6 +825,7 @@ void HandleButtonPress ()
 	klaxon ();
 	return;
     }
+
     /*
        * Button press in menus
     */
@@ -701,6 +841,10 @@ void HandleButtonPress ()
 	}
 	return;
     }
+    if ( ToolBarButton ( edwin, (XButtonEvent *) ev ) == True ) {
+      return;
+    }
+    
     /*
        * Button press in button window,
     */
@@ -762,10 +906,6 @@ void HandleButtonPress ()
 	    break;
 	  default:
 	    TextCursorOff ( edwin -> text );
-/*	    
-	    (void) MoveToXYinTextWindow ( edwin -> text,
-		       ev -> xbutton.x, ev -> xbutton.y );
-*/
 	    StorePosition ( edwin -> text );
 	    if ( MoveToXYinTextWindow ( edwin -> text,
 				       ev -> xbutton.x, ev -> xbutton.y ) == -1 ) {
@@ -775,7 +915,38 @@ void HandleButtonPress ()
 	      UpdatePage ( edwin -> text );
 	    }
 	    TextCursorOn ( edwin -> text );
-	    break;
+	    (void) HandleSelection (edwin -> text, ev);
+	    if (edwin -> text -> selection -> select == True) {
+	      /* Il faut effacer la derniere selection si elle existe */
+	      for ( t = TWin; t < TWin + MAXWIN; t++ ) {
+		if ( *t == 0 ) continue;
+		if ((*t) -> text -> selection -> select == True) {
+		  if ((*t) -> text != edwin -> text) {
+		    SetCurrentLine ((*t) -> text);
+		    (void) ClearSelection ((*t) -> text, True);
+		    SetCurrentLine ((*t) -> text);
+		    TextCursorOn((*t) -> text);
+		    FreezeTextCursor((*t) -> text);
+		    break;
+		  }
+		}
+	      }
+	      TextCursorOn ( edwin -> text );
+	      (void) BrowserClearSelection();
+	      w_sel = ev -> xbutton.window;
+	      XSetSelectionOwner ( dpy, XA_PRIMARY, w_sel, ev -> xbutton.time);
+	      if ( w_sel == XGetSelectionOwner ( dpy, XA_PRIMARY )) {
+#ifdef DEBUG
+		(void) fprintf (stderr, "Selection owner OK\n");
+#endif /* DEBUG */	      
+	      }
+	      else {
+#ifdef DEBUG
+		(void) fprintf (stderr, "Selection owner failed\n");
+#endif /* DEBUG */
+		w_sel = None;
+	      }
+	    }
 	  }
 	  break;
 	case Button2:
@@ -788,10 +959,8 @@ void HandleButtonPress ()
 		ev -> xbutton.x, ev -> xbutton.y, "middle_button_shift" );
 	    break;
 	  default:
-	      if ( strcmp( edwin ->text -> current_mode -> name, "Shell") == 0)
-		SetPosInShell (edwin ->text);
-	      GetBytesFromCutBuffer ( edwin -> text );
-	      break;
+	    PasteSelection(edwin->text);
+	    break;
 	  }
 	  break;
 	case Button3:
@@ -807,16 +976,52 @@ void HandleButtonPress ()
 	  default:
 	    TextCursorOff ( edwin -> text );
 	    XSync ( dpy, False );
-	    StoreBytesInCutBuffer ( edwin -> text,
-				 ev -> xbutton.x, ev -> xbutton.y );
+	    if (edwin -> text -> selection -> select == True) {
+	      StorePosition ( edwin -> text );
+	      if ( MoveToXYinTextWindow ( edwin -> text,
+					 ev -> xbutton.x, ev -> xbutton.y ) == -1 ) {
+		HoleToRight ( edwin -> text -> buf );
+		GotoLineNumber ( edwin -> text, edwin -> text -> lines_in_buf );
+		SetPosition ( edwin -> text );
+		UpdatePage ( edwin -> text );
+	      }
+	      (void) UpdateSelection(edwin -> text);
+	    }
+	    else {
+	      StoreBytesInCutBuffer ( edwin -> text,
+				     ev -> xbutton.x, ev -> xbutton.y );
+	    }
 	    TextCursorOn ( edwin -> text );
-	    break;
+	    RefreshSelection(edwin -> text);
+	    if (edwin -> text -> selection -> select == True) {
+	      /* Il faut effacer la derniere selection si elle existe */
+	      for ( t = TWin; t < TWin + MAXWIN; t++ ) {
+		if ( *t == 0 ) continue;
+		if ((*t) -> text -> selection -> select == True) {
+		  if ((*t) -> text != edwin -> text) {
+		    SetCurrentLine ((*t) -> text);
+		    (void) ClearSelection ((*t) -> text, True);
+		    SetCurrentLine ((*t) -> text);
+		    TextCursorOn((*t) -> text);
+		    FreezeTextCursor((*t) -> text);
+		    break;
+		  }
+		}
+	      }
+	      (void) BrowserClearSelection();
+	      w_sel = ev -> xbutton.window;
+	      XSetSelectionOwner ( dpy, XA_PRIMARY, w_sel, ev -> xbutton.time);
+	      if ( w_sel != XGetSelectionOwner ( dpy, XA_PRIMARY ))
+		w_sel = None;
+	      break;
+	    }
 	  }
 	  break;
 	}
     }
 }
 
+
 /*
 **	Function name : ButtonUserFunc
 **
@@ -902,6 +1107,8 @@ void HandleExpose ()
 #endif
 	    return;
 	}
+	if ( ExposeToolBar ((*t) -> tool_bar, ev ) == True )
+	  return;
     }
     if ( ExposeBrowser ( ev ) == True )
       return;
@@ -979,3 +1186,28 @@ void UnFreezeButtons ()
 {
     lock_buttons = 0;
 }
+
+/*
+**	Function name : PasteSelection
+**
+**	Description :
+**	Input :
+**	Output :
+*/
+void PasteSelection(text)
+    Text *text;
+{
+  if ( w_sel != None ) { /* xcoral a la selection */
+    SetCurrentLine (text);
+    ClearSelection(text, True);
+    if ( strcmp(text -> current_mode -> name, "Shell") == 0)
+      SetPosInShell (text);
+    GetBytesFromCutBuffer (text );
+  }
+  else {
+    /* Allons chercher les donnees de l'autre client */
+    XConvertSelection (dpy, XA_PRIMARY, XA_STRING, prop_sel,
+		       ev -> xbutton.window, ev -> xbutton.time);
+  }
+}
+
