@@ -44,14 +44,19 @@
 #include <sys/types.h>
 #include <sys/time.h>
 #include <signal.h>
-#include <string.h>
 
-#ifndef apollo
-#ifdef __FreeBSD__
-#include <stdlib.h>
-#else /* !__FreeBSD__ */
-#include <malloc.h>
+#if HAVE_STRING_H
+#include <string.h>
+#else
+#include <strings.h>
 #endif
+
+#if STDC_HEADERS
+#  include <stdlib.h>
+#else
+#  if HAVE_MALLOC_H
+#    include <malloc.h>
+#  endif
 #endif
 
 #include "xcoral.h"
@@ -62,10 +67,12 @@
 #include "text_cursor.h"
 #include "page.h"
 #include "browser_init.h"
+#include "list_box.h"
+#include "selection.h"
+#include "shadow.h"
 
 static TextResources	tr;
 extern char *getenv ();
-extern void Display3D ();
 
 FCT (static int, GetNbLinesInPage, (Text *text, XFontStruct *font) );
 FCT (static void, Set_mode_internal, (Text *text, char *mode_name) );
@@ -165,7 +172,7 @@ Text *MakeTextWindow ( display, parent, x, y )
     
     XSelectInput ( display, text -> window,
 		  ExposureMask | KeyPressMask | VisibilityChangeMask |
-		  KeyReleaseMask | ButtonPress | ButtonRelease );
+		  KeyReleaseMask | ButtonPressMask | ButtonReleaseMask | PointerMotionMask);
     
     gcv.foreground = tr.fg;
     gcv.background = tr.bg;
@@ -209,6 +216,7 @@ Text *MakeTextWindow ( display, parent, x, y )
     for (i=0;i<256;i++) {
       text -> page.wline [i] = 0;
       text -> page.sline [i] = 0;
+      text -> page.hs_dl [i] = 0;
     }
     text -> page.hs_delta = 0;
     
@@ -227,7 +235,8 @@ Text *MakeTextWindow ( display, parent, x, y )
     text -> s_pos = 0;
     text -> current_ce = 0;
     text -> last_ce = 0;
-    
+    text -> selection = (Selection *) malloc((unsigned) sizeof(Selection) + 2);
+    text -> selection -> select = False;
     return text;
 }
 
@@ -254,6 +263,7 @@ void LoadMode ( text )
 	    && (strcmp("default",mode -> name) != 0)
 	    && (strcmp("C-mode",mode -> name) != 0)
 	    && (strcmp("C++mode",mode -> name) != 0)
+	    && (strcmp("Java",mode -> name) != 0)
 	    && (strcmp("Shell",mode -> name) != 0)
 	    && (strcmp("shell",mode -> name) != 0)
 	    && (strcmp("Latex",mode -> name) != 0)
@@ -307,6 +317,11 @@ void NewMode ( text )
     char *str;    
     char c = '\007'; /* ^G */
     
+    if (text -> selection -> select == True) {
+      SetCurrentLine (text);
+      ClearSelection(text, True);
+    }
+
     str = (char *) GetStringFromDB ( "New mode name : ", False );
     if ( (str == 0) || (strncmp(str, &c, 1)) == 0 ) {
 	DisplayMessage ( text -> mwin, "Abort" );
@@ -846,6 +861,32 @@ void SetCCMode ( text )
     }
 }
 
+/*
+**	Function name : SetJavaMode
+**
+**	Description : 
+**	Input : 
+**	Ouput :
+*/
+void SetJavaMode ( text )
+    Text *text; 
+{
+    if ( strcmp(text -> current_mode -> name, "Java") != 0 ) {
+	if ( text -> shell_id != 0 )
+	  KillShell ( text );
+	
+	text -> current_mode = (Mode *) GetMode ("Java");
+	text -> mwin -> mode = text -> current_mode;
+	RefreshWindowMode ( text -> mwin );
+	if ( (text -> current_mode -> font != text -> font) 
+	    && (text -> current_mode -> font != 0) )
+	  (void) RefreshWithNewFont ( text, text -> current_mode -> font );
+    }
+    if ( strcmp( "Java", (char *) GetBrowserModeName()) != 0 ) {
+	SetBrowserMode ( text -> current_mode );
+	RefreshBrowserControl ();
+    }
+}
 
 /*
 **	Function name : ChangeDir
@@ -858,7 +899,7 @@ void ChangeDir ( text )
     Text *text;
 {
     char *tmp;
-    char buf [128];
+    char buf [MAXPATHLEN];
     int len;
     
     (void) chdir ( text -> current_dir );
@@ -872,12 +913,10 @@ void ChangeDir ( text )
     if ( strcmp ( text -> filename + ( len - strlen ( "NoName" )), "NoName" ) == 0 ) {
 	len = strlen ( text -> current_dir );
 	if ( len > 20 ) {
-/*	    (void) sprintf ( buf, "Dir : ...%s\n", */
    	    (void) sprintf ( buf, "Dir : ...%s", 
 			    (char *) text -> current_dir + ( len - 20) );
 	}
 	else
-/*	  (void) sprintf ( buf, "Dir : %s\n", text -> current_dir ); */
 	  (void) sprintf ( buf, "Dir : %s", text -> current_dir );
 	DisplayMessage ( text -> mwin, buf  );
 	return;
@@ -928,12 +967,18 @@ void ExposeTextWindow ( dpy, text, ev )
     XSetRegion ( dpy, text -> Cgc, region );
     ExposePage ( region, text );
     XSetClipMask ( dpy, text -> Cgc, None );
-    XDestroyRegion ( region );
     SetCurrentLine ( text );
-    if ( text -> mouse_in == True ) 
+    if (text -> selection -> select == True) {
+      (void) ClearSelection (text, False);
+      (void) RefreshSelection(text);
+    }    
+    XDestroyRegion ( region );
+    if ( text -> mouse_in == True ) {
       TextCursorOn ( text );
+    }
     else
       FreezeTextCursor ( text ); 
+    
     return;
 }
 
