@@ -42,7 +42,12 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/cursorfont.h>
+
+#if HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
+#endif
 
 #include "xcoral.h"
 #include "main_text.h"
@@ -55,7 +60,7 @@
 #include "text_cursor.h"
 #include "shadow.h"
 #include "bm_search.h"
-
+
 extern Display *dpy;
 MBox man_box;
 static  Atom man_del;
@@ -668,7 +673,6 @@ int KeyPressInManual(ev)
     char buf[32];
     static KeySym ksym;
     static XComposeStatus compose;
-    InfosKey *infos;
     
     bzero( buf, 32 );
     
