@@ -39,14 +39,22 @@
 
 
 #include <stdio.h>
-#include <string.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
-#ifdef __FreeBSD__
-#include <stdlib.h>
-#else /* !__FreeBSD__ */
-#include <malloc.h>
+
+#if HAVE_STRING_H
+#include <string.h>
+#else
+#include <strings.h>
+#endif
+
+#if STDC_HEADERS
+#  include <stdlib.h>
+#else
+#  if HAVE_MALLOC_H
+#    include <malloc.h>
+#  endif
 #endif
 
 #include "xcoral.h"
@@ -60,8 +68,9 @@
 #include "input_str.h"
 #include "new_window.h"
 #include "parse.h"
-#include "config.h"
+#include "xc_conf.h"
 #include "options.h"
+#include "main_events.h"
 
 /* On va construire une liste chainee dont le premier
     element sera le mode par default */
@@ -93,7 +102,7 @@ FCT (static void, ie_WR_undo, (Text *text) );
 FCT (static void, ie_WR_redo, (Text *text) );
 FCT (static void, ie_WR_next_page, (Text *text) );
 FCT (static void, ie_WR_kill_region, (Text *text) );
-FCT (static void, ie_WR_paste_region, (Text *text) );
+/* FCT (static void, ie_WR_paste_region, (Text *text) ); */
 FCT (static void, ie_WR_eval_region, (Text *text) );
 FCT (static void, ie_WR_indent_region, (Text *text) );
 FCT (static void, ie_WR_current_line_to_top, (Text *text) );
@@ -125,6 +134,11 @@ FCT (static void, ie_WR_shell_return, (Text *text) );
 FCT (static void, ie_WR_shell_CtrC, (Text *text) );
 FCT (static void, ie_WR_shell_CtrD, (Text *text) );
 FCT (static void, ie_WR_shell_key, (Text *text) );
+FCT (static void, ie_WR_yank, (Text *text) );
+
+#if !HAVE_STRDUP
+extern FCT (char*, strdup, (char*) );
+#endif
 
 /*
 **	Function name : AllocMode
@@ -135,13 +149,7 @@ FCT (static void, ie_WR_shell_key, (Text *text) );
 */
 static Mode *AllocMode ()
 {
-    Mode *tmp;
-    
-    if (( tmp = (Mode *) malloc ((unsigned) sizeof (Mode))) == 0 )
-      return 0;
-    
-    bzero ( (char *) tmp, sizeof (Mode));
-    return (tmp);
+  return (Mode *) malloc ((unsigned) sizeof (Mode));
 }
 
 
@@ -154,56 +162,64 @@ static Mode *AllocMode ()
 */
 void InitMode ()
 {
-    /* Le mode par default */
-    if ( (m_default = AllocMode ()) == 0 ) {
-	(void) fprintf ( stderr, "Malloc error\n" );
-	exit (1);
-    }
-    bzero ( (char *) m_default, sizeof (Mode));
-    
-    (void) SetModeName ( m_default, "default" );
-    m_default -> suffixes = 0;
-    m_default -> font = 0;
-    (void) SetDefaultModeValues ( m_default );
-/*    (void) SetModeFont ( m_default, TEXT_FONT ); */
-    m_default -> font = GetOpFont ( OP_TEXT_FONT );
-    
-    /* Le mode C et C++ */
-    if ( CreateNewMode ( "C-mode" ) == 0 )
-      (void) fprintf ( stderr, "Create C mode error\n" );
-
-    if ( CreateNewMode ( "C++mode" ) == 0 )
-      (void) fprintf ( stderr, "Create C++ mode error\n" );
-    
-    if ( CreateNewMode ( "Latex" ) == 0 )
-      (void) fprintf ( stderr, "Create Latex mode error\n" );
-    
-    if ( CreateNewMode ( "Html" ) == 0 )
-      (void) fprintf ( stderr, "Create Html mode error\n" );
-    
-    if ( CreateNewMode ( "Ada" ) == 0 )
-      (void) fprintf ( stderr, "Create Ada mode error\n" );
-    
-    if ( CreateNewMode ( "Perl" ) == 0 )
-      (void) fprintf ( stderr, "Create Perl mode error\n" );
-    
-    if ( CreateNewMode ( "Fortran" ) == 0 )
-      (void) fprintf ( stderr, "Create Fortran mode error\n" );
-    
-    if ( CreateNewMode ( "shell" ) == 0 )
-      (void) fprintf ( stderr, "Create shell mode error\n" );
-
-     if ( CreateNewMode ( "Edir" ) == 0 )
-      (void) fprintf ( stderr, "Create Edir mode error\n" );
-
-   /* Le mode pour input-string */
-    if ( CreateNewMode ( "input_str" ) == 0 )
-      (void) fprintf ( stderr, "Create input_mode error\n" );
-
-    /* Le mode Shell */
-    if ( CreateNewMode ( "Shell" ) == 0 )
-      (void) fprintf ( stderr, "Create shell-mode error\n" );
-    InitShellMode ();
+  Mode * mode;
+  /* Le mode par default */
+  if ( (m_default = AllocMode ()) == 0 ) {
+    (void) fprintf ( stderr, "Malloc error\n" );
+    exit (1);
+  }
+  bzero ( (char *) m_default, sizeof (Mode));
+  
+  (void) SetModeName ( m_default, "default" );
+  m_default -> suffixes = 0;
+  m_default -> font = 0;
+  (void) SetDefaultModeValues ( m_default );
+  /*    (void) SetModeFont ( m_default, TEXT_FONT ); */
+  m_default -> font = GetOpFont ( OP_TEXT_FONT );
+  
+  /* Le mode C et C++ */
+  if ( (mode=CreateNewMode("C-mode"))== 0)
+    (void) fprintf ( stderr, "Create C mode error\n" );
+  else
+    mode->pp_path = strdup("/lib/cpp");
+  
+  if ( (mode=CreateNewMode("C++mode"))==0)
+    (void) fprintf ( stderr, "Create C++ mode error\n" );
+  else
+    mode->pp_path = strdup("/lib/cpp");
+  
+  if ( CreateNewMode ( "Java" ) == 0 )
+    (void) fprintf ( stderr, "Create Java mode error\n" );
+  
+  if ( CreateNewMode ( "Latex" ) == 0 )
+    (void) fprintf ( stderr, "Create Latex mode error\n" );
+  
+  if ( CreateNewMode ( "Html" ) == 0 )
+    (void) fprintf ( stderr, "Create Html mode error\n" );
+  
+  if ( CreateNewMode ( "Ada" ) == 0 )
+    (void) fprintf ( stderr, "Create Ada mode error\n" );
+  
+  if ( CreateNewMode ( "Perl" ) == 0 )
+    (void) fprintf ( stderr, "Create Perl mode error\n" );
+  
+  if ( CreateNewMode ( "Fortran" ) == 0 )
+    (void) fprintf ( stderr, "Create Fortran mode error\n" );
+  
+  if ( CreateNewMode ( "shell" ) == 0 )
+    (void) fprintf ( stderr, "Create shell mode error\n" );
+  
+  if ( CreateNewMode ( "Edir" ) == 0 )
+    (void) fprintf ( stderr, "Create Edir mode error\n" );
+  
+  /* Le mode pour input-string */
+  if ( CreateNewMode ( "input_str" ) == 0 )
+    (void) fprintf ( stderr, "Create input_mode error\n" );
+  
+  /* Le mode Shell */
+  if ( CreateNewMode ( "Shell" ) == 0 )
+    (void) fprintf ( stderr, "Create shell-mode error\n" );
+  InitShellMode ();
 }
 
 /*
@@ -435,7 +451,8 @@ static void ie_WR_open_space ( text ) Text *text; { C_OFF;OpenSpace (text);C_ON;
 static void ie_WR_undo ( text ) Text *text; { C_OFF;DoUndo (text);C_ON;}
 static void ie_WR_redo ( text ) Text *text; { C_OFF;DoUndoUndo (text);C_ON;}
 static void ie_WR_kill_region ( text ) Text *text; { C_OFF;KillRegion (text);C_ON;}
-static void ie_WR_paste_region ( text ) Text *text; { C_OFF;PasteRegion (text);C_ON;} 
+/* static void ie_WR_paste_region ( text ) Text *text; { C_OFF;PasteRegion (text);C_ON;} */
+static void ie_WR_yank ( text ) Text *text; { C_OFF;Yank (text);C_ON;}
 static void ie_WR_eval_region ( text ) Text *text; { C_OFF;EvalRegion (text);C_ON;} 
 static void ie_WR_indent_region ( text ) Text *text; { C_OFF;IndentRegion (text);C_ON;}
 static void ie_WR_current_line_to_top ( text ) Text *text; { C_OFF;CurrentLineToTop (text);C_ON;}
@@ -505,15 +522,11 @@ f2s func_string [NB_BUILTIN_FUNC] = {
     ie_WR_undo, "undo",
     ie_WR_redo, "redo",
     ie_WR_next_page, "next_page",
-    ie_WR_kill_region, "kill_region",
-    ie_WR_paste_region, "paste_region",
     ie_WR_eval_region, "eval_region",
-/*    ie_WR_indent_region, "c_indent_region", */
     ie_WR_current_line_to_top, "current_line_to_top",
     ie_WR_first_page, "first_page",
     ie_WR_last_page, "last_page",
     ie_WR_previous_page, "previous_page",
-    ie_WR_copy_region, "copy_region",
     ie_WR_delete_window, "delete_window", 
     ie_WR_exchange_cursor_mark, "exchange_cursor_mark",
     ie_WR_kill_current_buffer, "kill_current_buffer",
@@ -549,22 +562,14 @@ f2s func_string [NB_BUILTIN_FUNC] = {
 **	Input : 
 **	Ouput :
 */
-#ifdef __alpha
 long GetFuncNameFromString ( s )
-#else
-int GetFuncNameFromString ( s )
-#endif    
     char *s;
 {
     int i = 0;
 
     while ( func_string [i].f != 0 ) {
 	if ( strcmp ( (char *) func_string [i].s, s ) == 0 )
-#ifdef __alpha	  
 	  return ((long) func_string [i].f);
-#else	
-	  return ((int) func_string [i].f);
-#endif	  
 	i++;
     }
     return (0);
@@ -606,7 +611,6 @@ static void SetDefaultModeValues ( mode )
 {
     extern void StartMacro (), EndMacro (), PlayMacro (), QuotedChar ();
     extern void DisplayLineNumber ();
-    extern FCT(int, Xk_Code,(char *));
 
     /* Les touches, Controle + touche et Delete, XK */
 
@@ -634,7 +638,7 @@ static void SetDefaultModeValues ( mode )
     mode -> key [CtrU].func = ie_WR_undo;
     mode -> key [CtrV].func = ie_WR_next_page;
     mode -> key [CtrW].func = ie_WR_kill_region;
-    mode -> key [CtrY].func = ie_WR_paste_region;
+    mode -> key [CtrY].func =  ie_WR_yank;
     mode -> key [DELETE].func = ie_WR_delete;
 
     mode -> key [Ctr_sp].type = BUILTIN_FUNC; 
@@ -665,97 +669,97 @@ static void SetDefaultModeValues ( mode )
     mode -> key [DELETE].type = BUILTIN_FUNC;
 
     {
-      int code = Xk_Code("Right");
+      unsigned code = Xk_Code((unsigned char *)"Right");
       
       mode -> key [code].type = BUILTIN_FUNC;
       mode -> key [code].func = ie_WR_goto_next_char;
     }
     {
-      int code = Xk_Code("R12");
+      unsigned code = Xk_Code((unsigned char *)"R12");
       
       mode -> key [code].type = BUILTIN_FUNC;
       mode -> key [code].func = ie_WR_goto_next_char;
     }    
     {
-      int code = Xk_Code("Left");
+      unsigned code = Xk_Code((unsigned char *)"Left");
       
       mode -> key [code].type = BUILTIN_FUNC;
       mode -> key [code].func = ie_WR_goto_previous_char;
     }
     {
-      int code = Xk_Code("R10");
+      unsigned code = Xk_Code((unsigned char *)"R10");
       
       mode -> key [code].type = BUILTIN_FUNC;
       mode -> key [code].func = ie_WR_goto_previous_char;
     }    
     {
-      int code = Xk_Code("Up");
+      unsigned code = Xk_Code((unsigned char *)"Up");
       
       mode -> key [code].type = BUILTIN_FUNC;
       mode -> key [code].func = ie_WR_cursor_up;
     }
     {
-      int code = Xk_Code("R8");
+      unsigned code = Xk_Code((unsigned char *)"R8");
       
       mode -> key [code].type = BUILTIN_FUNC;
       mode -> key [code].func = ie_WR_cursor_up;
     }    
     {
-      int code = Xk_Code("Down");
+      unsigned code = Xk_Code((unsigned char *)"Down");
       
       mode -> key [code].type = BUILTIN_FUNC;
       mode -> key [code].func = ie_WR_cursor_down;
     }
     {
-      int code = Xk_Code("R14");
+      unsigned code = Xk_Code((unsigned char *)"R14");
       
       mode -> key [code].type = BUILTIN_FUNC;
       mode -> key [code].func = ie_WR_cursor_down;
     }    
     {
-      int code = Xk_Code("Home");
+      unsigned code = Xk_Code((unsigned char *)"Home");
       
       mode -> key [code].type = BUILTIN_FUNC;
       mode -> key [code].func = ie_WR_first_page;
     }
     {
-      int code = Xk_Code("R7");
+      unsigned code = Xk_Code((unsigned char *)"R7");
       
       mode -> key [code].type = BUILTIN_FUNC;
       mode -> key [code].func = ie_WR_first_page;
     }
     {
-      int code = Xk_Code("Next");
+      unsigned code = Xk_Code((unsigned char *)"Next");
       
       mode -> key [code].type = BUILTIN_FUNC;
       mode -> key [code].func = ie_WR_next_page;
     }
     {
-      int code = Xk_Code("R15");
+      unsigned code = Xk_Code((unsigned char *)"R15");
       
       mode -> key [code].type = BUILTIN_FUNC;
       mode -> key [code].func = ie_WR_next_page;
     }    
     {
-      int code = Xk_Code("Prior");
+      unsigned code = Xk_Code((unsigned char *)"Prior");
       
       mode -> key [code].type = BUILTIN_FUNC;
       mode -> key [code].func = ie_WR_previous_page;
     }
     {
-      int code = Xk_Code("R9");
+      unsigned code = Xk_Code((unsigned char *)"R9");
       
       mode -> key [code].type = BUILTIN_FUNC;
       mode -> key [code].func = ie_WR_previous_page;
     }    
     {
-      int code = Xk_Code("End");
+      unsigned code = Xk_Code((unsigned char *)"End");
       
       mode -> key [code].type = BUILTIN_FUNC;
       mode -> key [code].func = ie_WR_last_page;
     }
     {
-      int code = Xk_Code("R13");
+      unsigned code = Xk_Code((unsigned char *)"R13");
       
       mode -> key [code].type = BUILTIN_FUNC;
       mode -> key [code].func = ie_WR_last_page;
@@ -794,6 +798,9 @@ static void SetDefaultModeValues ( mode )
     mode -> ctrX_key [CtrW].func = ie_WR_write_file;
     mode -> ctrX_key [CtrX].func = ie_WR_exchange_cursor_mark;
     
+    mode -> ctrX_key [(int) 'd'].func = KbdBrowserDump;
+    mode -> ctrX_key [(int) 'r'].func = KbdBrowserRestore;
+
     mode -> ctrX_key [(int) 'b'].func = ie_WR_list_open_file;
     mode -> ctrX_key [(int) 'k'].func = ie_WR_kill_current_buffer;
     mode -> ctrX_key [(int) 'l'].func = ie_WR_goto_line;
@@ -811,6 +818,9 @@ static void SetDefaultModeValues ( mode )
     mode -> ctrX_key [CtrW].type = BUILTIN_FUNC; 
     mode -> ctrX_key [CtrX].type = BUILTIN_FUNC; 
     
+    mode -> ctrX_key [(int) 'd'].type = BUILTIN_FUNC; 
+    mode -> ctrX_key [(int) 'r'].type = BUILTIN_FUNC; 
+
     mode -> ctrX_key [(int) 'b'].type = BUILTIN_FUNC; 
     mode -> ctrX_key [(int) 'k'].type = BUILTIN_FUNC; 
     mode -> ctrX_key [(int) 'l'].type = BUILTIN_FUNC; 
