@@ -48,7 +48,12 @@
 #include <stdio.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
+
+#if HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
+#endif
 
 #include "options.h"
 #include "xcoral.h"
@@ -62,7 +67,7 @@
 #include "dial_box.h"
 #include "get_file.h"
 #include "macros.h"
-#include "logo.bm"
+#include "buttons_bits.h"
 
 extern Trans st_initial;
 
@@ -83,8 +88,8 @@ void InitIconPixmap()
     text_icon = XCreatePixmapFromBitmapData ( dpy, DefaultRootWindow ( dpy ),
 		(char *) icon_bits,
 		icon_width, icon_height,
-		WhitePixel ( dpy, DefaultScreen ( dpy )),
 		BlackPixel ( dpy, DefaultScreen ( dpy )),
+		WhitePixel ( dpy, DefaultScreen ( dpy )),
 		DefaultDepth ( dpy, DefaultScreen ( dpy )));
 }
 
@@ -106,6 +111,7 @@ EdWin *CreateWindow ()
     MWin *MakeControlPanel ();
     static Atom wm_delete_window;
     int win_id;
+    int htb = 0;
     
     screen = DefaultScreen ( dpy );
     root = RootWindow ( dpy, screen );
@@ -131,17 +137,22 @@ EdWin *CreateWindow ()
     
     edtmp -> menu = MakeMenus ( dpy, edtmp->w_frame, 
 			       menu_names, item_names, func_names );
-    
+
+    edtmp -> tool_bar = CreateToolBar(edtmp->w_frame,W_SPACE,
+				      HeightMenuBar ( edtmp -> menu -> font ) + 1);
+    htb = TOOL_WINDOW_SIZE;
+
     edtmp -> mwin = MakeControlPanel ( edtmp->w_frame );
     
     edtmp -> scroll = MakeScroll ( dpy, edtmp->w_frame,
 				  GetOpGeo ( OP_WIDTH ) - GetScrollWidth () - W_SPACE,
-				  HeightMenuBar ( edtmp -> menu -> font ) + 1 + W_SPACE );
+				  HeightMenuBar ( edtmp -> menu -> font ) + 1 + W_SPACE + htb);
     
     SetScrollFont ( edtmp -> scroll, GetOpFont ( OP_TEXT_FONT) );
     
     edtmp -> text = MakeTextWindow ( dpy, edtmp -> w_frame, 
-				    W_SPACE, HeightMenuBar ( edtmp -> menu -> font ) + 1 + W_SPACE );
+				    W_SPACE, 
+				    HeightMenuBar ( edtmp -> menu -> font ) + 1 + W_SPACE + htb);
     edtmp -> text -> win_id = win_id;
     
     edtmp -> buf = GetBuffer ( (unsigned) SIZEOF_BUFFER );
@@ -178,12 +189,15 @@ void ConfigWindow ( e, width, height )
     int width, height;
 {
     int hbar, hmess, wscroll, i, x;
-    
+    int htb = 0;
+   
     e -> width = width;
     e -> height = height;
     hbar = HeightMenuBar ( e -> menu -> font );
     hmess = HeightOfMess ();
     wscroll = GetScrollWidth ();
+
+    htb = TOOL_WINDOW_SIZE;
     
 #ifdef DEBUG
     fprintf ( stderr, "ConfigWindow width = %d, height = %d\n",
@@ -192,7 +206,7 @@ void ConfigWindow ( e, width, height )
 	     hbar, hmess, wscroll );
 #endif
     
-    x = height - ( hbar + hmess + 2 ) - ( 2 * W_SPACE );
+    x = height - ( hbar + hmess + 2 ) - ( 2 * W_SPACE ) - htb;
     x -= ( 2 * e -> text -> y_or );
     i = x / e -> text -> font_height;
     
@@ -200,7 +214,7 @@ void ConfigWindow ( e, width, height )
     SetMenuBarWidth ( e -> menu, width );
     ShowControlPanel ( e -> mwin, width, height );
     ShowWindowText ( dpy, e -> text, width - wscroll - 1 - ( 2 * W_SPACE ) ,
-		    height - hbar - hmess - 2 - ( 2 * W_SPACE) );
+		    height - hbar - hmess - 2 - ( 2 * W_SPACE) - htb);
     
     if ( DoesSaveUnders ( DefaultScreenOfDisplay ( dpy ) ) != True )
       SetMenuPixmap ( dpy, e -> menu, width - wscroll - 1 );
@@ -211,7 +225,8 @@ void ConfigWindow ( e, width, height )
 	     e -> text -> no_current_line, e -> text -> n1 );
 #endif
     RefreshScroll ( dpy, e -> scroll, width - W_SPACE,
-		   height - hbar - hmess - 2 - ( 2 * W_SPACE) , i );
+		   height - hbar - hmess - 2 - ( 2 * W_SPACE) - htb, i );
+    DisplayToolBar (e -> tool_bar, width);
 }
 
 
@@ -243,8 +258,8 @@ static void SetProp ( w )
 #ifdef DEBUG
     (void) fprintf ( stderr, "x = %d	y = %d\n", sizehints.x, sizehints.y );
 #endif
-    sizehints.min_width = (DisplayWidth ( dpy,DefaultScreen ( dpy )) *2 ) / 5; 
-    sizehints.min_height = (DisplayHeight ( dpy,DefaultScreen ( dpy )) * 2 ) / 5; 
+    sizehints.min_width = (DisplayWidth ( dpy,DefaultScreen ( dpy )) *2 ) / 6; 
+    sizehints.min_height = (DisplayHeight ( dpy,DefaultScreen ( dpy )) * 2 ) / 7; 
     sizehints.max_width = DisplayWidth (dpy,DefaultScreen (dpy));
     sizehints.max_height = DisplayHeight (dpy,DefaultScreen (dpy));
     
@@ -425,25 +440,15 @@ int DeleteWindow ( text )
 {
     EdWin	**t, *tmp;
     extern Window window_kill;
+    extern Window w_sel;
     Window the_frame;
     
     if ( GetModif ( text ) == True ) {
 	if ( SaveCurrentBuffer ( text, F_MENU ) != True )
 	  return -1;
     }
-    if ( IsLastWindow ( 1 ) == True ) {
-/*
-   s = ( char * ) GetStringFromDB ( "Last window. Quit [y/n] : ", False );
-	if ( (s == 0) || (strncmp(s,"y",1) == 0) ) {
-	  exit(0);
-	}
-	else if ( (strncmp(s,"n",1 ) == 0) || (strncmp(s,&c,1)==0) ) {
-	    DisplayMessage ( text -> mwin, "Abort" );
-	    return -1;
-	}
-*/
+    if ( IsLastWindow ( 1 ) == True )
       exit(0);
-    }
     
     for ( t = TWin; t < TWin + MAXWIN; t++ ) {
 	if ( *t == 0 ) continue;
@@ -455,6 +460,12 @@ int DeleteWindow ( text )
     if ( tmp -> text -> shell_id != 0 )
       KillShell ( tmp -> text );
 
+    if ((tmp -> text -> selection -> select == True)
+	&&(tmp -> text -> window == w_sel)) {
+      /* Ya une selection dans la fenetre a tuee */
+      w_sel = None;
+    }
+    
     the_frame = tmp -> w_frame;
     XUnmapWindow ( dpy, tmp -> w_frame ); 
     DeleteMenu ( dpy, tmp -> menu );
@@ -465,8 +476,7 @@ int DeleteWindow ( text )
     DeleteControlPanel ( tmp -> mwin );
     DeleteScroll ( dpy, tmp -> scroll );
     DeleteMacro ( tmp -> text );
-/*    XSync ( dpy, False ); */
-    
+
     for ( t = TWin; t < TWin + MAXWIN; t++ ) {
 	if ( *t == 0 ) continue;
 	if ( *t == tmp ) { 
@@ -541,6 +551,33 @@ void Help ( text )
     DisplayManBox ();
 }
 
+/*
+**	Function name : Hanoi
+**
+**	Description : 
+**	Input : 
+**	Ouput :
+*/
+void Hanoi ( text )
+    Text *text;
+{
+  char *msg;
+  int nw = text->win_id;
+  int tn = 5; /* Nombre de tours */
+
+  StorePosition ( text );
+  msg = (char *) ie_call_function ( text, "hanoi", 1, &tn) ;
+  text = update_cwd(nw);
+  if ( msg != 0 )
+    DisplayMessage ( text -> mwin, msg );
+  ie_redisplay ( text );
+  if(TextInBuf(text)==False){
+     TextCursorOff(text);
+     ClearPage(text);
+     TextCursorOn(text);
+  }  
+}
+
 
 
 
