@@ -65,18 +65,23 @@
 #include <X11/cursorfont.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
-#include <string.h>
 #include <pwd.h>
 
-#ifndef apollo
-#ifdef __FreeBSD__
-#include <stdlib.h>
-#else /* !__FreeBSD__ */
-#include <malloc.h>
-#endif
+#if HAVE_STRING_H
+#include <string.h>
+#else
+#include <strings.h>
 #endif
 
-#include "config.h"
+#if STDC_HEADERS
+#  include <stdlib.h>
+#else
+#  if HAVE_MALLOC_H
+#    include <malloc.h>
+#  endif
+#endif
+
+#include "xc_conf.h"
 #include "options.h"
 #include "main_text.h"
 #include "parse.h"
@@ -233,9 +238,9 @@ void GetUserDatabase ()
 {
     XrmDatabase homeDB, serverDB, applicationDB;
     
-    char 	filename [128];	
+    char 	filename [MAXPATHLEN];	
     char 	*environment;
-    char 	name [256];
+    char 	name [MAXPATHLEN];
     char *str;
     
     (void)  sprintf ( name, "%s/%s", APP_DEFAULT, "Xcoral" );
@@ -353,8 +358,12 @@ static void SetDefaultBG ( bg, fg, bwbg, bwfg, colorbg, colorfg)
     }
     else { /* color */
 	*bg = PixelValue ( dpy, colorbg, &result );
+	if ( result == False )
+	  *bg = BlackPixel ( dpy, DefaultScreen (dpy));
 	if ( *bg == *fg ) {
 	    *bg = PixelValue ( dpy, colorfg, &result );
+	    if ( result == False )
+	      *bg = WhitePixel ( dpy, DefaultScreen (dpy));
 	}
     }
 }
@@ -386,15 +395,17 @@ static void SetDialogFG ()
 	    if ( (strcmp ( buf, "black" ) != 0 ) && (strcmp ( buf, "white" ) != 0 )) {
 		options.cfg = PixelValue ( dpy, BW_CONTROL_FG, &result );
 	    }
-	    else {
+	    else 
 		options.cfg = PixelValue ( dpy, buf, &result );
-	    }
+	    if ( result == False )
+	      options.cfg = BlackPixel ( dpy, DefaultScreen (dpy));		
 	}
 	else { /* Color Display */
 	    options.cfg = PixelValue ( dpy, buf, &result );
-	    if ( result == False ) {
+	    if ( result == False )
 		options.cfg = PixelValue ( dpy, COLOR_CONTROL_FG, &result );
-	    }
+	    if ( result == False )
+		options.cfg = BlackPixel ( dpy, DefaultScreen (dpy));
 	}
 	if ( buf != 0 )
 	  (void) free ( buf );
@@ -402,6 +413,8 @@ static void SetDialogFG ()
     else  {
 	options.cfg = IsColor ? PixelValue ( dpy, COLOR_CONTROL_FG, &result ) :
 	  PixelValue ( dpy, BW_CONTROL_FG, &result );
+	if ( result == False )
+	  options.cfg = BlackPixel ( dpy, DefaultScreen (dpy));
     }
 }
 
@@ -432,6 +445,8 @@ static void SetDialogBG ()
 	      options.cbg = PixelValue ( dpy, BW_CONTROL_BG, &result );
 	    else  
 	      options.cbg = PixelValue ( dpy, buf, &result );
+	    if(result == False)
+	      options.cbg = WhitePixel ( dpy, DefaultScreen (dpy));
 	    if ( options.cbg == options.cfg ) {
 		if ( XrmGetResource ( savecommandlineDB, "xcoral.cbg",
 				     "xcoral.cbackground", str_type,&value ) == True ) {
@@ -452,9 +467,18 @@ static void SetDialogBG ()
 	    options.cbg = PixelValue ( dpy, buf, &result );
 	    if ( result == False )
 	      options.cbg = PixelValue ( dpy, COLOR_CONTROL_BG, &result );
-	    if ( options.cbg == options.cfg ) {
-		options.cbg = PixelValue ( dpy, COLOR_CONTROL_BG, &result );
+	    if(result == False) {
+	       options.cbg = WhitePixel ( dpy, DefaultScreen (dpy));
+	       options.cfg = BlackPixel ( dpy, DefaultScreen (dpy));
+	    }
+	    else {
+	      if ( options.cbg == options.cfg ) {
 		options.cfg = PixelValue ( dpy, COLOR_CONTROL_FG, &result );
+		if(result == False) {
+		  options.cbg = BlackPixel ( dpy, DefaultScreen (dpy));
+		  options.cfg = WhitePixel ( dpy, DefaultScreen (dpy));
+		}
+	      }   
 	    }
 	}
 	if ( buf != 0 )
@@ -491,18 +515,25 @@ static void SetMenuFG ()
 	      options.mfg = PixelValue ( dpy, BW_MENU_FG, &result );
 	    else 
 	      options.mfg = PixelValue ( dpy, buf, &result );
+	    if (result == False)
+	      options.mfg = BlackPixel ( dpy, DefaultScreen (dpy));
 	}
 	else { /* Color Display */
 	    options.mfg = PixelValue ( dpy, buf, &result );
 	    if ( result == False ) 
 	      options.mfg = PixelValue ( dpy, COLOR_MENU_FG, &result );
+	    if (result == False)
+	      options.mfg = BlackPixel ( dpy, DefaultScreen (dpy));
 	}
 	if ( buf != 0 )
 	  (void) free ( buf );
     }
-    else 
+    else {
       options.mfg = IsColor ? PixelValue ( dpy, COLOR_MENU_FG, &result ) :
 	PixelValue ( dpy, BW_MENU_FG, &result );
+      if (result == False)
+	options.mfg = BlackPixel ( dpy, DefaultScreen (dpy));
+    }
 }
 
 /*
@@ -530,6 +561,9 @@ static void SetMenuBG ()
 	      options.mbg = PixelValue ( dpy, BW_MENU_BG, &result );
 	    else  
 	      options.mbg = PixelValue ( dpy, buf, &result );
+	    if (result == False)
+	      options.mbg = WhitePixel ( dpy, DefaultScreen (dpy));
+	    
 	    if ( options.mbg == options.mfg ) {
 		if ( XrmGetResource ( savecommandlineDB, "xcoral.mbg",
 				     "xcoral.mbackground", str_type,&value ) == True ) {
@@ -550,9 +584,18 @@ static void SetMenuBG ()
 	    options.mbg = PixelValue ( dpy, buf, &result );
 	    if ( result == False )
 	      options.mbg = PixelValue ( dpy, COLOR_MENU_BG, &result );
-	    if ( options.mbg == options.mfg ) {
-		options.mbg = PixelValue ( dpy, COLOR_MENU_BG, &result );
+	    if (result == False) {
+	      options.mbg = BlackPixel ( dpy, DefaultScreen (dpy));
+	      options.mfg = WhitePixel ( dpy, DefaultScreen (dpy));
+	    }
+	    else {
+	      if ( options.mbg == options.mfg ) {
 		options.mfg = PixelValue ( dpy, COLOR_MENU_FG, &result );
+		if (result == False) {
+		  options.mbg = WhitePixel ( dpy, DefaultScreen (dpy));
+		  options.mfg = BlackPixel ( dpy, DefaultScreen (dpy));
+		}
+	      }
 	    }
 	}
 	if ( buf != 0 )
@@ -589,11 +632,15 @@ static void SetTextFG ()
 	      options.fg = PixelValue ( dpy, BW_TEXT_FG, &result );
 	    else 
 	      options.fg = PixelValue ( dpy, buf, &result );
+	    if (result == False)
+	      options.fg = WhitePixel ( dpy, DefaultScreen (dpy));
 	}
 	else { /* Color Display */
 	    options.fg = PixelValue ( dpy, buf, &result );
 	    if ( result == False ) {
 		options.fg = PixelValue ( dpy, COLOR_TEXT_FG, &result );
+		if (result == False)
+		  options.fg = WhitePixel ( dpy, DefaultScreen (dpy));
 	    }
 	}
 	if ( buf != 0 )
@@ -602,6 +649,8 @@ static void SetTextFG ()
     else {
 	options.fg = IsColor ? PixelValue ( dpy, COLOR_TEXT_FG, &result ) :
 	  PixelValue ( dpy, BW_TEXT_FG, &result );
+	if (result == False)
+	  options.fg = WhitePixel ( dpy, DefaultScreen (dpy));
     }
 }
 
@@ -655,6 +704,9 @@ static void SetTextBG ()
 	      options.bg = PixelValue ( dpy, BW_TEXT_BG, &result );
 	    else  
 	      options.bg = PixelValue ( dpy, buf, &result );
+	    if (result == False)
+	      options.bg = BlackPixel ( dpy, DefaultScreen (dpy));
+	    
 	    if ( options.bg == options.fg ) {
 		if ( XrmGetResource ( savecommandlineDB, "xcoral.bg",
 				     "xcoral.background", str_type,&value ) == True ) {
@@ -676,9 +728,16 @@ static void SetTextBG ()
 	    options.bg = PixelValue ( dpy, buf, &result );
 	    if ( result == False )
 	      options.bg = PixelValue ( dpy, COLOR_TEXT_BG, &result );
+	    if (result == False) {
+	      options.bg = BlackPixel ( dpy, DefaultScreen (dpy));
+	      options.fg = WhitePixel ( dpy, DefaultScreen (dpy));
+	    }
 	    if ( options.bg == options.fg ) {
-		options.bg = PixelValue ( dpy, COLOR_TEXT_BG, &result );
 		options.fg = PixelValue ( dpy, COLOR_TEXT_FG, &result );
+		if (result == False) {
+		  options.bg = WhitePixel ( dpy, DefaultScreen (dpy));
+		  options.fg = BlackPixel ( dpy, DefaultScreen (dpy));
+		}
 	    }
 	}
 	if ( buf != 0 )
@@ -757,7 +816,7 @@ static void SetFont ()
 **	Function name : SetGeometry
 **
 **	Description : Initialisation de la geometrie de 
-**		la feneter printcipale.
+**		la fenetre principale.
 **	Input : 
 **	Ouput :
 */
@@ -967,9 +1026,8 @@ unsigned long PixelValue ( display, str, result )
     else {
 	if ( IsColor == True ) {
 	    (void) fprintf ( stderr, "Warning : can't alloc '%s' color.\n", str );
-	    color.pixel = (unsigned long) BlackPixel(display,DefaultScreen ( display ));
-	    *result = True;
-/*	    (void) exit (1); */
+/*  color.pixel = (unsigned long) BlackPixel(display,DefaultScreen ( display )); */
+	    *result = False;
 	}
 	else
 	  *result = False;
