@@ -37,13 +37,17 @@
 
    ######################################################################## */
 
+#if HAVE_STRING_H
+#include <string.h>
+#else
+#include <strings.h>
+#endif
+#include <stdio.h>
 
 #include "result_types.h"
 #include "file_dict.h"
 #include "proc_dict.h"
 #include "browser_util.h"
-#include <string.h>
-#include <stdio.h>
 
 /*------------------------------------------------------------------------------
 //                         Le dictionnaire des procedures
@@ -57,15 +61,22 @@ int proc_count = 0;
 
 /*------------------------------------------------------------------------------
 */
-ProcRec* create_proc(proc_name)
-  char* proc_name;
+ProcRec* create_proc(proc_name, staticp, parsed_file)
+    char* proc_name;
+    int staticp;
+    char * parsed_file;
 {
   ProcRec**  head;
   ProcRec*   current_proc;
   int        x_size;
   
   get_head_Rec(proc_name, proc_dict, PROC_DICT_SIZE, head);
-  search_Rec(proc_name, ProcRec, head, current_proc);
+  iter_Rec(proc_name, ProcRec, head, current_proc) {
+    if (current_proc->_impl_file &&
+	((! strcmp(parsed_file, current_proc->_impl_file->_name)) ||
+	 ((! staticp) && (! (current_proc->_decl & STATIC_PROC)))))
+      return current_proc;
+  }
   if (current_proc == Null) {
     x_size = sizeof(ProcRec) + PROC_PLENGTH + strlen(proc_name)  + 1;
     current_proc = (ProcRec*) xmalloc(x_size);
@@ -73,7 +84,8 @@ ProcRec* create_proc(proc_name)
       create_Rec(proc_name, ProcRec, head, current_proc, PROC_PREFIX, PROC_PLENGTH);
       current_proc->_impl_file = Null;
       current_proc->_impl_line = 0;
-      current_proc->_proc_type = UNKNOWN_PROC;
+      current_proc->_decl = 0;
+      current_proc->_hide = 0;
       proc_count++;
     }
   }
@@ -90,7 +102,7 @@ ProcRec* find_proc(proc_name)
   ProcRec*   current_proc;
 
   get_head_Rec(proc_name, proc_dict, PROC_DICT_SIZE, head);
-  search_Rec(proc_name, ProcRec, head, current_proc);
+  assoc_Rec(proc_name, ProcRec, head, current_proc);
   return(current_proc);
 }
 
