@@ -41,23 +41,27 @@
 #include <stdio.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
-#if defined(SVR4) || defined(UNIXWARE)
+
+#if HAVE_STRING_H
 #include <string.h>
 #else
 #include <strings.h>
 #endif
-#ifdef __FreeBSD__
-#include <stdlib.h>
-#else /* !__FreeBSD__ */
-#include <malloc.h>
+
+#if STDC_HEADERS
+#  include <stdlib.h>
+#else
+#  if HAVE_MALLOC_H
+#    include <malloc.h>
+#  endif
 #endif
 
 #include "main_text.h"
 #include "text_cursor.h"
 #include "page.h"
+#include "selection.h"
 
 FCT (static void, GetCurrentXY, (Text *text, int *x, int *y, char *c) );
-FCT (static int, MyTextWidth, (Text *text,char *str, int i) );
 
 extern Display *dpy;
 
@@ -99,7 +103,7 @@ void TextCursorOn ( text )
       return;
     
     if ( text -> mouse_in == True ) {
-	if ( text -> cursor_stat == FREESE )
+	if ( text -> cursor_stat == FREEZE )
 	  UnFreezeTextCursor ( text );
 #ifdef DEBUG
 	(void) fprintf ( stderr, "Cursor On\n" );
@@ -150,7 +154,7 @@ void FreezeTextCursor ( text )
     XSetForeground (dpy, text -> Igc, text -> fg ^ text -> bg );
     XSetPlaneMask (dpy, text -> Igc, text -> fg ^ text -> bg );
     
-    text -> cursor_stat = FREESE;
+    text -> cursor_stat = FREEZE;
 }
 
 
@@ -170,7 +174,7 @@ void UnFreezeTextCursor ( text )
     (void) fprintf ( stderr, "UnFreezeCursor x = %d y = %d\n",
 		    text -> x_pos, text -> y_pos);
 #endif
-    if ( text -> cursor_stat != FREESE )
+    if ( text -> cursor_stat != FREEZE )
       return;
     
     c_color = GetCursorColor(text);
@@ -204,7 +208,7 @@ void TextCursorOff ( text )
     if ( text -> cursor_stat == OFF )
       return;
     
-    if ( text -> cursor_stat == FREESE ) {
+    if ( text -> cursor_stat == FREEZE ) {
 	UnFreezeTextCursor ( text );
 	return;
     }
@@ -303,6 +307,11 @@ void MoveToBline ( text )
     Text *text;
 {
     int n;
+
+    if(text->selection->select == True){
+      ClearSelection(text,True);
+      TextCursorOff(text);      
+    }
     
     if ( text -> sl )
       GotoLeft ( text );
@@ -325,6 +334,11 @@ void MoveToEline ( text )
 {
     int n;
     
+    if(text->selection->select == True) {
+      ClearSelection(text,True);
+      TextCursorOff(text);
+    }
+    
     n = GetNcFromRight ( text -> buf );
     (void) MoveHole ( text -> buf, n );
 }
@@ -350,6 +364,11 @@ void UpCursor ( text )
 	return;
     }
     
+    if(text->selection->select == True){
+      ClearSelection(text,True);
+      TextCursorOff(text);
+    }
+    
     if ( text -> n1 == 0 )                /* il faut scroller */
       CurrentLineToMiddle ( text );
     
@@ -379,6 +398,11 @@ void DownCursor ( text )
 	return;
     }
     
+    if(text->selection->select == True){
+      ClearSelection(text,True);
+      TextCursorOff(text);
+    }
+    
     if ( text -> n2 == 0 ) {
 	CurrentLineToMiddle ( text );
 	y = text -> y_or + (( text -> n1 + 1 ) * text -> font_height);
@@ -412,6 +436,11 @@ int ForwardChar ( text )
 	klaxon ();
 	return False;
     }
+    if(text->selection->select == True){
+      ClearSelection(text,True);
+      TextCursorOff(text);
+    }
+    
     if ( c == '\n' ) {
 	if ( text -> no_current_line > text -> lines_in_buf )
 	  return False;
@@ -446,10 +475,16 @@ void BackwardChar ( text )
 #ifdef DEBUG
     (void) fprintf ( stderr, "Backward char\n" );
 #endif
+    if(text->selection->select == True){
+      ClearSelection(text,True);
+      TextCursorOff(text);
+    }
+
     if ( MoveHole ( text -> buf, -1 ) < 0 ) {
 	klaxon ();
 	return;
     }
+    
     (void) GetCurrentChar ( text -> buf, &c );
     if ( c == '\n' ) {
 	text -> no_current_line --;
@@ -572,7 +607,7 @@ static void GetCurrentXY ( text, x, y, c )
 **	Input : Le text courant, la chaine, nb caracteres.
 **	Ouput : La longueur en pixel.
 */
-static int MyTextWidth ( text, str, i )
+int MyTextWidth ( text, str, i )
     Text *text;
     char *str;
     int i;
