@@ -41,12 +41,20 @@
 #include <stdio.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
-#ifdef __FreeBSD__
-#include <stdlib.h>
-#else /* !__FreeBSD__ */
-#include <malloc.h>
+
+#if STDC_HEADERS
+#  include <stdlib.h>
+#else
+#  if HAVE_MALLOC_H
+#    include <malloc.h>
+#  endif
 #endif
+
+#if HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
+#endif
 
 #include "main_text.h"
 #include "page.h"
@@ -54,6 +62,7 @@
 #include "mark_reg.h"
 #include "bm_search.h"
 #include "chars_cmds.h"
+#include "selection.h"
 
 FCT (static Undo *, AllocUndoBuf, () );
 FCT (static void, CompactUndo, (Text *text) );
@@ -581,6 +590,9 @@ void DoUndo ( text )
     Mode *save_mode = text -> current_mode;
     Undo *tmp = GetCurrentUndo ( text );
     
+    if (text -> selection -> select == True)
+      ClearSelection(text, True);
+    
     text -> current_mode = (Mode *) GetMode ("default");
 #ifdef U_TRACE
     (void) fprintf ( stderr, "DoUndo\n" );
@@ -987,7 +999,7 @@ void DoUndoUndo ( text )
 #ifdef U_TRACE
     (void) fprintf ( stderr, "DoUndoUndo\n" );
 #endif /* U_TRACE */
-    
+
     if ( text -> u_index == 0 ) {
 #ifdef U_TRACE
 	(void) fprintf ( stderr, "DoUndoUndo : index (null)\n" );
@@ -996,6 +1008,7 @@ void DoUndoUndo ( text )
 	klaxon ();
 	return;
     }
+
     text -> u_index --;
     flag_redo = True;
     DoUndo ( text );
