@@ -43,6 +43,7 @@
 #include "main_text.h"
 #include "handle_key.h"
 #include "init_menus.h"
+#include "tool_bar.h"
 
 /*
  *	La structure associe a chaque fenetre d'edition.
@@ -57,6 +58,7 @@ typedef struct {
 	SWin		*scroll;	/* La fenetre de scroll */	
 	XYMenu		*menu;		/* Les menus XY */
 	XSizeHints	*shints;	/* Les options */
+	ToolBar         *tool_bar;      /* la tool bar */
 } EdWin;
 
 extern Display	*dpy;		/* Le display */
@@ -69,6 +71,6 @@ extern char 	**item_names [];
 extern void		(**func_names []) ();	/* Callback */
 
 #define	MAXWIN	32	/* Nb max de fenetres d'editions. 32 ou 64 si on veut */
-#define CURRENT_VERSION	"xcoral version 2.5 "
+#define CURRENT_VERSION	"xcoral version 3.2 "
 
 #endif /* _XCORAL_H_ */
